@@ -30,11 +30,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <boost/python.hpp>
 #include <libtorrent/error_code.hpp>
-#include <libtorrent/lazy_entry.hpp>
+#include <libtorrent/bdecode.hpp>
 #include <libtorrent/upnp.hpp>
 #include <libtorrent/socks5_stream.hpp>
+#include "boost_python.hpp"
 
 using namespace boost::python;
 using namespace libtorrent;
