@@ -3,7 +3,7 @@
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include <libtorrent/fingerprint.hpp>
-#include <boost/python.hpp>
+#include "boost_python.hpp"
 
 void bind_fingerprint()
 {
