@@ -17,8 +17,10 @@
 #include <libtorrent/torrent_info.hpp>
 #include <libtorrent/kademlia/item.hpp> // for sign_mutable_item
 
+#ifndef TORRENT_NO_DEPRECATE
 #include <libtorrent/extensions/lt_trackers.hpp>
 #include <libtorrent/extensions/metadata_transfer.hpp>
+#endif
 #include <libtorrent/extensions/smart_ban.hpp>
 #include <libtorrent/extensions/ut_metadata.hpp>
 #include <libtorrent/extensions/ut_pex.hpp>
@@ -88,9 +90,9 @@ namespace
             s.add_extension(create_ut_pex_plugin);
        else if (name == "smart_ban")
             s.add_extension(create_smart_ban_plugin);
+#ifndef TORRENT_NO_DEPRECATE
        else if (name == "lt_trackers")
             s.add_extension(create_lt_trackers_plugin);
-#ifndef TORRENT_NO_DEPRECATE
        else if (name == "metadata_transfer")
             s.add_extension(create_metadata_plugin);
 #endif // TORRENT_NO_DEPRECATE
@@ -101,16 +103,17 @@ namespace
 	void make_settings_pack(lt::settings_pack& p, dict const& sett_dict)
 	{
 		list iterkeys = (list)sett_dict.keys();
-		for (int i = 0; i < boost::python::len(iterkeys); i++)
+		int const len = boost::python::len(iterkeys);
+		for (int i = 0; i < len; i++)
 		{
-			std::string key = extract<std::string>(iterkeys[i]);
+			std::string const key = extract<std::string>(iterkeys[i]);
 
 			int sett = setting_by_name(key);
-			if (sett == 0) continue;
+			if (sett < 0) continue;
 
 			TORRENT_TRY
 			{
-				object value = sett_dict[key];
+				object const value = sett_dict[key];
 				switch (sett & settings_pack::type_mask)
 				{
 					case settings_pack::string_type_base:
@@ -497,9 +500,6 @@ namespace
        }
        return pieces;
     }
-
-    void load_asnum_db(lt::session& s, std::string file) {}
-    void load_country_db(lt::session& s, std::string file) {}
 #endif
 
     entry save_state(lt::session const& s, boost::uint32_t flags)
@@ -771,7 +771,7 @@ void bind_session()
                 , arg("alert_mask")=int(alert::error_notification)))
         )
 #endif
-        .def("post_torrent_updates", allow_threads(&lt::session::post_torrent_updates))
+        .def("post_torrent_updates", allow_threads(&lt::session::post_torrent_updates), arg("flags") = 0xffffffff)
         .def("post_session_stats", allow_threads(&lt::session::post_session_stats))
         .def("outgoing_ports", &outgoing_ports)
         .def("is_listening", allow_threads(&lt::session::is_listening))
@@ -810,7 +810,7 @@ void bind_session()
 #ifndef TORRENT_NO_DEPRECATE
         .def("add_feed", &add_feed)
         .def("status", allow_threads(&lt::session::status))
-        .def("settings", &session_get_settings)
+        .def("settings", &lt::session::settings)
         .def("set_settings", &session_set_settings)
 #endif
         .def("get_settings", &session_get_settings)
@@ -874,8 +874,6 @@ void bind_session()
         .def("set_max_connections", allow_threads(&lt::session::set_max_connections))
         .def("max_connections", allow_threads(&lt::session::max_connections))
         .def("num_connections", allow_threads(&lt::session::num_connections))
-        .def("load_asnum_db", &load_asnum_db)
-        .def("load_country_db", &load_country_db)
         .def("set_max_half_open_connections", allow_threads(&lt::session::set_max_half_open_connections))
         .def("set_severity_level", allow_threads(&lt::session::set_severity_level))
         .def("set_alert_queue_size_limit", allow_threads(&lt::session::set_alert_queue_size_limit))
