@@ -193,9 +193,9 @@ void dict_to_announce_entry(dict d, announce_entry& ae)
    if (d.has_key("source"))
       ae.source = extract<int>(d["source"]);
    if (d.has_key("verified"))
-      ae.verified = extract<int>(d["verified"]);
+      ae.verified = extract<bool>(d["verified"]);
    if (d.has_key("send_stats"))
-      ae.send_stats = extract<int>(d["send_stats"]);
+      ae.send_stats = extract<bool>(d["send_stats"]);
 }
 
 void replace_trackers(torrent_handle& h, object trackers)
