@@ -1,23 +1,26 @@
+// ignore warnings in this file
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include "fixedint.h"
 #include "sc.h"
 
-static uint64_t load_3(const unsigned char *in) {
-    uint64_t result;
+static u64 load_3(const unsigned char *in) {
+    u64 result;
 
-    result = (uint64_t) in[0];
-    result |= ((uint64_t) in[1]) << 8;
-    result |= ((uint64_t) in[2]) << 16;
+    result = (u64) in[0];
+    result |= ((u64) in[1]) << 8;
+    result |= ((u64) in[2]) << 16;
 
     return result;
 }
 
-static uint64_t load_4(const unsigned char *in) {
-    uint64_t result;
+static u64 load_4(const unsigned char *in) {
+    u64 result;
 
-    result = (uint64_t) in[0];
-    result |= ((uint64_t) in[1]) << 8;
-    result |= ((uint64_t) in[2]) << 16;
-    result |= ((uint64_t) in[3]) << 24;
+    result = (u64) in[0];
+    result |= ((u64) in[1]) << 8;
+    result |= ((u64) in[2]) << 16;
+    result |= ((u64) in[3]) << 24;
     
     return result;
 }
@@ -33,47 +36,47 @@ Output:
 */
 
 void sc_reduce(unsigned char *s) {
-    int64_t s0 = 2097151 & load_3(s);
-    int64_t s1 = 2097151 & (load_4(s + 2) >> 5);
-    int64_t s2 = 2097151 & (load_3(s + 5) >> 2);
-    int64_t s3 = 2097151 & (load_4(s + 7) >> 7);
-    int64_t s4 = 2097151 & (load_4(s + 10) >> 4);
-    int64_t s5 = 2097151 & (load_3(s + 13) >> 1);
-    int64_t s6 = 2097151 & (load_4(s + 15) >> 6);
-    int64_t s7 = 2097151 & (load_3(s + 18) >> 3);
-    int64_t s8 = 2097151 & load_3(s + 21);
-    int64_t s9 = 2097151 & (load_4(s + 23) >> 5);
-    int64_t s10 = 2097151 & (load_3(s + 26) >> 2);
-    int64_t s11 = 2097151 & (load_4(s + 28) >> 7);
-    int64_t s12 = 2097151 & (load_4(s + 31) >> 4);
-    int64_t s13 = 2097151 & (load_3(s + 34) >> 1);
-    int64_t s14 = 2097151 & (load_4(s + 36) >> 6);
-    int64_t s15 = 2097151 & (load_3(s + 39) >> 3);
-    int64_t s16 = 2097151 & load_3(s + 42);
-    int64_t s17 = 2097151 & (load_4(s + 44) >> 5);
-    int64_t s18 = 2097151 & (load_3(s + 47) >> 2);
-    int64_t s19 = 2097151 & (load_4(s + 49) >> 7);
-    int64_t s20 = 2097151 & (load_4(s + 52) >> 4);
-    int64_t s21 = 2097151 & (load_3(s + 55) >> 1);
-    int64_t s22 = 2097151 & (load_4(s + 57) >> 6);
-    int64_t s23 = (load_4(s + 60) >> 3);
-    int64_t carry0;
-    int64_t carry1;
-    int64_t carry2;
-    int64_t carry3;
-    int64_t carry4;
-    int64_t carry5;
-    int64_t carry6;
-    int64_t carry7;
-    int64_t carry8;
-    int64_t carry9;
-    int64_t carry10;
-    int64_t carry11;
-    int64_t carry12;
-    int64_t carry13;
-    int64_t carry14;
-    int64_t carry15;
-    int64_t carry16;
+    i64 s0 = 2097151 & load_3(s);
+    i64 s1 = 2097151 & (load_4(s + 2) >> 5);
+    i64 s2 = 2097151 & (load_3(s + 5) >> 2);
+    i64 s3 = 2097151 & (load_4(s + 7) >> 7);
+    i64 s4 = 2097151 & (load_4(s + 10) >> 4);
+    i64 s5 = 2097151 & (load_3(s + 13) >> 1);
+    i64 s6 = 2097151 & (load_4(s + 15) >> 6);
+    i64 s7 = 2097151 & (load_3(s + 18) >> 3);
+    i64 s8 = 2097151 & load_3(s + 21);
+    i64 s9 = 2097151 & (load_4(s + 23) >> 5);
+    i64 s10 = 2097151 & (load_3(s + 26) >> 2);
+    i64 s11 = 2097151 & (load_4(s + 28) >> 7);
+    i64 s12 = 2097151 & (load_4(s + 31) >> 4);
+    i64 s13 = 2097151 & (load_3(s + 34) >> 1);
+    i64 s14 = 2097151 & (load_4(s + 36) >> 6);
+    i64 s15 = 2097151 & (load_3(s + 39) >> 3);
+    i64 s16 = 2097151 & load_3(s + 42);
+    i64 s17 = 2097151 & (load_4(s + 44) >> 5);
+    i64 s18 = 2097151 & (load_3(s + 47) >> 2);
+    i64 s19 = 2097151 & (load_4(s + 49) >> 7);
+    i64 s20 = 2097151 & (load_4(s + 52) >> 4);
+    i64 s21 = 2097151 & (load_3(s + 55) >> 1);
+    i64 s22 = 2097151 & (load_4(s + 57) >> 6);
+    i64 s23 = (load_4(s + 60) >> 3);
+    i64 carry0;
+    i64 carry1;
+    i64 carry2;
+    i64 carry3;
+    i64 carry4;
+    i64 carry5;
+    i64 carry6;
+    i64 carry7;
+    i64 carry8;
+    i64 carry9;
+    i64 carry10;
+    i64 carry11;
+    i64 carry12;
+    i64 carry13;
+    i64 carry14;
+    i64 carry15;
+    i64 carry16;
 
     s11 += s23 * 666643;
     s12 += s23 * 470296;
@@ -360,89 +363,89 @@ Output:
 */
 
 void sc_muladd(unsigned char *s, const unsigned char *a, const unsigned char *b, const unsigned char *c) {
-    int64_t a0 = 2097151 & load_3(a);
-    int64_t a1 = 2097151 & (load_4(a + 2) >> 5);
-    int64_t a2 = 2097151 & (load_3(a + 5) >> 2);
-    int64_t a3 = 2097151 & (load_4(a + 7) >> 7);
-    int64_t a4 = 2097151 & (load_4(a + 10) >> 4);
-    int64_t a5 = 2097151 & (load_3(a + 13) >> 1);
-    int64_t a6 = 2097151 & (load_4(a + 15) >> 6);
-    int64_t a7 = 2097151 & (load_3(a + 18) >> 3);
-    int64_t a8 = 2097151 & load_3(a + 21);
-    int64_t a9 = 2097151 & (load_4(a + 23) >> 5);
-    int64_t a10 = 2097151 & (load_3(a + 26) >> 2);
-    int64_t a11 = (load_4(a + 28) >> 7);
-    int64_t b0 = 2097151 & load_3(b);
-    int64_t b1 = 2097151 & (load_4(b + 2) >> 5);
-    int64_t b2 = 2097151 & (load_3(b + 5) >> 2);
-    int64_t b3 = 2097151 & (load_4(b + 7) >> 7);
-    int64_t b4 = 2097151 & (load_4(b + 10) >> 4);
-    int64_t b5 = 2097151 & (load_3(b + 13) >> 1);
-    int64_t b6 = 2097151 & (load_4(b + 15) >> 6);
-    int64_t b7 = 2097151 & (load_3(b + 18) >> 3);
-    int64_t b8 = 2097151 & load_3(b + 21);
-    int64_t b9 = 2097151 & (load_4(b + 23) >> 5);
-    int64_t b10 = 2097151 & (load_3(b + 26) >> 2);
-    int64_t b11 = (load_4(b + 28) >> 7);
-    int64_t c0 = 2097151 & load_3(c);
-    int64_t c1 = 2097151 & (load_4(c + 2) >> 5);
-    int64_t c2 = 2097151 & (load_3(c + 5) >> 2);
-    int64_t c3 = 2097151 & (load_4(c + 7) >> 7);
-    int64_t c4 = 2097151 & (load_4(c + 10) >> 4);
-    int64_t c5 = 2097151 & (load_3(c + 13) >> 1);
-    int64_t c6 = 2097151 & (load_4(c + 15) >> 6);
-    int64_t c7 = 2097151 & (load_3(c + 18) >> 3);
-    int64_t c8 = 2097151 & load_3(c + 21);
-    int64_t c9 = 2097151 & (load_4(c + 23) >> 5);
-    int64_t c10 = 2097151 & (load_3(c + 26) >> 2);
-    int64_t c11 = (load_4(c + 28) >> 7);
-    int64_t s0;
-    int64_t s1;
-    int64_t s2;
-    int64_t s3;
-    int64_t s4;
-    int64_t s5;
-    int64_t s6;
-    int64_t s7;
-    int64_t s8;
-    int64_t s9;
-    int64_t s10;
-    int64_t s11;
-    int64_t s12;
-    int64_t s13;
-    int64_t s14;
-    int64_t s15;
-    int64_t s16;
-    int64_t s17;
-    int64_t s18;
-    int64_t s19;
-    int64_t s20;
-    int64_t s21;
-    int64_t s22;
-    int64_t s23;
-    int64_t carry0;
-    int64_t carry1;
-    int64_t carry2;
-    int64_t carry3;
-    int64_t carry4;
-    int64_t carry5;
-    int64_t carry6;
-    int64_t carry7;
-    int64_t carry8;
-    int64_t carry9;
-    int64_t carry10;
-    int64_t carry11;
-    int64_t carry12;
-    int64_t carry13;
-    int64_t carry14;
-    int64_t carry15;
-    int64_t carry16;
-    int64_t carry17;
-    int64_t carry18;
-    int64_t carry19;
-    int64_t carry20;
-    int64_t carry21;
-    int64_t carry22;
+    i64 a0 = 2097151 & load_3(a);
+    i64 a1 = 2097151 & (load_4(a + 2) >> 5);
+    i64 a2 = 2097151 & (load_3(a + 5) >> 2);
+    i64 a3 = 2097151 & (load_4(a + 7) >> 7);
+    i64 a4 = 2097151 & (load_4(a + 10) >> 4);
+    i64 a5 = 2097151 & (load_3(a + 13) >> 1);
+    i64 a6 = 2097151 & (load_4(a + 15) >> 6);
+    i64 a7 = 2097151 & (load_3(a + 18) >> 3);
+    i64 a8 = 2097151 & load_3(a + 21);
+    i64 a9 = 2097151 & (load_4(a + 23) >> 5);
+    i64 a10 = 2097151 & (load_3(a + 26) >> 2);
+    i64 a11 = (load_4(a + 28) >> 7);
+    i64 b0 = 2097151 & load_3(b);
+    i64 b1 = 2097151 & (load_4(b + 2) >> 5);
+    i64 b2 = 2097151 & (load_3(b + 5) >> 2);
+    i64 b3 = 2097151 & (load_4(b + 7) >> 7);
+    i64 b4 = 2097151 & (load_4(b + 10) >> 4);
+    i64 b5 = 2097151 & (load_3(b + 13) >> 1);
+    i64 b6 = 2097151 & (load_4(b + 15) >> 6);
+    i64 b7 = 2097151 & (load_3(b + 18) >> 3);
+    i64 b8 = 2097151 & load_3(b + 21);
+    i64 b9 = 2097151 & (load_4(b + 23) >> 5);
+    i64 b10 = 2097151 & (load_3(b + 26) >> 2);
+    i64 b11 = (load_4(b + 28) >> 7);
+    i64 c0 = 2097151 & load_3(c);
+    i64 c1 = 2097151 & (load_4(c + 2) >> 5);
+    i64 c2 = 2097151 & (load_3(c + 5) >> 2);
+    i64 c3 = 2097151 & (load_4(c + 7) >> 7);
+    i64 c4 = 2097151 & (load_4(c + 10) >> 4);
+    i64 c5 = 2097151 & (load_3(c + 13) >> 1);
+    i64 c6 = 2097151 & (load_4(c + 15) >> 6);
+    i64 c7 = 2097151 & (load_3(c + 18) >> 3);
+    i64 c8 = 2097151 & load_3(c + 21);
+    i64 c9 = 2097151 & (load_4(c + 23) >> 5);
+    i64 c10 = 2097151 & (load_3(c + 26) >> 2);
+    i64 c11 = (load_4(c + 28) >> 7);
+    i64 s0;
+    i64 s1;
+    i64 s2;
+    i64 s3;
+    i64 s4;
+    i64 s5;
+    i64 s6;
+    i64 s7;
+    i64 s8;
+    i64 s9;
+    i64 s10;
+    i64 s11;
+    i64 s12;
+    i64 s13;
+    i64 s14;
+    i64 s15;
+    i64 s16;
+    i64 s17;
+    i64 s18;
+    i64 s19;
+    i64 s20;
+    i64 s21;
+    i64 s22;
+    i64 s23;
+    i64 carry0;
+    i64 carry1;
+    i64 carry2;
+    i64 carry3;
+    i64 carry4;
+    i64 carry5;
+    i64 carry6;
+    i64 carry7;
+    i64 carry8;
+    i64 carry9;
+    i64 carry10;
+    i64 carry11;
+    i64 carry12;
+    i64 carry13;
+    i64 carry14;
+    i64 carry15;
+    i64 carry16;
+    i64 carry17;
+    i64 carry18;
+    i64 carry19;
+    i64 carry20;
+    i64 carry21;
+    i64 carry22;
 
     s0 = c0 + a0 * b0;
     s1 = c1 + a0 * b1 + a1 * b0;
