@@ -39,9 +39,14 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/file.hpp"
 #include "libtorrent/file_pool.hpp"
+#include "libtorrent/hex.hpp" // for from_hex
 
 #include <boost/bind.hpp>
 
+#ifdef TORRENT_WINDOWS
+#include <direct.h> // for _getcwd
+#endif
+
 using namespace libtorrent;
 
 int load_file(std::string const& filename, std::vector<char>& v, libtorrent::error_code& ec, int limit = 8000000)
@@ -50,21 +55,21 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	FILE* f = fopen(filename.c_str(), "rb");
 	if (f == NULL)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		return -1;
 	}
 
 	int r = fseek(f, 0, SEEK_END);
 	if (r != 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
 	long s = ftell(f);
 	if (s < 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -78,7 +83,7 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	r = fseek(f, 0, SEEK_SET);
 	if (r != 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -93,7 +98,7 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	r = fread(&v[0], 1, v.size(), f);
 	if (r < 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -105,11 +110,50 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	return 0;
 }
 
+std::string branch_path(std::string const& f)
+{
+	if (f.empty()) return f;
+
+#ifdef TORRENT_WINDOWS
+	if (f == "\\\\") return "";
+#endif
+	if (f == "/") return "";
+
+	int len = f.size();
+	// if the last character is / or \ ignore it
+	if (f[len-1] == '/' || f[len-1] == '\\') --len;
+	while (len > 0)
+	{
+		--len;
+		if (f[len] == '/' || f[len] == '\\')
+			break;
+	}
+
+	if (f[len] == '/' || f[len] == '\\') ++len;
+	return std::string(f.c_str(), len);
+}
+
 // do not include files and folders whose
 // name starts with a .
 bool file_filter(std::string const& f)
 {
-	if (filename(f)[0] == '.') return false;
+	if (f.empty()) return false;
+
+	char const* first = f.c_str();
+	char const* sep = strrchr(first, '/');
+#if defined(TORRENT_WINDOWS) || defined(TORRENT_OS2)
+	char const* altsep = strrchr(first, '\\');
+	if (sep == NULL || altsep > sep) sep = altsep;
+#endif
+	// if there is no parent path, just set 'sep'
+	// to point to the filename.
+	// if there is a parent path, skip the '/' character
+	if (sep == NULL) sep = first;
+	else ++sep;
+
+	// return false if the first character of the filename is a .
+	if (sep[0] == '.') return false;
+
 	fprintf(stderr, "%s\n", f.c_str());
 	return true;
 }
@@ -126,32 +170,38 @@ void print_usage()
 		"Generates a torrent file from the specified file\n"
 		"or directory and writes it to standard out\n\n"
 		"OPTIONS:\n"
-		"-m file     generate a merkle hash tree torrent.\n"
-		"            merkle torrents require client support\n"
-		"            the resulting full merkle tree is written to\n"
-		"            the specified file\n"
-		"-f          include sha-1 file hashes in the torrent\n"
-		"            this helps supporting mixing sources from\n"
-		"            other networks\n"
-		"-w url      adds a web seed to the torrent with\n"
-		"            the specified url\n"
-		"-t url      adds the specified tracker to the\n"
-		"            torrent. For multiple trackers, specify more\n"
-		"            -t options\n"
-		"-c comment  sets the comment to the specified string\n"
-		"-C creator  sets the created-by field to the specified string\n"
-		"-p bytes    enables padding files. Files larger\n"
-		"            than bytes will be piece-aligned\n"
-		"-s bytes    specifies a piece size for the torrent\n"
-		"            This has to be a multiple of 16 kiB\n"
-		"-l          Don't follow symlinks, instead encode them as\n"
-		"            links in the torrent file\n"
-		"-o file     specifies the output filename of the torrent file\n"
-		"            If this is not specified, the torrent file is\n"
-		"            printed to the standard out, except on windows\n"
-		"            where the filename defaults to a.torrent\n"
-		"-r file     add root certificate to the torrent, to verify\n"
-		"            the HTTPS tracker\n"
+		"-m file       generate a merkle hash tree torrent.\n"
+		"              merkle torrents require client support\n"
+		"              the resulting full merkle tree is written to\n"
+		"              the specified file\n"
+		"-w url        adds a web seed to the torrent with\n"
+		"              the specified url\n"
+		"-t url        adds the specified tracker to the\n"
+		"              torrent. For multiple trackers, specify more\n"
+		"              -t options\n"
+		"-c comment    sets the comment to the specified string\n"
+		"-C creator    sets the created-by field to the specified string\n"
+		"-p bytes      enables padding files. Files larger\n"
+		"              than bytes will be piece-aligned\n"
+		"-s bytes      specifies a piece size for the torrent\n"
+		"              This has to be a multiple of 16 kiB\n"
+		"-l            Don't follow symlinks, instead encode them as\n"
+		"              links in the torrent file\n"
+		"-o file       specifies the output filename of the torrent file\n"
+		"              If this is not specified, the torrent file is\n"
+		"              printed to the standard out, except on windows\n"
+		"              where the filename defaults to a.torrent\n"
+		"-r file       add root certificate to the torrent, to verify\n"
+		"              the HTTPS tracker\n"
+		"-S info-hash  add a similar torrent by info-hash. The similar\n"
+		"              torrent is expected to share some files with this one\n"
+		"-L collection add a collection name to this torrent. Other torrents\n"
+		"              in the same collection is expected to share files\n"
+		"              with this one.\n"
+		"-M            make the torrent compatible with mutable torrents\n"
+		"              this means aligning large files and pad them in order\n"
+		"              for piece hashes to uniquely indentify a file without\n"
+		"              overlap\n"
 		, stderr);
 }
 
@@ -174,6 +224,8 @@ int main(int argc, char* argv[])
 #endif
 		std::vector<std::string> web_seeds;
 		std::vector<std::string> trackers;
+		std::vector<std::string> collections;
+		std::vector<sha1_hash> similar;
 		int pad_file_limit = -1;
 		int piece_size = 0;
 		int flags = 0;
@@ -205,10 +257,14 @@ int main(int argc, char* argv[])
 					++i;
 					trackers.push_back(argv[i]);
 					break;
+				case 'M':
+					flags |= create_torrent::mutable_torrent_support;
+					pad_file_limit = 0x4000;
+					break;
 				case 'p':
 					++i;
 					pad_file_limit = atoi(argv[i]);
-					flags |= create_torrent::optimize;
+					flags |= create_torrent::optimize_alignment;
 					break;
 				case 's':
 					++i;
@@ -223,9 +279,6 @@ int main(int argc, char* argv[])
 					++i;
 					outfile = argv[i];
 					break;
-				case 'f':
-					flags |= create_torrent::calculate_file_hashes;
-					break;
 				case 'l':
 					flags |= create_torrent::symlinks;
 					break;
@@ -241,6 +294,30 @@ int main(int argc, char* argv[])
 					++i;
 					root_cert = argv[i];
 					break;
+				case 'S':
+					{
+					++i;
+					if (strlen(argv[i]) != 40)
+					{
+						fprintf(stderr, "invalid info-hash for -S. "
+							"Expected 40 hex characters\n");
+						print_usage();
+						return 1;
+					}
+					sha1_hash ih;
+					if (!from_hex(argv[i], 40, (char*)&ih[0]))
+					{
+						fprintf(stderr, "invalid info-hash for -S\n");
+						print_usage();
+						return 1;
+					}
+					similar.push_back(ih);
+					}
+					break;
+				case 'L':
+					++i;
+					collections.push_back(argv[i]);
+					break;
 				default:
 					print_usage();
 					return 1;
@@ -248,7 +325,22 @@ int main(int argc, char* argv[])
 		}
 
 		file_storage fs;
-		std::string full_path = libtorrent::complete(argv[1]);
+		std::string full_path = argv[1];
+#ifdef TORRENT_WINDOWS
+		if (full_path[1] != ':')
+#else
+		if (full_path[0] != '/')
+#endif
+		{
+			char cwd[TORRENT_MAX_PATH];
+#ifdef TORRENT_WINDOWS
+			_getcwd(cwd, sizeof(cwd));
+			full_path = cwd + ("\\" + full_path);
+#else
+			getcwd(cwd, sizeof(cwd));
+			full_path = cwd + ("/" + full_path);
+#endif
+		}
 
 		add_files(fs, full_path, file_filter, flags);
 		if (fs.num_files() == 0)
@@ -267,8 +359,16 @@ int main(int argc, char* argv[])
 			, end(web_seeds.end()); i != end; ++i)
 			t.add_url_seed(*i);
 
+		for (std::vector<std::string>::iterator i = collections.begin()
+			, end(collections.end()); i != end; ++i)
+			t.add_collection(*i);
+
+		for (std::vector<sha1_hash>::iterator i = similar.begin()
+			, end(similar.end()); i != end; ++i)
+			t.add_similar_torrent(*i);
+
 		error_code ec;
-		set_piece_hashes(t, parent_path(full_path)
+		set_piece_hashes(t, branch_path(full_path)
 			, boost::bind(&print_progress, _1, t.num_pieces()), ec);
 		if (ec)
 		{
