@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2005-2014, Arvid Norberg
+Copyright (c) 2005-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,23 +33,20 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_IP_FILTER_HPP
 #define TORRENT_IP_FILTER_HPP
 
+#include "libtorrent/config.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <set>
 #include <vector>
 
-#ifdef _MSC_VER
-#pragma warning(push, 1)
-#endif
-
 #include <boost/limits.hpp>
 #include <boost/utility.hpp>
+#include <boost/cstdint.hpp>
 #include <boost/tuple/tuple.hpp>
 
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
 
-
-#include "libtorrent/config.hpp"
 #include "libtorrent/address.hpp"
 #include "libtorrent/assert.hpp"
 
@@ -68,7 +65,7 @@ struct ip_range
 {
 	Addr first;
 	Addr last;
-	int flags;
+	boost::uint32_t flags;
 };
 
 namespace detail
@@ -152,17 +149,17 @@ namespace detail
 		{
 			TORRENT_ASSERT(!m_access_list.empty());
 			TORRENT_ASSERT(first < last || first == last);
-			
+
 			typename range_t::iterator i = m_access_list.upper_bound(first);
 			typename range_t::iterator j = m_access_list.upper_bound(last);
 
 			if (i != m_access_list.begin()) --i;
-			
+
 			TORRENT_ASSERT(j != m_access_list.begin());
 			TORRENT_ASSERT(j != i);
-			
-			int first_access = i->access;
-			int last_access = boost::prior(j)->access;
+
+			boost::uint32_t first_access = i->access;
+			boost::uint32_t last_access = boost::prior(j)->access;
 
 			if (i->start != first && first_access != flags)
 			{
@@ -182,13 +179,13 @@ namespace detail
 				// we can do this const-cast because we know that the new
 				// start address will keep the set correctly ordered
 				const_cast<Addr&>(i->start) = first;
-				const_cast<int&>(i->access) = flags;
+				const_cast<boost::uint32_t&>(i->access) = flags;
 			}
 			else if (first_access != flags)
 			{
 				m_access_list.insert(i, range(first, flags));
 			}
-			
+
 			if ((j != m_access_list.end()
 					&& minus_one(j->start) != last)
 				|| (j == m_access_list.end()
@@ -203,7 +200,7 @@ namespace detail
 			TORRENT_ASSERT(!m_access_list.empty());
 		}
 
-		int access(Addr const& addr) const
+		boost::uint32_t access(Addr const& addr) const
 		{
 			TORRENT_ASSERT(!m_access_list.empty());
 			typename range_t::const_iterator i = m_access_list.upper_bound(addr);
@@ -232,14 +229,14 @@ namespace detail
 					r.last = ExternalAddressType(max_addr<Addr>());
 				else
 					r.last = ExternalAddressType(minus_one(i->start));
-			
+
 				ret.push_back(r);
 			}
 			return ret;
 		}
 
 	private:
-	
+
 		struct range
 		{
 			range(Addr addr, int a = 0): start(addr), access(a) {}
@@ -250,7 +247,7 @@ namespace detail
 			Addr start;
 			// the end of the range is implicit
 			// and given by the next entry in the set
-			int access;
+			boost::uint32_t access;
 		};
 
 		typedef std::set<range> range_t;
@@ -290,7 +287,7 @@ struct TORRENT_EXPORT ip_filter
 	// 
 	// This means that in a case of overlapping ranges, the last one applied takes
 	// precedence.
-	void add_rule(address first, address last, int flags);
+	void add_rule(address first, address last, boost::uint32_t flags);
 
 	// Returns the access permissions for the given address (``addr``). The permission
 	// can currently be 0 or ``ip_filter::blocked``. The complexity of this operation
@@ -304,7 +301,7 @@ struct TORRENT_EXPORT ip_filter
 #else
 	typedef std::vector<ip_range<address_v4> > filter_tuple_t;
 #endif
-	
+
 	// This function will return the current state of the filter in the minimum number of
 	// ranges possible. They are sorted from ranges in low addresses to high addresses. Each
 	// entry in the returned vector is a range with the access control specified in its
@@ -315,7 +312,7 @@ struct TORRENT_EXPORT ip_filter
 	filter_tuple_t export_filter() const;
 
 //	void print() const;
-	
+
 private:
 
 	detail::filter_impl<address_v4::bytes_type> m_filter4;
@@ -343,7 +340,7 @@ public:
 	// set the flags for the specified port range (``first``, ``last``) to
 	// ``flags`` overwriting any existing rule for those ports. The range
 	// is inclusive, i.e. the port ``last`` also has the flag set on it.
-	void add_rule(boost::uint16_t first, boost::uint16_t last, int flags);
+	void add_rule(boost::uint16_t first, boost::uint16_t last, boost::uint32_t flags);
 
 	// test the specified port (``port``) for whether it is blocked
 	// or not. The returned value is the flags set for this port.
