@@ -78,6 +78,9 @@ namespace libtorrent
 	// This function parses out information from the magnet link and populates the
 	// add_torrent_params object.
 	TORRENT_EXPORT void parse_magnet_uri(std::string const& uri, add_torrent_params& p, error_code& ec);
+
+	// internal, delete when merge in master
+	TORRENT_EXTRA_EXPORT void parse_magnet_uri_peers(std::string const& uri, std::vector<tcp::endpoint>& peers);
 }
 
 #endif
