@@ -1,8 +1,8 @@
 /*
  * Copyright 2001-2004 Unicode, Inc.
- * 
+ *
  * Disclaimer
- * 
+ *
  * This source code is provided as is by Unicode, Inc. No claims are
  * made as to fitness for any particular purpose. No warranties of any
  * kind are expressed or implied. The recipient agrees to determine
@@ -10,9 +10,9 @@
  * purchased on magnetic or optical media from Unicode, Inc., the
  * sole remedy for any claim will be exchange of defective media
  * within 90 days of receipt.
- * 
+ *
  * Limitations on Rights to Redistribute This Code
- * 
+ *
  * Unicode, Inc. hereby grants the right to freely use the information
  * supplied in this file in the creation of products supporting the
  * Unicode Standard, and to make copies of this file in any form
@@ -26,10 +26,10 @@
     Author: Mark E. Davis, 1994.
     Rev History: Rick McGowan, fixes & updates May 2001.
     Sept 2001: fixed const & error conditions per
-	mods suggested by S. Parent & A. Lillich.
+    mods suggested by S. Parent & A. Lillich.
     June 2002: Tim Dodd added detection and handling of incomplete
-	source sequences, enhanced error detection, added casts
-	to eliminate compiler warnings.
+    source sequences, enhanced error detection, added casts
+    to eliminate compiler warnings.
     July 2003: slight mods to back out aggressive FFFE detection.
     Jan 2004: updated switches in from-UTF8 conversions.
     Oct 2004: updated to use UNI_MAX_LEGAL_UTF32 in UTF-32 conversions.
@@ -38,6 +38,8 @@
 
 ------------------------------------------------------------------------ */
 
+// ignore warnings in this file
+#include "libtorrent/aux_/disable_warnings_push.hpp"
 
 #include "libtorrent/ConvertUTF.h"
 #ifdef CVTUTF_DEBUG
@@ -53,113 +55,111 @@ static const UTF32 halfMask = 0x3FFUL;
 #define UNI_SUR_HIGH_END    (UTF32)0xDBFF
 #define UNI_SUR_LOW_START   (UTF32)0xDC00
 #define UNI_SUR_LOW_END     (UTF32)0xDFFF
-#define false	   0
-#define true	    1
 
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF32toUTF16 (
-	const UTF32** sourceStart, const UTF32* sourceEnd, 
+	const UTF32** sourceStart, const UTF32* sourceEnd,
 	UTF16** targetStart, UTF16* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF32* source = *sourceStart;
-    UTF16* target = *targetStart;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF32* source = *sourceStart;
+	UTF16* target = *targetStart;
+	while (source < sourceEnd) {
 	UTF32 ch;
 	if (target >= targetEnd) {
-	    result = targetExhausted; break;
+		result = targetExhausted; break;
 	}
 	ch = *source++;
 	if (ch <= UNI_MAX_BMP) { /* Target is a character <= 0xFFFF */
-	    /* UTF-16 surrogate values are illegal in UTF-32; 0xffff or 0xfffe are both reserved values */
-	    if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
+		/* UTF-16 surrogate values are illegal in UTF-32; 0xffff or 0xfffe are both reserved values */
+		if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
 		if (flags == strictConversion) {
-		    --source; /* return to the illegal value itself */
-		    result = sourceIllegal;
-		    break;
+			--source; /* return to the illegal value itself */
+			result = sourceIllegal;
+			break;
 		} else {
-		    *target++ = UNI_REPLACEMENT_CHAR;
+			*target++ = UNI_REPLACEMENT_CHAR;
 		}
-	    } else {
+		} else {
 		*target++ = (UTF16)ch; /* normal case */
-	    }
+		}
 	} else if (ch > UNI_MAX_LEGAL_UTF32) {
-	    if (flags == strictConversion) {
+		if (flags == strictConversion) {
 		result = sourceIllegal;
-	    } else {
+		} else {
 		*target++ = UNI_REPLACEMENT_CHAR;
-	    }
+		}
 	} else {
-	    /* target is a character in range 0xFFFF - 0x10FFFF. */
-	    if (target + 1 >= targetEnd) {
+		/* target is a character in range 0xFFFF - 0x10FFFF. */
+		if (target + 1 >= targetEnd) {
 		--source; /* Back up source pointer! */
 		result = targetExhausted; break;
-	    }
-	    ch -= halfBase;
-	    *target++ = (UTF16)((ch >> halfShift) + UNI_SUR_HIGH_START);
-	    *target++ = (UTF16)((ch & halfMask) + UNI_SUR_LOW_START);
+		}
+		ch -= halfBase;
+		*target++ = (UTF16)((ch >> halfShift) + UNI_SUR_HIGH_START);
+		*target++ = (UTF16)((ch & halfMask) + UNI_SUR_LOW_START);
 	}
-    }
-    *sourceStart = source;
-    *targetStart = target;
-    return result;
+	}
+	*sourceStart = source;
+	*targetStart = target;
+	return result;
 }
 
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF16toUTF32 (
-	const UTF16** sourceStart, const UTF16* sourceEnd, 
+	const UTF16** sourceStart, const UTF16* sourceEnd,
 	UTF32** targetStart, UTF32* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF16* source = *sourceStart;
-    UTF32* target = *targetStart;
-    UTF32 ch, ch2;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF16* source = *sourceStart;
+	UTF32* target = *targetStart;
+	UTF32 ch, ch2;
+	while (source < sourceEnd) {
 	const UTF16* oldSource = source; /*  In case we have to back up because of target overflow. */
 	ch = *source++;
 	/* If we have a surrogate pair, convert to UTF32 first. */
 	if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_HIGH_END) {
-	    /* If the 16 bits following the high surrogate are in the source buffer... */
-	    if (source < sourceEnd) {
+		/* If the 16 bits following the high surrogate are in the source buffer... */
+		if (source < sourceEnd) {
 		ch2 = *source;
 		/* If it's a low surrogate, convert to UTF32. */
 		if (ch2 >= UNI_SUR_LOW_START && ch2 <= UNI_SUR_LOW_END) {
-		    ch = ((ch - UNI_SUR_HIGH_START) << halfShift)
+			ch = ((ch - UNI_SUR_HIGH_START) << halfShift)
 			+ (ch2 - UNI_SUR_LOW_START) + halfBase;
-		    ++source;
+			++source;
 		} else if (flags == strictConversion) { /* it's an unpaired high surrogate */
-		    --source; /* return to the illegal value itself */
-		    result = sourceIllegal;
-		    break;
+			--source; /* return to the illegal value itself */
+			result = sourceIllegal;
+			break;
 		}
-	    } else { /* We don't have the 16 bits following the high surrogate. */
+		} else { /* We don't have the 16 bits following the high surrogate. */
 		--source; /* return to the high surrogate */
 		result = sourceExhausted;
 		break;
-	    }
+		}
 	} else if (flags == strictConversion) {
-	    /* UTF-16 surrogate values are illegal in UTF-32 */
-	    if (ch >= UNI_SUR_LOW_START && ch <= UNI_SUR_LOW_END) {
+		/* UTF-16 surrogate values are illegal in UTF-32 */
+		if (ch >= UNI_SUR_LOW_START && ch <= UNI_SUR_LOW_END) {
 		--source; /* return to the illegal value itself */
 		result = sourceIllegal;
 		break;
-	    }
+		}
 	}
 	if (target >= targetEnd) {
-	    source = oldSource; /* Back up source pointer! */
-	    result = targetExhausted; break;
+		source = oldSource; /* Back up source pointer! */
+		result = targetExhausted; break;
 	}
 	*target++ = ch;
-    }
-    *sourceStart = source;
-    *targetStart = target;
+	}
+	*sourceStart = source;
+	*targetStart = target;
 #ifdef CVTUTF_DEBUG
 if (result == sourceIllegal) {
-    fprintf(stderr, "ConvertUTF16toUTF32 illegal seq 0x%04x,%04x\n", ch, ch2);
-    fflush(stderr);
+	fprintf(stderr, "ConvertUTF16toUTF32 illegal seq 0x%04x,%04x\n", ch, ch2);
+	fflush(stderr);
 }
 #endif
-    return result;
+	return result;
 }
 
 /* --------------------------------------------------------------------- */
@@ -172,14 +172,14 @@ if (result == sourceIllegal) {
  * allowed in earlier algorithms.
  */
 static const char trailingBytesForUTF8[256] = {
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
-    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
-    2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
+	2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5
 };
 
 /*
@@ -187,8 +187,8 @@ static const char trailingBytesForUTF8[256] = {
  * This table contains as many values as there might be trailing bytes
  * in a UTF-8 sequence.
  */
-static const UTF32 offsetsFromUTF8[6] = { 0x00000000UL, 0x00003080UL, 0x000E2080UL, 
-		     0x03C82080UL, 0xFA082080UL, 0x82082080UL };
+static const UTF32 offsetsFromUTF8[6] = { 0x00000000UL, 0x00003080UL, 0x000E2080UL,
+	0x03C82080UL, 0xFA082080UL, 0x82082080UL };
 
 /*
  * Once the bits are split out into bytes of UTF-8, this is a mask OR-ed
@@ -212,71 +212,71 @@ static const UTF8 firstByteMark[7] = { 0x00, 0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF16toUTF8 (
-	const UTF16** sourceStart, const UTF16* sourceEnd, 
+	const UTF16** sourceStart, const UTF16* sourceEnd,
 	UTF8** targetStart, UTF8* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF16* source = *sourceStart;
-    UTF8* target = *targetStart;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF16* source = *sourceStart;
+	UTF8* target = *targetStart;
+	while (source < sourceEnd) {
 	UTF32 ch;
 	unsigned short bytesToWrite = 0;
 	const UTF32 byteMask = 0xBF;
-	const UTF32 byteMark = 0x80; 
+	const UTF32 byteMark = 0x80;
 	const UTF16* oldSource = source; /* In case we have to back up because of target overflow. */
 	ch = *source++;
 	/* If we have a surrogate pair, convert to UTF32 first. */
 	if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_HIGH_END) {
-	    /* If the 16 bits following the high surrogate are in the source buffer... */
-	    if (source < sourceEnd) {
+		/* If the 16 bits following the high surrogate are in the source buffer... */
+		if (source < sourceEnd) {
 		UTF32 ch2 = *source;
 		/* If it's a low surrogate, convert to UTF32. */
 		if (ch2 >= UNI_SUR_LOW_START && ch2 <= UNI_SUR_LOW_END) {
-		    ch = ((ch - UNI_SUR_HIGH_START) << halfShift)
+			ch = ((ch - UNI_SUR_HIGH_START) << halfShift)
 			+ (ch2 - UNI_SUR_LOW_START) + halfBase;
-		    ++source;
+			++source;
 		} else if (flags == strictConversion) { /* it's an unpaired high surrogate */
-		    --source; /* return to the illegal value itself */
-		    result = sourceIllegal;
-		    break;
+			--source; /* return to the illegal value itself */
+			result = sourceIllegal;
+			break;
 		}
-	    } else { /* We don't have the 16 bits following the high surrogate. */
+		} else { /* We don't have the 16 bits following the high surrogate. */
 		--source; /* return to the high surrogate */
 		result = sourceExhausted;
 		break;
-	    }
+		}
 	} else if (flags == strictConversion) {
-	    /* UTF-16 surrogate values are illegal in UTF-32 */
-	    if (ch >= UNI_SUR_LOW_START && ch <= UNI_SUR_LOW_END) {
+		/* UTF-16 surrogate values are illegal in UTF-32 */
+		if (ch >= UNI_SUR_LOW_START && ch <= UNI_SUR_LOW_END) {
 		--source; /* return to the illegal value itself */
 		result = sourceIllegal;
 		break;
-	    }
+		}
 	}
 	/* Figure out how many bytes the result will require */
-	if (ch < (UTF32)0x80) {	     bytesToWrite = 1;
-	} else if (ch < (UTF32)0x800) {     bytesToWrite = 2;
+	if (ch < (UTF32)0x80) {		 bytesToWrite = 1;
+	} else if (ch < (UTF32)0x800) {	 bytesToWrite = 2;
 	} else if (ch < (UTF32)0x10000) {   bytesToWrite = 3;
 	} else if (ch < (UTF32)0x110000) {  bytesToWrite = 4;
-	} else {			    bytesToWrite = 3;
-					    ch = UNI_REPLACEMENT_CHAR;
+	} else {				bytesToWrite = 3;
+						ch = UNI_REPLACEMENT_CHAR;
 	}
 
 	target += bytesToWrite;
 	if (target > targetEnd) {
-	    source = oldSource; /* Back up source pointer! */
-	    target -= bytesToWrite; result = targetExhausted; break;
+		source = oldSource; /* Back up source pointer! */
+		target -= bytesToWrite; result = targetExhausted; break;
 	}
 	switch (bytesToWrite) { /* note: everything falls through. */
-	    case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 1: *--target =  (UTF8)(ch | firstByteMark[bytesToWrite]);
+		case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 1: *--target =  (UTF8)(ch | firstByteMark[bytesToWrite]);
 	}
 	target += bytesToWrite;
-    }
-    *sourceStart = source;
-    *targetStart = target;
-    return result;
+	}
+	*sourceStart = source;
+	*targetStart = target;
+	return result;
 }
 
 /* --------------------------------------------------------------------- */
@@ -293,28 +293,28 @@ ConversionResult ConvertUTF16toUTF8 (
  */
 
 static Boolean isLegalUTF8(const UTF8 *source, int length) {
-    UTF8 a;
-    const UTF8 *srcptr = source+length;
-    switch (length) {
-    default: return false;
+	UTF8 a;
+	const UTF8 *srcptr = source+length;
+	switch (length) {
+	default: return false;
 	/* Everything else falls through when "true"... */
-    case 4: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
-    case 3: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
-    case 2: if ((a = (*--srcptr)) > 0xBF) return false;
+	case 4: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
+	case 3: if ((a = (*--srcptr)) < 0x80 || a > 0xBF) return false;
+	case 2: if ((a = (*--srcptr)) > 0xBF) return false;
 
 	switch (*source) {
-	    /* no fall-through in this inner switch */
-	    case 0xE0: if (a < 0xA0) return false; break;
-	    case 0xED: if (a > 0x9F) return false; break;
-	    case 0xF0: if (a < 0x90) return false; break;
-	    case 0xF4: if (a > 0x8F) return false; break;
-	    default:   if (a < 0x80) return false;
+		/* no fall-through in this inner switch */
+		case 0xE0: if (a < 0xA0) return false; break;
+		case 0xED: if (a > 0x9F) return false; break;
+		case 0xF0: if (a < 0x90) return false; break;
+		case 0xF4: if (a > 0x8F) return false; break;
+		default:   if (a < 0x80) return false;
 	}
 
-    case 1: if (*source >= 0x80 && *source < 0xC2) return false;
-    }
-    if (*source > 0xF4) return false;
-    return true;
+	case 1: if (*source >= 0x80 && *source < 0xC2) return false;
+	}
+	if (*source > 0xF4) return false;
+	return true;
 }
 
 /* --------------------------------------------------------------------- */
@@ -324,216 +324,216 @@ static Boolean isLegalUTF8(const UTF8 *source, int length) {
  * This is not used here; it's just exported.
  */
 Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd) {
-    int length = trailingBytesForUTF8[*source]+1;
-    if (source+length > sourceEnd) {
+	int length = trailingBytesForUTF8[*source]+1;
+	if (source+length > sourceEnd) {
 	return false;
-    }
-    return isLegalUTF8(source, length);
+	}
+	return isLegalUTF8(source, length);
 }
 
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF8toUTF16 (
-	const UTF8** sourceStart, const UTF8* sourceEnd, 
+	const UTF8** sourceStart, const UTF8* sourceEnd,
 	UTF16** targetStart, UTF16* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF8* source = *sourceStart;
-    UTF16* target = *targetStart;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF8* source = *sourceStart;
+	UTF16* target = *targetStart;
+	while (source < sourceEnd) {
 	UTF32 ch = 0;
 	unsigned short extraBytesToRead = trailingBytesForUTF8[*source];
 	if (source + extraBytesToRead >= sourceEnd) {
-	    result = sourceExhausted; break;
+		result = sourceExhausted; break;
 	}
 	/* Do this check whether lenient or strict */
 	if (! isLegalUTF8(source, extraBytesToRead+1)) {
-	    result = sourceIllegal;
-	    break;
+		result = sourceIllegal;
+		break;
 	}
 	/*
 	 * The cases all fall through. See "Note A" below.
 	 */
 	switch (extraBytesToRead) {
-	    case 5: ch += *source++; ch <<= 6; /* remember, illegal UTF-8 */
-	    case 4: ch += *source++; ch <<= 6; /* remember, illegal UTF-8 */
-	    case 3: ch += *source++; ch <<= 6;
-	    case 2: ch += *source++; ch <<= 6;
-	    case 1: ch += *source++; ch <<= 6;
-	    case 0: ch += *source++;
+		case 5: ch += *source++; ch <<= 6; /* remember, illegal UTF-8 */
+		case 4: ch += *source++; ch <<= 6; /* remember, illegal UTF-8 */
+		case 3: ch += *source++; ch <<= 6;
+		case 2: ch += *source++; ch <<= 6;
+		case 1: ch += *source++; ch <<= 6;
+		case 0: ch += *source++;
 	}
 	ch -= offsetsFromUTF8[extraBytesToRead];
 
 	if (target >= targetEnd) {
-	    source -= (extraBytesToRead+1); /* Back up source pointer! */
-	    result = targetExhausted; break;
+		source -= (extraBytesToRead+1); /* Back up source pointer! */
+		result = targetExhausted; break;
 	}
 	if (ch <= UNI_MAX_BMP) { /* Target is a character <= 0xFFFF */
-	    /* UTF-16 surrogate values are illegal in UTF-32 */
-	    if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
+		/* UTF-16 surrogate values are illegal in UTF-32 */
+		if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
 		if (flags == strictConversion) {
-		    source -= (extraBytesToRead+1); /* return to the illegal value itself */
-		    result = sourceIllegal;
-		    break;
+			source -= (extraBytesToRead+1); /* return to the illegal value itself */
+			result = sourceIllegal;
+			break;
 		} else {
-		    *target++ = UNI_REPLACEMENT_CHAR;
+			*target++ = UNI_REPLACEMENT_CHAR;
 		}
-	    } else {
+		} else {
 		*target++ = (UTF16)ch; /* normal case */
-	    }
+		}
 	} else if (ch > UNI_MAX_UTF16) {
-	    if (flags == strictConversion) {
+		if (flags == strictConversion) {
 		result = sourceIllegal;
 		source -= (extraBytesToRead+1); /* return to the start */
 		break; /* Bail out; shouldn't continue */
-	    } else {
+		} else {
 		*target++ = UNI_REPLACEMENT_CHAR;
-	    }
+		}
 	} else {
-	    /* target is a character in range 0xFFFF - 0x10FFFF. */
-	    if (target + 1 >= targetEnd) {
+		/* target is a character in range 0xFFFF - 0x10FFFF. */
+		if (target + 1 >= targetEnd) {
 		source -= (extraBytesToRead+1); /* Back up source pointer! */
 		result = targetExhausted; break;
-	    }
-	    ch -= halfBase;
-	    *target++ = (UTF16)((ch >> halfShift) + UNI_SUR_HIGH_START);
-	    *target++ = (UTF16)((ch & halfMask) + UNI_SUR_LOW_START);
+		}
+		ch -= halfBase;
+		*target++ = (UTF16)((ch >> halfShift) + UNI_SUR_HIGH_START);
+		*target++ = (UTF16)((ch & halfMask) + UNI_SUR_LOW_START);
 	}
-    }
-    *sourceStart = source;
-    *targetStart = target;
-    return result;
+	}
+	*sourceStart = source;
+	*targetStart = target;
+	return result;
 }
 
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF32toUTF8 (
-	const UTF32** sourceStart, const UTF32* sourceEnd, 
+	const UTF32** sourceStart, const UTF32* sourceEnd,
 	UTF8** targetStart, UTF8* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF32* source = *sourceStart;
-    UTF8* target = *targetStart;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF32* source = *sourceStart;
+	UTF8* target = *targetStart;
+	while (source < sourceEnd) {
 	UTF32 ch;
 	unsigned short bytesToWrite = 0;
 	const UTF32 byteMask = 0xBF;
-	const UTF32 byteMark = 0x80; 
+	const UTF32 byteMark = 0x80;
 	ch = *source++;
 	if (flags == strictConversion ) {
-	    /* UTF-16 surrogate values are illegal in UTF-32 */
-	    if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
+		/* UTF-16 surrogate values are illegal in UTF-32 */
+		if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
 		--source; /* return to the illegal value itself */
 		result = sourceIllegal;
 		break;
-	    }
+		}
 	}
 	/*
 	 * Figure out how many bytes the result will require. Turn any
 	 * illegally large UTF32 things (> Plane 17) into replacement chars.
 	 */
-	if (ch < (UTF32)0x80) {	     bytesToWrite = 1;
-	} else if (ch < (UTF32)0x800) {     bytesToWrite = 2;
-	} else if (ch < (UTF32)0x10000) {   bytesToWrite = 3;
-	} else if (ch <= UNI_MAX_LEGAL_UTF32) {  bytesToWrite = 4;
-	} else {			    bytesToWrite = 3;
-					    ch = UNI_REPLACEMENT_CHAR;
-					    result = sourceIllegal;
+	if (ch < (UTF32)0x80) { bytesToWrite = 1;
+	} else if (ch < (UTF32)0x800) { bytesToWrite = 2;
+	} else if (ch < (UTF32)0x10000) { bytesToWrite = 3;
+	} else if (ch <= UNI_MAX_LEGAL_UTF32) { bytesToWrite = 4;
+	} else { bytesToWrite = 3;
+		ch = UNI_REPLACEMENT_CHAR;
+		result = sourceIllegal;
 	}
-	
+
 	target += bytesToWrite;
 	if (target > targetEnd) {
-	    --source; /* Back up source pointer! */
-	    target -= bytesToWrite; result = targetExhausted; break;
+		--source; /* Back up source pointer! */
+		target -= bytesToWrite; result = targetExhausted; break;
 	}
 	switch (bytesToWrite) { /* note: everything falls through. */
-	    case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
-	    case 1: *--target = (UTF8) (ch | firstByteMark[bytesToWrite]);
+		case 4: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 3: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 2: *--target = (UTF8)((ch | byteMark) & byteMask); ch >>= 6;
+		case 1: *--target = (UTF8) (ch | firstByteMark[bytesToWrite]);
 	}
 	target += bytesToWrite;
-    }
-    *sourceStart = source;
-    *targetStart = target;
-    return result;
+	}
+	*sourceStart = source;
+	*targetStart = target;
+	return result;
 }
 
 /* --------------------------------------------------------------------- */
 
 ConversionResult ConvertUTF8toUTF32 (
-	const UTF8** sourceStart, const UTF8* sourceEnd, 
+	const UTF8** sourceStart, const UTF8* sourceEnd,
 	UTF32** targetStart, UTF32* targetEnd, ConversionFlags flags) {
-    ConversionResult result = conversionOK;
-    const UTF8* source = *sourceStart;
-    UTF32* target = *targetStart;
-    while (source < sourceEnd) {
+	ConversionResult result = conversionOK;
+	const UTF8* source = *sourceStart;
+	UTF32* target = *targetStart;
+	while (source < sourceEnd) {
 	UTF32 ch = 0;
 	unsigned short extraBytesToRead = trailingBytesForUTF8[*source];
 	if (source + extraBytesToRead >= sourceEnd) {
-	    result = sourceExhausted; break;
+		result = sourceExhausted; break;
 	}
 	/* Do this check whether lenient or strict */
 	if (! isLegalUTF8(source, extraBytesToRead+1)) {
-	    result = sourceIllegal;
-	    break;
+		result = sourceIllegal;
+		break;
 	}
 	/*
-	 * The cases all fall through. See "Note A" below.
-	 */
+	* The cases all fall through. See "Note A" below.
+	*/
 	switch (extraBytesToRead) {
-	    case 5: ch += *source++; ch <<= 6;
-	    case 4: ch += *source++; ch <<= 6;
-	    case 3: ch += *source++; ch <<= 6;
-	    case 2: ch += *source++; ch <<= 6;
-	    case 1: ch += *source++; ch <<= 6;
-	    case 0: ch += *source++;
+		case 5: ch += *source++; ch <<= 6;
+		case 4: ch += *source++; ch <<= 6;
+		case 3: ch += *source++; ch <<= 6;
+		case 2: ch += *source++; ch <<= 6;
+		case 1: ch += *source++; ch <<= 6;
+		case 0: ch += *source++;
 	}
 	ch -= offsetsFromUTF8[extraBytesToRead];
 
 	if (target >= targetEnd) {
-	    source -= (extraBytesToRead+1); /* Back up the source pointer! */
-	    result = targetExhausted; break;
+		source -= (extraBytesToRead+1); /* Back up the source pointer! */
+		result = targetExhausted; break;
 	}
 	if (ch <= UNI_MAX_LEGAL_UTF32) {
-	    /*
-	     * UTF-16 surrogate values are illegal in UTF-32, and anything
-	     * over Plane 17 (> 0x10FFFF) is illegal.
-	     */
-	    if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
+		/*
+		* UTF-16 surrogate values are illegal in UTF-32, and anything
+		* over Plane 17 (> 0x10FFFF) is illegal.
+		*/
+		if (ch >= UNI_SUR_HIGH_START && ch <= UNI_SUR_LOW_END) {
 		if (flags == strictConversion) {
-		    source -= (extraBytesToRead+1); /* return to the illegal value itself */
-		    result = sourceIllegal;
-		    break;
+			source -= (extraBytesToRead+1); /* return to the illegal value itself */
+			result = sourceIllegal;
+			break;
 		} else {
-		    *target++ = UNI_REPLACEMENT_CHAR;
+			*target++ = UNI_REPLACEMENT_CHAR;
 		}
-	    } else {
+		} else {
 		*target++ = ch;
-	    }
+		}
 	} else { /* i.e., ch > UNI_MAX_LEGAL_UTF32 */
-	    result = sourceIllegal;
-	    *target++ = UNI_REPLACEMENT_CHAR;
+		result = sourceIllegal;
+		*target++ = UNI_REPLACEMENT_CHAR;
 	}
-    }
-    *sourceStart = source;
-    *targetStart = target;
-    return result;
+	}
+	*sourceStart = source;
+	*targetStart = target;
+	return result;
 }
 
 /* ---------------------------------------------------------------------
 
-    Note A.
-    The fall-through switches in UTF-8 reading code save a
-    temp variable, some decrements & conditionals.  The switches
-    are equivalent to the following loop:
+	Note A.
+	The fall-through switches in UTF-8 reading code save a
+	temp variable, some decrements & conditionals.  The switches
+	are equivalent to the following loop:
 	{
-	    int tmpBytesToRead = extraBytesToRead+1;
-	    do {
+		int tmpBytesToRead = extraBytesToRead+1;
+		do {
 		ch += *source++;
 		--tmpBytesToRead;
 		if (tmpBytesToRead) ch <<= 6;
-	    } while (tmpBytesToRead > 0);
+		} while (tmpBytesToRead > 0);
 	}
-    In UTF-8 writing code, the switches on "bytesToWrite" are
-    similarly unrolled loops.
+	In UTF-8 writing code, the switches on "bytesToWrite" are
+	similarly unrolled loops.
 
    --------------------------------------------------------------------- */
