@@ -31,6 +31,7 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/create_torrent.hpp"
+#include "libtorrent/utf8.hpp"
 #include "libtorrent/file_pool.hpp"
 #include "libtorrent/storage.hpp"
 #include "libtorrent/aux_/escape_string.hpp" // for convert_to_wstring
@@ -81,14 +82,14 @@ namespace libtorrent
 			struct stat s;
 			if (lstat(convert_to_native(p).c_str(), &s) < 0) return 0;
 			int file_attr = 0;
-			if (s.st_mode & S_IXUSR) 
+			if (s.st_mode & S_IXUSR)
 				file_attr += file_storage::attribute_executable;
 			if (S_ISLNK(s.st_mode))
 				file_attr += file_storage::attribute_symlink;
 			return file_attr;
 #endif
 		}
-	
+
 #ifndef TORRENT_WINDOWS
 		std::string get_symlink_path_impl(char const* path)
 		{
@@ -149,7 +150,7 @@ namespace libtorrent
 
 				// mask all bits to check if the file is a symlink
 				if ((file_flags & file_storage::attribute_symlink)
-					&& (flags & create_torrent::symlinks)) 
+					&& (flags & create_torrent::symlinks))
 				{
 					std::string sym_path = get_symlink_path(f);
 					fs.add_file(l, 0, file_flags, s.mtime, sym_path);
@@ -262,6 +263,12 @@ namespace libtorrent
 			return;
 		}
 
+		if (t.files().total_size() == 0)
+		{
+			ec = error_code(errors::torrent_invalid_length, get_libtorrent_category());
+			return;
+		}
+
 		// dummy torrent object pointer
 		boost::shared_ptr<char> dummy;
 		counters cnt;
@@ -317,10 +324,8 @@ namespace libtorrent
 		, m_include_mtime((flags & modification_time) != 0)
 		, m_include_symlinks((flags & symlinks) != 0)
 	{
-		TORRENT_ASSERT(fs.num_files() > 0);
-
 		// return instead of crash in release mode
-		if (fs.num_files() == 0) return;
+		if (fs.num_files() == 0 || fs.total_size() == 0) return;
 
 		if (!m_multifile && has_parent_path(m_files.file_path(0))) m_multifile = true;
 
@@ -379,6 +384,10 @@ namespace libtorrent
 		, m_include_symlinks(false)
 	{
 		TORRENT_ASSERT(ti.is_valid());
+		TORRENT_ASSERT(ti.num_pieces() > 0);
+		TORRENT_ASSERT(ti.num_files() > 0);
+		TORRENT_ASSERT(ti.total_size() > 0);
+
 		if (ti.creation_date()) m_creation_date = *ti.creation_date();
 
 		if (!ti.creator().empty()) set_creator(ti.creator().c_str());
@@ -407,7 +416,9 @@ namespace libtorrent
 		m_piece_hash.resize(m_files.num_pieces());
 		for (int i = 0; i < num_pieces(); ++i) set_hash(i, ti.hash_for_piece(i));
 
-		m_info_dict = bdecode(&ti.metadata()[0], &ti.metadata()[0] + ti.metadata_size());
+		boost::shared_array<char> const info = ti.metadata();
+		int const size = ti.metadata_size();
+		m_info_dict.preformatted().assign(&info[0], &info[0] + size);
 		m_info_hash = ti.info_hash();
 	}
 
@@ -421,7 +432,7 @@ namespace libtorrent
 			return dict;
 
 		if (!m_urls.empty()) dict["announce"] = m_urls.front().first;
-		
+
 		if (!m_nodes.empty())
 		{
 			entry& nodes = dict["nodes"];
@@ -463,7 +474,7 @@ namespace libtorrent
 
 		if (!m_created_by.empty())
 			dict["created by"] = m_created_by;
-			
+
 		if (!m_url_seeds.empty())
 		{
 			if (m_url_seeds.size() == 1)
@@ -499,7 +510,8 @@ namespace libtorrent
 		}
 
 		entry& info = dict["info"];
-		if (m_info_dict.type() == entry::dictionary_t)
+		if (m_info_dict.type() == entry::dictionary_t
+			|| m_info_dict.type() == entry::preformatted_t)
 		{
 			info = m_info_dict;
 			return dict;
@@ -536,7 +548,7 @@ namespace libtorrent
 		{
 			if (m_include_mtime) info["mtime"] = m_files.mtime(0);
 			info["length"] = m_files.file_size(0);
-			int flags = m_files.file_flags(0);
+			int const flags = m_files.file_flags(0);
 			if (flags & (file_storage::flag_pad_file
 				| file_storage::flag_hidden
 				| file_storage::flag_executable
