@@ -90,7 +90,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #if TORRENT_USE_IFADDRS || TORRENT_USE_IFCONF || TORRENT_USE_NETLINK || TORRENT_USE_SYSCTL
 // capture this here where warnings are disabled (the macro generates warnings)
-const int siocgifmtu = SIOCGIFMTU;
+const unsigned long siocgifmtu = SIOCGIFMTU;
 #endif
 
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
@@ -275,6 +275,7 @@ int _System __libsocket_sysctl(int* mib, u_int namelen, void *oldp, size_t *oldl
 			, rt_info->destination.is_v4() ? AF_INET : AF_INET6);
 		if_indextoname(rtm->rtm_index, rt_info->name);
 
+		// TODO: get the MTU (and other interesting metrics) from the rt_msghdr instead
 		ifreq req;
 		memset(&req, 0, sizeof(req));
 		if_indextoname(rtm->rtm_index, req.ifr_name);
@@ -884,8 +885,11 @@ namespace libtorrent
 	for (char* next = buf.get(); next < end; next += rtm->rtm_msglen)
 	{
 		rtm = reinterpret_cast<rt_msghdr*>(next);
-		if (rtm->rtm_version != RTM_VERSION)
+		if (rtm->rtm_version != RTM_VERSION
+			|| rtm->rtm_type != RTM_ADD)
+		{
 			continue;
+		}
 
 		ip_route r;
 		if (parse_route(s, rtm, &r)) ret.push_back(r);
