@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008-2014, Arvid Norberg
+Copyright (c) 2008-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,12 +34,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 #include "libtorrent/error_code.hpp"
-#include "libtorrent/escape_string.hpp" // for to_string()
+#include "libtorrent/string_util.hpp" // for to_string()
+#include "libtorrent/aux_/escape_string.hpp" // for convert_to_native
 
 namespace libtorrent
 {
-#if BOOST_VERSION >= 103500
-
 	struct libtorrent_error_category : boost::system::error_category
 	{
 		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT;
@@ -50,7 +49,7 @@ namespace libtorrent
 
 	const char* libtorrent_error_category::name() const BOOST_SYSTEM_NOEXCEPT
 	{
-		return "libtorrent error";
+		return "libtorrent";
 	}
 
 	std::string libtorrent_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
@@ -171,7 +170,7 @@ namespace libtorrent
 			"SSL connection required",
 			"invalid SSL certificate",
 			"not an SSL torrent",
-			"",
+			"banned by port filter",
 			"",
 			"",
 			"",
@@ -204,7 +203,7 @@ namespace libtorrent
 			"invalid entry type in slot list",
 			"invalid piece index in slot list",
 			"pieces needs to be reordered",
-			"",
+			"fastresume not modified since last save",
 			"",
 			"",
 			"",
@@ -322,10 +321,9 @@ namespace libtorrent
 		static http_error_category http_category;
 		return http_category;
 	}
-#endif
 
 #ifndef BOOST_NO_EXCEPTIONS
-	const char* libtorrent_exception::what() const throw()
+	const char* libtorrent_exception::what() const TORRENT_EXCEPTION_THROW_SPECIFIER
 	{
 		if (!m_msg)
 		{
@@ -336,7 +334,7 @@ namespace libtorrent
 		return m_msg;
 	}
 
-	libtorrent_exception::~libtorrent_exception() throw()
+	libtorrent_exception::~libtorrent_exception() TORRENT_EXCEPTION_THROW_SPECIFIER
 	{
 		free(m_msg);
 	}
