@@ -684,6 +684,44 @@ namespace libtorrent
 #endif // TORRENT_WINDOWS
 	}
 
+	void move_file(std::string const& inf, std::string const& newf, error_code& ec)
+	{
+		ec.clear();
+
+		file_status s;
+		stat_file(inf, &s, ec);
+		if (ec) return;
+
+		if (has_parent_path(newf))
+		{
+			create_directories(parent_path(newf), ec);
+			if (ec) return;
+		}
+
+		rename(inf, newf, ec);
+
+		// on OSX, the error when trying to rename a file across different
+		// volumes is EXDEV, which will make it fall back to copying.
+
+		if (ec)
+		{
+			if (ec != boost::system::errc::no_such_file_or_directory
+				&& ec != boost::system::errc::invalid_argument
+				&& ec != boost::system::errc::permission_denied)
+			{
+				ec.clear();
+				copy_file(inf, newf, ec);
+
+				if (!ec)
+				{
+					// ignore errors when removing
+					error_code ignore;
+					remove(inf, ignore);
+				}
+			}
+		}
+	}
+
 	std::string split_path(std::string const& f)
 	{
 		if (f.empty()) return f;
@@ -879,7 +917,7 @@ namespace libtorrent
 				++len;
 			}
 			return std::string(first, len);
-			
+
 		}
 		return std::string(sep + 1);
 	}
@@ -1017,7 +1055,7 @@ namespace libtorrent
 		ret.resize(write_cur - &ret[0]);
 		return ret;
 	}
-#endif	
+#endif
 
 	boost::int64_t file_size(std::string const& f)
 	{
