@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2014, Arvid Norberg
+Copyright (c) 2007-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,84 +31,96 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/http_connection.hpp"
-#include "libtorrent/escape_string.hpp"
+#include "libtorrent/aux_/escape_string.hpp"
 #include "libtorrent/instantiate_connection.hpp"
 #include "libtorrent/gzip.hpp"
 #include "libtorrent/parse_url.hpp"
 #include "libtorrent/socket.hpp"
-#include "libtorrent/connection_queue.hpp"
 #include "libtorrent/socket_type.hpp" // for async_shutdown
+#include "libtorrent/resolver_interface.hpp"
+#include "libtorrent/settings_pack.hpp"
+#include "libtorrent/aux_/time.hpp"
+#include "libtorrent/random.hpp"
 
 #if defined TORRENT_ASIO_DEBUGGING
 #include "libtorrent/debug.hpp"
 #endif
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/bind.hpp>
 #include <string>
 #include <algorithm>
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 namespace libtorrent {
 
-http_connection::http_connection(io_service& ios, connection_queue& cc
+http_connection::http_connection(io_service& ios
+	, resolver_interface& resolver
 	, http_handler const& handler
 	, bool bottled
 	, int max_bottled_buffer_size
 	, http_connect_handler const& ch
 	, http_filter_handler const& fh
 #ifdef TORRENT_USE_OPENSSL
-	, boost::asio::ssl::context* ssl_ctx
+	, ssl::context* ssl_ctx
 #endif
 	)
-	: m_sock(ios)
-#if TORRENT_USE_I2P
-	, m_i2p_conn(0)
-#endif
-	, m_read_pos(0)
-	, m_resolver(ios)
-	, m_handler(handler)
-	, m_connect_handler(ch)
-	, m_filter_handler(fh)
-	, m_timer(ios)
-	, m_last_receive(time_now())
-	, m_start_time(time_now())
-	, m_bottled(bottled)
-	, m_max_bottled_buffer_size(max_bottled_buffer_size)
-	, m_called(false)
+	: m_next_ep(0)
+	, m_sock(ios)
 #ifdef TORRENT_USE_OPENSSL
 	, m_ssl_ctx(ssl_ctx)
 	, m_own_ssl_context(false)
 #endif
+#if TORRENT_USE_I2P
+	, m_i2p_conn(0)
+#endif
+	, m_resolver(resolver)
+	, m_handler(handler)
+	, m_connect_handler(ch)
+	, m_filter_handler(fh)
+	, m_timer(ios)
+	, m_read_timeout(seconds(5))
+	, m_completion_timeout(seconds(5))
+	, m_limiter_timer(ios)
+	, m_last_receive(aux::time_now())
+	, m_start_time(aux::time_now())
+	, m_read_pos(0)
+	, m_redirects(5)
+	, m_max_bottled_buffer_size(max_bottled_buffer_size)
 	, m_rate_limit(0)
 	, m_download_quota(0)
-	, m_limiter_timer_active(false)
-	, m_limiter_timer(ios)
-	, m_redirects(5)
-	, m_connection_ticket(-1)
-	, m_cc(cc)
-	, m_ssl(false)
 	, m_priority(0)
+	, m_resolve_flags(0)
+	, m_port(0)
+	, m_bottled(bottled)
+	, m_called(false)
+	, m_limiter_timer_active(false)
+	, m_ssl(false)
 	, m_abort(false)
+	, m_connecting(false)
 {
 	TORRENT_ASSERT(!m_handler.empty());
 }
 
 http_connection::~http_connection()
 {
-	TORRENT_ASSERT(m_connection_ticket == -1);
 #ifdef TORRENT_USE_OPENSSL
 	if (m_own_ssl_context) delete m_ssl_ctx;
 #endif
 }
 
 void http_connection::get(std::string const& url, time_duration timeout, int prio
-	, proxy_settings const* ps, int handle_redirects, std::string const& user_agent
-	, address const& bind_addr
+	, aux::proxy_settings const* ps, int handle_redirects, std::string const& user_agent
+	, address const& bind_addr, int resolve_flags, std::string const& auth_
 #if TORRENT_USE_I2P
 	, i2p_connection* i2p_conn
 #endif
 	)
 {
 	m_user_agent = user_agent;
+	m_resolve_flags = resolve_flags;
 
 	std::string protocol;
 	std::string auth;
@@ -120,6 +132,10 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	boost::tie(protocol, auth, hostname, port, path)
 		= parse_url_components(url, ec);
 
+	if (auth.empty()) auth = auth_;
+
+	m_auth = auth;
+
 	int default_port = protocol == "https" ? 443 : 80;
 	if (port == -1) port = default_port;
 
@@ -127,22 +143,22 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 	// deletes this object
 	boost::shared_ptr<http_connection> me(shared_from_this());
 
+	if (ec)
+	{
+		m_timer.get_io_service().post(boost::bind(&http_connection::callback
+			, me, ec, static_cast<char*>(NULL), 0));
+		return;
+	}
+
 	if (protocol != "http"
 #ifdef TORRENT_USE_OPENSSL
 		&& protocol != "https"
 #endif
 		)
 	{
-		error_code ec(errors::unsupported_url_protocol);
-		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-			, me, ec, (char*)0, 0));
-		return;
-	}
-
-	if (ec)
-	{
-		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-			, me, ec, (char*)0, 0));
+		error_code err(errors::unsupported_url_protocol);
+		m_timer.get_io_service().post(boost::bind(&http_connection::callback
+			, me, err, static_cast<char*>(NULL), 0));
 		return;
 	}
 
@@ -150,7 +166,7 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 
 	bool ssl = false;
 	if (protocol == "https") ssl = true;
-	
+
 	char request[4096];
 	char* end = request + sizeof(request);
 	char* ptr = request;
@@ -161,14 +177,14 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 
 	// exclude ssl here, because SSL assumes CONNECT support in the
 	// proxy and is handled at the lower layer
-	if (ps && (ps->type == proxy_settings::http
-		|| ps->type == proxy_settings::http_pw)
+	if (ps && (ps->type == settings_pack::http
+		|| ps->type == settings_pack::http_pw)
 		&& !ssl)
 	{
 		// if we're using an http proxy and not an ssl
 		// connection, just do a regular http proxy request
 		APPEND_FMT1("GET %s HTTP/1.1\r\n", url.c_str());
-		if (ps->type == proxy_settings::http_pw)
+		if (ps->type == settings_pack::http_pw)
 			APPEND_FMT1("Proxy-Authorization: Basic %s\r\n", base64encode(
 				ps->username + ":" + ps->password).c_str());
 
@@ -191,7 +207,7 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 
 	if (!m_user_agent.empty())
 		APPEND_FMT1("User-Agent: %s\r\n", m_user_agent.c_str());
-	
+
 	if (m_bottled)
 		APPEND_FMT("Accept-Encoding: gzip\r\n");
 
@@ -200,19 +216,21 @@ void http_connection::get(std::string const& url, time_duration timeout, int pri
 
 	APPEND_FMT("Connection: close\r\n\r\n");
 
-	sendbuffer.assign(request);
+	m_sendbuffer.assign(request);
 	m_url = url;
-	start(hostname, to_string(port).elems, timeout, prio
-		, ps, ssl, handle_redirects, bind_addr
+	start(hostname, port, timeout, prio
+		, ps, ssl, handle_redirects, bind_addr, m_resolve_flags
 #if TORRENT_USE_I2P
 		, i2p_conn
 #endif
 		);
 }
 
-void http_connection::start(std::string const& hostname, std::string const& port
-	, time_duration timeout, int prio, proxy_settings const* ps, bool ssl, int handle_redirects
+void http_connection::start(std::string const& hostname, int port
+	, time_duration timeout, int prio, aux::proxy_settings const* ps, bool ssl
+	, int handle_redirects
 	, address const& bind_addr
+	, int resolve_flags
 #if TORRENT_USE_I2P
 	, i2p_connection* i2p_conn
 #endif
@@ -221,6 +239,7 @@ void http_connection::start(std::string const& hostname, std::string const& port
 	TORRENT_ASSERT(prio >= 0 && prio < 3);
 
 	m_redirects = handle_redirects;
+	m_resolve_flags = resolve_flags;
 	if (ps) m_proxy = *ps;
 
 	// keep ourselves alive even if the callback function
@@ -228,9 +247,11 @@ void http_connection::start(std::string const& hostname, std::string const& port
 	boost::shared_ptr<http_connection> me(shared_from_this());
 
 	m_completion_timeout = timeout;
-	m_read_timeout = (std::max)(seconds(5), timeout / 5);
+	m_read_timeout = seconds(5);
+	if (m_read_timeout < timeout / 5) m_read_timeout = timeout / 5;
 	error_code ec;
-	m_timer.expires_from_now(m_completion_timeout, ec);
+	m_timer.expires_from_now((std::min)(
+		m_read_timeout, m_completion_timeout), ec);
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_timeout");
 #endif
@@ -244,8 +265,8 @@ void http_connection::start(std::string const& hostname, std::string const& port
 
 	if (ec)
 	{
-		m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-			, me, ec, (char*)0, 0));
+		m_timer.get_io_service().post(boost::bind(&http_connection::callback
+			, me, ec, static_cast<char*>(NULL), 0));
 		return;
 	}
 
@@ -255,15 +276,15 @@ void http_connection::start(std::string const& hostname, std::string const& port
 #if defined TORRENT_ASIO_DEBUGGING
 		add_outstanding_async("http_connection::on_write");
 #endif
-		async_write(m_sock, asio::buffer(sendbuffer)
+		async_write(m_sock, boost::asio::buffer(m_sendbuffer)
 			, boost::bind(&http_connection::on_write, me, _1));
 	}
 	else
 	{
 		m_ssl = ssl;
 		m_bind_addr = bind_addr;
-		error_code ec;
-		if (m_sock.is_open()) m_sock.close(ec);
+		error_code err;
+		if (m_sock.is_open()) m_sock.close(err);
 
 #if TORRENT_USE_I2P
 		bool is_i2p = false;
@@ -282,29 +303,34 @@ void http_connection::start(std::string const& hostname, std::string const& port
 #endif
 
 #if TORRENT_USE_I2P
-		if (is_i2p && i2p_conn->proxy().type != proxy_settings::i2p_proxy)
+		if (is_i2p && i2p_conn->proxy().type != settings_pack::i2p_proxy)
 		{
-			m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-				, me, error_code(errors::no_i2p_router, get_libtorrent_category()), (char*)0, 0));
+			m_timer.get_io_service().post(boost::bind(&http_connection::callback
+				, me, error_code(errors::no_i2p_router, get_libtorrent_category()), static_cast<char*>(NULL), 0));
 			return;
 		}
 #endif
 
-		proxy_settings const* proxy = ps;
+		aux::proxy_settings const* proxy = ps;
 #if TORRENT_USE_I2P
-		if (is_i2p) proxy = &i2p_conn->proxy();
+		aux::proxy_settings i2p_proxy;
+		if (is_i2p)
+		{
+			i2p_proxy = i2p_conn->proxy();
+			proxy = &i2p_proxy;
+		}
 #endif
 
 		// in this case, the upper layer is assumed to have taken
 		// care of the proxying already. Don't instantiate the socket
 		// with this proxy
-		if (proxy && (proxy->type == proxy_settings::http
-			|| proxy->type == proxy_settings::http_pw)
+		if (proxy && (proxy->type == settings_pack::http
+			|| proxy->type == settings_pack::http_pw)
 			&& !ssl)
 		{
 			proxy = 0;
 		}
-		proxy_settings null_proxy;
+		aux::proxy_settings null_proxy;
 
 		void* userdata = 0;
 #ifdef TORRENT_USE_OPENSSL
@@ -312,31 +338,37 @@ void http_connection::start(std::string const& hostname, std::string const& port
 		{
 			if (m_ssl_ctx == 0)
 			{
-				m_ssl_ctx = new (std::nothrow) boost::asio::ssl::context(
-					m_resolver.get_io_service(), asio::ssl::context::sslv23_client);
+				m_ssl_ctx = new (std::nothrow) ssl::context(
+					m_timer.get_io_service(), ssl::context::sslv23_client);
 				if (m_ssl_ctx)
 				{
 					m_own_ssl_context = true;
-					error_code ec;
-					m_ssl_ctx->set_verify_mode(asio::ssl::context::verify_none, ec);
-					TORRENT_ASSERT(!ec);
+					m_ssl_ctx->set_verify_mode(ssl::context::verify_none, ec);
+					if (ec)
+					{
+						m_timer.get_io_service().post(boost::bind(&http_connection::callback
+								, me, ec, static_cast<char*>(NULL), 0));
+						return;
+					}
 				}
 			}
 			userdata = m_ssl_ctx;
 		}
 #endif
-		instantiate_connection(m_resolver.get_io_service()
-			, proxy ? *proxy : null_proxy, m_sock, userdata);
+		// assume this is not a tracker connection. Tracker connections that
+		// shouldn't be subject to the proxy should pass in NULL as the proxy
+		// pointer.
+		instantiate_connection(m_timer.get_io_service()
+			, proxy ? *proxy : null_proxy, m_sock, userdata, NULL, false, false);
 
 		if (m_bind_addr != address_v4::any())
 		{
-			error_code ec;
 			m_sock.open(m_bind_addr.is_v4()?tcp::v4():tcp::v6(), ec);
 			m_sock.bind(tcp::endpoint(m_bind_addr, 0), ec);
 			if (ec)
 			{
-				m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-					, me, ec, (char*)0, 0));
+				m_timer.get_io_service().post(boost::bind(&http_connection::callback
+					, me, ec, static_cast<char*>(NULL), 0));
 				return;
 			}
 		}
@@ -344,39 +376,46 @@ void http_connection::start(std::string const& hostname, std::string const& port
 		setup_ssl_hostname(m_sock, hostname, ec);
 		if (ec)
 		{
-			m_resolver.get_io_service().post(boost::bind(&http_connection::callback
-				, me, ec, (char*)0, 0));
+			m_timer.get_io_service().post(boost::bind(&http_connection::callback
+				, me, ec, static_cast<char*>(NULL), 0));
 			return;
 		}
 
+		m_endpoints.clear();
+		m_next_ep = 0;
+
 #if TORRENT_USE_I2P
 		if (is_i2p)
 		{
+			if (hostname.length() < 516) // Base64 encoded  destination with optional .i2p
+			{
 #if defined TORRENT_ASIO_DEBUGGING
-			add_outstanding_async("http_connection::on_i2p_resolve");
+				add_outstanding_async("http_connection::on_i2p_resolve");
 #endif
-			i2p_conn->async_name_lookup(hostname.c_str(), boost::bind(&http_connection::on_i2p_resolve
-				, me, _1, _2));
+				i2p_conn->async_name_lookup(hostname.c_str(), boost::bind(&http_connection::on_i2p_resolve
+					, me, _1, _2));
+			}
+			else
+				connect_i2p_tracker(hostname.c_str());
 		}
 		else
 #endif
 		if (ps && ps->proxy_hostnames
-			&& (ps->type == proxy_settings::socks5
-				|| ps->type == proxy_settings::socks5_pw))
+			&& (ps->type == settings_pack::socks5
+				|| ps->type == settings_pack::socks5_pw))
 		{
 			m_hostname = hostname;
 			m_port = port;
-			m_endpoints.push_back(tcp::endpoint(address(), atoi(port.c_str())));
-			queue_connect();
+			m_endpoints.push_back(tcp::endpoint(address(), port));
+			connect();
 		}
 		else
 		{
 #if defined TORRENT_ASIO_DEBUGGING
 			add_outstanding_async("http_connection::on_resolve");
 #endif
-			m_endpoints.clear();
-			tcp::resolver::query query(hostname, port);
-			m_resolver.async_resolve(query, boost::bind(&http_connection::on_resolve
+			m_resolver.async_resolve(hostname, m_resolve_flags
+				, boost::bind(&http_connection::on_resolve
 				, me, _1, _2));
 		}
 		m_hostname = hostname;
@@ -384,18 +423,6 @@ void http_connection::start(std::string const& hostname, std::string const& port
 	}
 }
 
-void http_connection::on_connect_timeout()
-{
-	TORRENT_ASSERT(m_connection_ticket > -1);
-
-	// keep ourselves alive even if the callback function
-	// deletes this object
-	boost::shared_ptr<http_connection> me(shared_from_this());
-
-	error_code ec;
-	m_sock.close(ec);
-}
-
 void http_connection::on_timeout(boost::weak_ptr<http_connection> p
 	, error_code const& e)
 {
@@ -405,36 +432,35 @@ void http_connection::on_timeout(boost::weak_ptr<http_connection> p
 	boost::shared_ptr<http_connection> c = p.lock();
 	if (!c) return;
 
-	if (e == asio::error::operation_aborted) return;
+	if (e == boost::asio::error::operation_aborted) return;
 
 	if (c->m_abort) return;
 
-	ptime now = time_now_hires();
+	time_point now = clock_type::now();
 
-	if (c->m_start_time + c->m_completion_timeout < now
-		|| c->m_last_receive + c->m_read_timeout < now)
+	if (c->m_start_time + c->m_completion_timeout <= now
+		|| c->m_last_receive + c->m_read_timeout <= now)
 	{
-		if (c->m_connection_ticket > -1 && !c->m_endpoints.empty())
+		// the connection timed out. If we have more endpoints to try, just
+		// close this connection. The on_connect handler will try the next
+		// endpoint in the list.
+		if (c->m_next_ep < c->m_endpoints.size())
 		{
-#if defined TORRENT_ASIO_DEBUGGING
-			add_outstanding_async("http_connection::on_timeout");
-#endif
 			error_code ec;
-			async_shutdown(c->m_sock, c);
-			c->m_timer.expires_at((std::min)(
-				c->m_last_receive + c->m_read_timeout
-				, c->m_start_time + c->m_completion_timeout), ec);
-			c->m_timer.async_wait(boost::bind(&http_connection::on_timeout, p, _1));
+			c->m_sock.close(ec);
+			if (!c->m_connecting) c->connect();
 		}
 		else
 		{
-			c->callback(asio::error::timed_out);
-			c->close(true);
+			c->callback(boost::asio::error::timed_out);
 		}
 		return;
 	}
+	else
+	{
+		if (!c->m_sock.is_open()) return;
+	}
 
-	if (!c->m_sock.is_open()) return;
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_timeout");
 #endif
@@ -450,35 +476,23 @@ void http_connection::close(bool force)
 	if (m_abort) return;
 
 	error_code ec;
-	m_timer.cancel(ec);
-	m_resolver.cancel();
-	m_limiter_timer.cancel(ec);
-
 	if (force)
 		m_sock.close(ec);
 	else
 		async_shutdown(m_sock, shared_from_this());
 
+	m_timer.cancel(ec);
+	m_limiter_timer.cancel(ec);
+
 	m_hostname.clear();
-	m_port.clear();
+	m_port = 0;
 	m_handler.clear();
 	m_abort = true;
 }
 
 #if TORRENT_USE_I2P
-void http_connection::on_i2p_resolve(error_code const& e
-	, char const* destination)
+void http_connection::connect_i2p_tracker(char const* destination)
 {
-#if defined TORRENT_ASIO_DEBUGGING
-	complete_async("http_connection::on_i2p_resolve");
-#endif
-	if (e)
-	{
-		callback(e);
-		close();
-		return;
-	}
-
 #ifdef TORRENT_USE_OPENSSL
 	TORRENT_ASSERT(m_ssl == false);
 	TORRENT_ASSERT(m_sock.get<socket_type>());
@@ -497,26 +511,38 @@ void http_connection::on_i2p_resolve(error_code const& e
 	m_sock.async_connect(tcp::endpoint(), boost::bind(&http_connection::on_connect
 		, shared_from_this(), _1));
 }
+
+void http_connection::on_i2p_resolve(error_code const& e
+	, char const* destination)
+{
+#if defined TORRENT_ASIO_DEBUGGING
+	complete_async("http_connection::on_i2p_resolve");
+#endif
+	if (e)
+	{
+		callback(e);
+		return;
+	}
+	connect_i2p_tracker(destination);
+}
 #endif
 
 void http_connection::on_resolve(error_code const& e
-	, tcp::resolver::iterator i)
+	, std::vector<address> const& addresses)
 {
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("http_connection::on_resolve");
 #endif
 	if (e)
 	{
-		boost::shared_ptr<http_connection> me(shared_from_this());
-
 		callback(e);
-		close();
 		return;
 	}
-	TORRENT_ASSERT(i != tcp::resolver::iterator());
+	TORRENT_ASSERT(!addresses.empty());
 
-	std::transform(i, tcp::resolver::iterator(), std::back_inserter(m_endpoints)
-		, boost::bind(&tcp::resolver::iterator::value_type::endpoint, _1));
+	for (std::vector<address>::const_iterator i = addresses.begin()
+		, end(addresses.end()); i != end; ++i)
+		m_endpoints.push_back(tcp::endpoint(*i, m_port));
 
 	if (m_filter_handler) m_filter_handler(*this, m_endpoints);
 	if (m_endpoints.empty())
@@ -525,6 +551,8 @@ void http_connection::on_resolve(error_code const& e
 		return;
 	}
 
+	std::random_shuffle(m_endpoints.begin(), m_endpoints.end(), randint);
+
 	// The following statement causes msvc to crash (ICE). Since it's not
 	// necessary in the vast majority of cases, just ignore the endpoint
 	// order for windows
@@ -538,51 +566,59 @@ void http_connection::on_resolve(error_code const& e
 				== m_bind_addr.is_v4());
 #endif
 
-	queue_connect();
+	connect();
 }
 
-void http_connection::queue_connect()
+void http_connection::connect()
 {
-	TORRENT_ASSERT(!m_endpoints.empty());
-	tcp::endpoint target = m_endpoints.front();
-	m_endpoints.pop_front();
+	TORRENT_ASSERT(m_next_ep < m_endpoints.size());
 
-	m_cc.enqueue(boost::bind(&http_connection::connect, shared_from_this(), _1, target)
-		, boost::bind(&http_connection::on_connect_timeout, shared_from_this())
-		, m_read_timeout, m_priority);
-}
+	boost::shared_ptr<http_connection> me(shared_from_this());
 
-void http_connection::connect(int ticket, tcp::endpoint target_address)
-{
-	if (ticket == -1)
-	{
-		close();
-		return;
-	}
-
-	m_connection_ticket = ticket;
 	if (m_proxy.proxy_hostnames
-		&& (m_proxy.type == proxy_settings::socks5
-			|| m_proxy.type == proxy_settings::socks5_pw))
+		&& (m_proxy.type == settings_pack::socks5
+			|| m_proxy.type == settings_pack::socks5_pw))
 	{
-		// we're using a socks proxy and we're resolving
-		// hostnames through it
-#ifdef TORRENT_USE_OPENSSL
-		if (m_ssl)
+		// test to see if m_hostname really just is an IP (and not a hostname). If it
+		// is, ec will be represent "success". If so, don't set it as the socks5
+		// hostname, just connect to the IP
+		error_code ec;
+		address adr = address::from_string(m_hostname, ec);
+
+		if (ec)
 		{
-			TORRENT_ASSERT(m_sock.get<ssl_stream<socks5_stream> >());
-			m_sock.get<ssl_stream<socks5_stream> >()->next_layer().set_dst_name(m_hostname);
+			// we're using a socks proxy and we're resolving
+			// hostnames through it
+#ifdef TORRENT_USE_OPENSSL
+			if (m_ssl)
+			{
+				TORRENT_ASSERT(m_sock.get<ssl_stream<socks5_stream> >());
+				m_sock.get<ssl_stream<socks5_stream> >()->next_layer().set_dst_name(m_hostname);
+			}
+			else
+#endif
+			{
+				TORRENT_ASSERT(m_sock.get<socks5_stream>());
+				m_sock.get<socks5_stream>()->set_dst_name(m_hostname);
+			}
 		}
 		else
-#endif
 		{
-			TORRENT_ASSERT(m_sock.get<socks5_stream>());
-			m_sock.get<socks5_stream>()->set_dst_name(m_hostname);
+			m_endpoints[0].address(adr);
 		}
 	}
+
+	TORRENT_ASSERT(m_next_ep < m_endpoints.size());
+	if (m_next_ep >= m_endpoints.size()) return;
+
+	tcp::endpoint target_address = m_endpoints[m_next_ep];
+	++m_next_ep;
+
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_connect");
 #endif
+	TORRENT_ASSERT(!m_connecting);
+	m_connecting = true;
 	m_sock.async_connect(target_address, boost::bind(&http_connection::on_connect
 		, shared_from_this(), _1));
 }
@@ -592,35 +628,30 @@ void http_connection::on_connect(error_code const& e)
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("http_connection::on_connect");
 #endif
-	if (m_connection_ticket >= 0)
-	{
-		m_cc.done(m_connection_ticket);
-		m_connection_ticket = -1;
-	}
+	TORRENT_ASSERT(m_connecting);
+	m_connecting = false;
 
-	m_last_receive = time_now_hires();
+	m_last_receive = clock_type::now();
 	m_start_time = m_last_receive;
 	if (!e)
-	{ 
+	{
 		if (m_connect_handler) m_connect_handler(*this);
 #if defined TORRENT_ASIO_DEBUGGING
 		add_outstanding_async("http_connection::on_write");
 #endif
-		async_write(m_sock, asio::buffer(sendbuffer)
+		async_write(m_sock, boost::asio::buffer(m_sendbuffer)
 			, boost::bind(&http_connection::on_write, shared_from_this(), _1));
 	}
-	else if (!m_endpoints.empty() && !m_abort)
+	else if (m_next_ep < m_endpoints.size() && !m_abort)
 	{
 		// The connection failed. Try the next endpoint in the list.
 		error_code ec;
 		m_sock.close(ec);
-		queue_connect();
-	} 
+		connect();
+	}
 	else
-	{ 
-		boost::shared_ptr<http_connection> me(shared_from_this());
+	{
 		callback(e);
-		close();
 	}
 }
 
@@ -631,7 +662,7 @@ void http_connection::callback(error_code e, char* data, int size)
 	std::vector<char> buf;
 	if (data && m_bottled && m_parser.header_finished())
 	{
-		size = m_parser.collapse_chunk_headers((char*)data, size);
+		size = m_parser.collapse_chunk_headers(data, size);
 
 		std::string const& encoding = m_parser.header("content-encoding");
 		if ((encoding == "gzip" || encoding == "x-gzip") && size > 0 && data)
@@ -642,7 +673,6 @@ void http_connection::callback(error_code e, char* data, int size)
 			if (ec)
 			{
 				if (m_handler) m_handler(ec, m_parser, data, size, *this);
-				close();
 				return;
 			}
 			size = int(buf.size());
@@ -666,19 +696,17 @@ void http_connection::on_write(error_code const& e)
 	complete_async("http_connection::on_write");
 #endif
 
-	if (e == asio::error::operation_aborted) return;
+	if (e == boost::asio::error::operation_aborted) return;
 
 	if (e)
 	{
-		boost::shared_ptr<http_connection> me(shared_from_this());
 		callback(e);
-		close();
 		return;
 	}
 
 	if (m_abort) return;
 
-	std::string().swap(sendbuffer);
+	std::string().swap(m_sendbuffer);
 	m_recvbuffer.resize(4096);
 
 	int amount_to_read = m_recvbuffer.size() - m_read_pos;
@@ -700,7 +728,7 @@ void http_connection::on_write(error_code const& e)
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_read");
 #endif
-	m_sock.async_read_some(asio::buffer(&m_recvbuffer[0] + m_read_pos
+	m_sock.async_read_some(boost::asio::buffer(&m_recvbuffer[0] + m_read_pos
 		, amount_to_read)
 		, boost::bind(&http_connection::on_read
 			, shared_from_this(), _1, _2));
@@ -719,7 +747,7 @@ void http_connection::on_read(error_code const& e
 		TORRENT_ASSERT(m_download_quota >= 0);
 	}
 
-	if (e == asio::error::operation_aborted) return;
+	if (e == boost::asio::error::operation_aborted) return;
 
 	if (m_abort) return;
 
@@ -729,9 +757,9 @@ void http_connection::on_read(error_code const& e
 
 	// when using the asio SSL wrapper, it seems like
 	// we get the shut_down error instead of EOF
-	if (e == asio::error::eof || e == asio::error::shut_down)
+	if (e == boost::asio::error::eof || e == boost::asio::error::shut_down)
 	{
-		error_code ec = asio::error::eof;
+		error_code ec = boost::asio::error::eof;
 		TORRENT_ASSERT(bytes_transferred == 0);
 		char* data = 0;
 		std::size_t size = 0;
@@ -741,7 +769,6 @@ void http_connection::on_read(error_code const& e
 			size = m_parser.get_body().left();
 		}
 		callback(ec, data, size);
-		close();
 		return;
 	}
 
@@ -749,7 +776,6 @@ void http_connection::on_read(error_code const& e
 	{
 		TORRENT_ASSERT(bytes_transferred == 0);
 		callback(e);
-		close();
 		return;
 	}
 
@@ -783,7 +809,6 @@ void http_connection::on_read(error_code const& e
 				{
 					// missing location header
 					callback(error_code(errors::http_missing_location));
-					close();
 					return;
 				}
 
@@ -796,14 +821,14 @@ void http_connection::on_read(error_code const& e
 
 				std::string url = resolve_redirect_location(m_url, location);
 				get(url, m_completion_timeout, m_priority, &m_proxy, m_redirects - 1
-					, m_user_agent, m_bind_addr
+					, m_user_agent, m_bind_addr, m_resolve_flags, m_auth
 #if TORRENT_USE_I2P
 					, m_i2p_conn
 #endif
 					);
 				return;
 			}
-	
+
 			m_redirects = 0;
 		}
 
@@ -813,7 +838,7 @@ void http_connection::on_read(error_code const& e
 				callback(e, &m_recvbuffer[0] + m_parser.body_start()
 					, m_read_pos - m_parser.body_start());
 			m_read_pos = 0;
-			m_last_receive = time_now_hires();
+			m_last_receive = clock_type::now();
 		}
 		else if (m_bottled && m_parser.finished())
 		{
@@ -827,7 +852,7 @@ void http_connection::on_read(error_code const& e
 		TORRENT_ASSERT(!m_bottled);
 		callback(e, &m_recvbuffer[0], m_read_pos);
 		m_read_pos = 0;
-		m_last_receive = time_now_hires();
+		m_last_receive = clock_type::now();
 	}
 
 	// if we've hit the limit, double the buffer size
@@ -839,7 +864,6 @@ void http_connection::on_read(error_code const& e
 		// if we've reached the size limit, terminate the connection and
 		// report the error
 		callback(error_code(boost::system::errc::file_too_large, generic_category()));
-		close();
 		return;
 	}
 	int amount_to_read = m_recvbuffer.size() - m_read_pos;
@@ -861,7 +885,7 @@ void http_connection::on_read(error_code const& e
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_read");
 #endif
-	m_sock.async_read_some(asio::buffer(&m_recvbuffer[0] + m_read_pos
+	m_sock.async_read_some(boost::asio::buffer(&m_recvbuffer[0] + m_read_pos
 		, amount_to_read)
 		, boost::bind(&http_connection::on_read
 			, me, _1, _2));
@@ -872,11 +896,11 @@ void http_connection::on_assign_bandwidth(error_code const& e)
 #if defined TORRENT_ASIO_DEBUGGING
 	complete_async("http_connection::on_assign_bandwidth");
 #endif
-	if ((e == asio::error::operation_aborted
+	if ((e == boost::asio::error::operation_aborted
 		&& m_limiter_timer_active)
 		|| !m_sock.is_open())
 	{
-		callback(asio::error::eof);
+		callback(boost::asio::error::eof);
 		return;
 	}
 	m_limiter_timer_active = false;
@@ -895,7 +919,7 @@ void http_connection::on_assign_bandwidth(error_code const& e)
 #if defined TORRENT_ASIO_DEBUGGING
 	add_outstanding_async("http_connection::on_read");
 #endif
-	m_sock.async_read_some(asio::buffer(&m_recvbuffer[0] + m_read_pos
+	m_sock.async_read_some(boost::asio::buffer(&m_recvbuffer[0] + m_read_pos
 		, amount_to_read)
 		, boost::bind(&http_connection::on_read
 			, shared_from_this(), _1, _2));
