@@ -78,7 +78,8 @@ void get_item::got_data(bdecode_node const& v,
 	// data can reach here, which means pk and sig must be valid.
 	if (!pk || !sig) return;
 
-	std::pair<char const*, int> salt(m_salt.c_str(), int(m_salt.size()));
+    std::string temp_copy(m_data.salt());
+	std::pair<char const*, int> salt(temp_copy.c_str(), int(temp_copy.size()));
 	sha1_hash incoming_target = item_target_id(salt, pk);
 	if (incoming_target != m_target) return;
 
