@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2008-2014, Arvid Norberg
+Copyright (c) 2008-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,8 +30,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
+#ifndef TORRENT_NO_DEPRECATE
+
 #include "libtorrent/config.hpp"
 #include "libtorrent/lazy_entry.hpp"
+#include "libtorrent/bdecode.hpp" // for error codes
 #include <cstring>
 #include <limits> // for numeric_limits
 
@@ -64,45 +67,10 @@ namespace libtorrent
 			if (error_pos) *error_pos = start - orig_start;
 			return -1;
 		}
-	}
 
-#define TORRENT_FAIL_BDECODE(code) do { ec = make_error_code(code); return fail(error_pos, stack, start, orig_start); } while (false)
+#define TORRENT_FAIL_BDECODE(code) do { ec = make_error_code(code); return fail(error_pos, stack, start, orig_start); } TORRENT_WHILE_0
 
-	namespace { bool numeric(char c) { return c >= '0' && c <= '9'; } }
-
-	// fills in 'val' with what the string between start and the
-	// first occurance of the delimiter is interpreted as an int.
-	// return the pointer to the delimiter, or 0 if there is a
-	// parse error. val should be initialized to zero
-	char const* parse_int(char const* start, char const* end, char delimiter
-		, boost::int64_t& val, bdecode_errors::error_code_enum& ec)
-	{
-		while (start < end && *start != delimiter)
-		{
-			if (!numeric(*start))
-			{
-				ec = bdecode_errors::expected_string;
-				return start;
-			}
-			if (val > (std::numeric_limits<boost::int64_t>::max)() / 10)
-			{
-				ec = bdecode_errors::overflow;
-				return start;
-			}
-			val *= 10;
-			int digit = *start - '0';
-			if (val > (std::numeric_limits<boost::int64_t>::max)() - digit)
-			{
-				ec = bdecode_errors::overflow;
-				return start;
-			}
-			val += digit;
-			++start;
-		}
-		if (*start != delimiter)
-			ec = bdecode_errors::expected_colon;
-		return start;
-	}
+	bool numeric(char c) { return c >= '0' && c <= '9'; }
 
 	char const* find_char(char const* start, char const* end, char delimiter)
 	{
@@ -110,6 +78,8 @@ namespace libtorrent
 		return start;
 	}
 
+	} // anonymous namespace
+
 #ifndef TORRENT_NO_DEPRECATE
 	int lazy_bdecode(char const* start, char const* end
 		, lazy_entry& ret, int depth_limit, int item_limit)
@@ -129,7 +99,8 @@ namespace libtorrent
 
 		std::vector<lazy_entry*> stack;
 
-		if (start == end) TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
+		if (start == end)
+			TORRENT_FAIL_BDECODE(bdecode_errors::unexpected_eof);
 
 		stack.push_back(&ret);
 		while (start <= end)
@@ -154,7 +125,7 @@ namespace libtorrent
 						stack.pop_back();
 						continue;
 					}
-					if (!numeric(t)) TORRENT_FAIL_BDECODE(bdecode_errors::expected_string);
+					if (!numeric(t)) TORRENT_FAIL_BDECODE(bdecode_errors::expected_digit);
 					boost::int64_t len = t - '0';
 					bdecode_errors::error_code_enum e = bdecode_errors::no_error;
 					start = parse_int(start, end, ':', len, e);
@@ -193,7 +164,10 @@ namespace libtorrent
 					stack.push_back(ent);
 					break;
 				}
-				default: break;
+				case lazy_entry::int_t:
+				case lazy_entry::string_t:
+				case lazy_entry::none_t:
+					break;
 			}
 
 			--item_limit;
@@ -204,10 +178,10 @@ namespace libtorrent
 			{
 				case 'd':
 					top->construct_dict(start - 1);
-					continue;
+					break;
 				case 'l':
 					top->construct_list(start - 1);
-					continue;
+					break;
 				case 'i':
 				{
 					char const* int_start = start;
@@ -217,7 +191,7 @@ namespace libtorrent
 					TORRENT_ASSERT(*start == 'e');
 					++start;
 					stack.pop_back();
-					continue;
+					break;
 				}
 				default:
 				{
@@ -242,14 +216,23 @@ namespace libtorrent
 					top->construct_string(start, int(len));
 					start += len;
 					stack.pop_back();
-					continue;
+					break;
 				}
 			}
-			return 0;
 		}
 		return 0;
 	}
 
+	int lazy_entry::capacity() const
+	{
+		TORRENT_ASSERT(m_type == dict_t || m_type == list_t);
+		if (m_data.list == NULL) return 0;
+		if (m_type == dict_t)
+			return m_data.dict[0].val.m_len;
+		else
+			return m_data.list[0].m_len;
+	}
+
 	boost::int64_t lazy_entry::int_value() const
 	{
 		TORRENT_ASSERT(m_type == int_t);
@@ -267,28 +250,29 @@ namespace libtorrent
 	lazy_entry* lazy_entry::dict_append(char const* name)
 	{
 		TORRENT_ASSERT(m_type == dict_t);
-		TORRENT_ASSERT(m_size <= m_capacity);
-		if (m_capacity == 0)
+		TORRENT_ASSERT(m_size <= this->capacity());
+		if (m_data.dict == NULL)
 		{
 			int capacity = lazy_entry_dict_init;
-			m_data.dict = new (std::nothrow) lazy_dict_entry[capacity];
-			if (m_data.dict == 0) return 0;
-			m_capacity = capacity;
+			m_data.dict = new (std::nothrow) lazy_dict_entry[capacity+1];
+			if (m_data.dict == NULL) return NULL;
+			m_data.dict[0].val.m_len = capacity;
 		}
-		else if (m_size == m_capacity)
+		else if (m_size == this->capacity())
 		{
-			int capacity = m_capacity * lazy_entry_grow_factor / 100;
-			lazy_dict_entry* tmp = new (std::nothrow) lazy_dict_entry[capacity];
-			if (tmp == 0) return 0;
-			std::memcpy(tmp, m_data.dict, sizeof(lazy_dict_entry) * m_size);
-			for (int i = 0; i < int(m_size); ++i) m_data.dict[i].val.release();
+			int capacity = this->capacity() * lazy_entry_grow_factor / 100;
+			lazy_dict_entry* tmp = new (std::nothrow) lazy_dict_entry[capacity+1];
+			if (tmp == NULL) return NULL;
+			std::memcpy(tmp, m_data.dict, sizeof(lazy_dict_entry) * (m_size + 1));
+			for (int i = 0; i < int(m_size); ++i) m_data.dict[i+1].val.release();
+
 			delete[] m_data.dict;
 			m_data.dict = tmp;
-			m_capacity = capacity;
+			m_data.dict[0].val.m_len = capacity;
 		}
 
-		TORRENT_ASSERT(m_size < m_capacity);
-		lazy_dict_entry& ret = m_data.dict[m_size++];
+		TORRENT_ASSERT(m_size < this->capacity());
+		lazy_dict_entry& ret = m_data.dict[1+m_size++];
 		ret.name = name;
 		return &ret.val;
 	}
@@ -346,7 +330,7 @@ namespace libtorrent
 	{
 		TORRENT_ASSERT(m_type == dict_t);
 		TORRENT_ASSERT(i < int(m_size));
-		lazy_dict_entry const& e = m_data.dict[i];
+		lazy_dict_entry const& e = m_data.dict[i+1];
 		return std::make_pair(std::string(e.name, e.val.m_begin - e.name), &e.val);
 	}
 
@@ -378,7 +362,8 @@ namespace libtorrent
 		return e;
 	}
 
-	boost::int64_t lazy_entry::dict_find_int_value(char const* name, boost::int64_t default_val) const
+	boost::int64_t lazy_entry::dict_find_int_value(char const* name
+		, boost::int64_t default_val) const
 	{
 		lazy_entry const* e = dict_find(name);
 		if (e == 0 || e->type() != lazy_entry::int_t) return default_val;
@@ -411,7 +396,7 @@ namespace libtorrent
 		TORRENT_ASSERT(m_type == dict_t);
 		for (int i = 0; i < int(m_size); ++i)
 		{
-			lazy_dict_entry& e = m_data.dict[i];
+			lazy_dict_entry& e = m_data.dict[i+1];
 			if (string_equal(name, e.name, e.val.m_begin - e.name))
 				return &e.val;
 		}
@@ -423,7 +408,7 @@ namespace libtorrent
 		TORRENT_ASSERT(m_type == dict_t);
 		for (int i = 0; i < int(m_size); ++i)
 		{
-			lazy_dict_entry& e = m_data.dict[i];
+			lazy_dict_entry& e = m_data.dict[i+1];
 			if (name.size() != e.val.m_begin - e.name) continue;
 			if (std::equal(name.begin(), name.end(), e.name))
 				return &e.val;
@@ -434,28 +419,29 @@ namespace libtorrent
 	lazy_entry* lazy_entry::list_append()
 	{
 		TORRENT_ASSERT(m_type == list_t);
-		TORRENT_ASSERT(m_size <= m_capacity);
-		if (m_capacity == 0)
+		TORRENT_ASSERT(m_size <= this->capacity());
+		if (m_data.start == NULL)
 		{
 			int capacity = lazy_entry_list_init;
-			m_data.list = new (std::nothrow) lazy_entry[capacity];
+			m_data.list = new (std::nothrow) lazy_entry[capacity+1];
 			if (m_data.list == 0) return 0;
-			m_capacity = capacity;
+			m_data.list[0].m_len = capacity;
 		}
-		else if (m_size == m_capacity)
+		else if (m_size == this->capacity())
 		{
-			int capacity = m_capacity * lazy_entry_grow_factor / 100;
-			lazy_entry* tmp = new (std::nothrow) lazy_entry[capacity];
-			if (tmp == 0) return 0;
-			std::memcpy(tmp, m_data.list, sizeof(lazy_entry) * m_size);
-			for (int i = 0; i < int(m_size); ++i) m_data.list[i].release();
+			int capacity = this->capacity() * lazy_entry_grow_factor / 100;
+			lazy_entry* tmp = new (std::nothrow) lazy_entry[capacity+1];
+			if (tmp == NULL) return NULL;
+			std::memcpy(tmp, m_data.list, sizeof(lazy_entry) * (m_size+1));
+			for (int i = 0; i < int(m_size); ++i) m_data.list[i+1].release();
+
 			delete[] m_data.list;
 			m_data.list = tmp;
-			m_capacity = capacity;
+			m_data.list[0].m_len = capacity;
 		}
 
-		TORRENT_ASSERT(m_size < m_capacity);
-		return m_data.list + (m_size++);
+		TORRENT_ASSERT(m_size < this->capacity());
+		return &m_data.list[1 + (m_size++)];
 	}
 
 	std::string lazy_entry::list_string_value_at(int i) const
@@ -483,13 +469,16 @@ namespace libtorrent
 	{
 		switch (m_type)
 		{
-			case list_t: delete[] m_data.list; break;
-			case dict_t: delete[] m_data.dict; break;
+			case list_t:
+				delete[] m_data.list;
+				break;
+			case dict_t:
+				delete[] m_data.dict;
+				break;
 			default: break;
 		}
-		m_data.start = 0;
+		m_data.start = NULL;
 		m_size = 0;
-		m_capacity = 0;
 		m_type = none_t;
 	}
 
@@ -499,6 +488,8 @@ namespace libtorrent
 		return return_t(m_begin, m_len);
 	}
 
+	namespace {
+
 	int line_longer_than(lazy_entry const& e, int limit)
 	{
 		int line_len = 0;
@@ -560,7 +551,7 @@ namespace libtorrent
 			else
 			{
 				char tmp[5];
-				snprintf(tmp, sizeof(tmp), "\\x%02x", (unsigned char)str[i]);
+				snprintf(tmp, sizeof(tmp), "\\x%02x", boost::uint8_t(str[i]));
 				ret += tmp;
 			}
 		}
@@ -602,6 +593,7 @@ namespace libtorrent
 		}
 		ret += "'";
 	}
+	} // anonymous namespace
 
 	std::string print_entry(lazy_entry const& e, bool single_line, int indent)
 	{
@@ -664,50 +656,7 @@ namespace libtorrent
 		}
 		return ret;
 	}
+}
 
-	struct bdecode_error_category : boost::system::error_category
-	{
-		virtual const char* name() const BOOST_SYSTEM_NOEXCEPT;
-		virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT;
-		virtual boost::system::error_condition default_error_condition(int ev) const BOOST_SYSTEM_NOEXCEPT
-		{ return boost::system::error_condition(ev, *this); }
-	};
-
-	const char* bdecode_error_category::name() const BOOST_SYSTEM_NOEXCEPT
-	{
-		return "bdecode error";
-	}
-
-	std::string bdecode_error_category::message(int ev) const BOOST_SYSTEM_NOEXCEPT
-	{
-		static char const* msgs[] =
-		{
-			"no error",
-			"expected string in bencoded string",
-			"expected colon in bencoded string",
-			"unexpected end of file in bencoded string",
-			"expected value (list, dict, int or string) in bencoded string",
-			"bencoded nesting depth exceeded",
-			"bencoded item count limit exceeded",
-			"integer overflow",
-		};
-		if (ev < 0 || ev >= int(sizeof(msgs)/sizeof(msgs[0])))
-			return "Unknown error";
-		return msgs[ev];
-	}
-
-	boost::system::error_category& get_bdecode_category()
-	{
-		static bdecode_error_category bdecode_category;
-		return bdecode_category;
-	}
-
-	namespace bdecode_errors
-	{
-		boost::system::error_code make_error_code(error_code_enum e)
-		{
-			return boost::system::error_code(e, get_bdecode_category());
-		}
-	}
-};
+#endif // TORRENT_NO_DEPRECATE
 
