@@ -137,7 +137,8 @@ namespace libtorrent
 		SET_NOPREV(proxy_username, "", &session_impl::update_proxy),
 		SET_NOPREV(proxy_password, "", &session_impl::update_proxy),
 		SET_NOPREV(i2p_hostname, "", &session_impl::update_i2p_bridge),
-		SET_NOPREV(peer_fingerprint, "-LT1100-", &session_impl::update_peer_fingerprint)
+		SET_NOPREV(peer_fingerprint, "-LT1110-", &session_impl::update_peer_fingerprint),
+		SET_NOPREV(dht_bootstrap_nodes, "dht.libtorrent.org:25401", &session_impl::update_dht_bootstrap_nodes)
 	};
 
 	bool_setting_entry_t bool_settings[settings_pack::num_bool_settings] =
@@ -597,6 +598,9 @@ namespace libtorrent
 			if (index < 0 || index >= settings_pack::num_string_settings)
 				continue;
 
+			// if the vaue did not change, don't call the update callback
+			if (sett.get_str(i->first) == i->second) continue;
+
 			sett.set_str(i->first, i->second);
 			str_setting_entry_t const& sa = str_settings[i->first & settings_pack::index_mask];
 			if (sa.fun && ses
@@ -616,6 +620,9 @@ namespace libtorrent
 			if (index < 0 || index >= settings_pack::num_int_settings)
 				continue;
 
+			// if the vaue did not change, don't call the update callback
+			if (sett.get_int(i->first) == i->second) continue;
+
 			sett.set_int(i->first, i->second);
 			int_setting_entry_t const& sa = int_settings[i->first & settings_pack::index_mask];
 			if (sa.fun && ses
@@ -635,6 +642,9 @@ namespace libtorrent
 			if (index < 0 || index >= settings_pack::num_bool_settings)
 				continue;
 
+			// if the vaue did not change, don't call the update callback
+			if (sett.get_bool(i->first) == i->second) continue;
+
 			sett.set_bool(i->first, i->second);
 			bool_setting_entry_t const& sa = bool_settings[i->first & settings_pack::index_mask];
 			if (sa.fun && ses
