@@ -230,9 +230,7 @@ namespace libtorrent
 			, m_flags(flags)
 		{}
 
-		int file_op(int const file_index
-			, boost::int64_t const file_offset
-			, int const size
+		int file_op(int file_index, boost::int64_t file_offset, int size
 			, file::iovec_t const* bufs, storage_error& ec)
 			TORRENT_OVERRIDE TORRENT_FINAL
 		{
@@ -321,9 +319,7 @@ namespace libtorrent
 			, m_flags(flags)
 		{}
 
-		int file_op(int const file_index
-			, boost::int64_t const file_offset
-			, int const size
+		int file_op(int file_index, boost::int64_t file_offset, int size
 			, file::iovec_t const* bufs, storage_error& ec)
 			TORRENT_OVERRIDE TORRENT_FINAL
 		{
@@ -799,8 +795,8 @@ namespace libtorrent
 			typedef std::set<std::string>::iterator iter_t;
 			for (int i = 0; i < files().num_files(); ++i)
 			{
-				std::string fp = files().file_path(i);
-				bool complete = is_complete(fp);
+				std::string const fp = files().file_path(i);
+				bool const complete = files().file_absolute_path(i);
 				std::string p = complete ? fp : combine_path(m_save_path, fp);
 				if (!complete)
 				{
@@ -965,7 +961,7 @@ namespace libtorrent
 		bdecode_node slots = rd.dict_find_list("slots");
 		if (slots)
 		{
-			if (int(slots.list_size()) == m_files.num_pieces())
+			if (slots.list_size() == m_files.num_pieces())
 			{
 				seed = true;
 				for (int i = 0; i < slots.list_size(); ++i)
@@ -978,7 +974,7 @@ namespace libtorrent
 		}
 		else if (bdecode_node pieces = rd.dict_find_string("pieces"))
 		{
-			if (int(pieces.string_length()) == m_files.num_pieces())
+			if (pieces.string_length() == m_files.num_pieces())
 			{
 				seed = true;
 				char const* p = pieces.string_ptr();
@@ -1128,32 +1124,32 @@ namespace libtorrent
 		return true;
 	}
 
-	int default_storage::move_storage(std::string const& sp, int flags, storage_error& ec)
+	int default_storage::move_storage(std::string const& sp, int const flags
+		, storage_error& ec)
 	{
 		int ret = piece_manager::no_error;
-		std::string save_path = complete(sp);
+		std::string const save_path = complete(sp);
 
 		// check to see if any of the files exist
-		error_code e;
 		file_storage const& f = files();
 
-		file_status s;
 		if (flags == fail_if_exist)
 		{
-			stat_file(save_path, &s, e);
-			if (e != boost::system::errc::no_such_file_or_directory)
+			file_status s;
+			error_code err;
+			stat_file(save_path, &s, err);
+			if (err != boost::system::errc::no_such_file_or_directory)
 			{
 				// the directory exists, check all the files
 				for (int i = 0; i < f.num_files(); ++i)
 				{
 					// files moved out to absolute paths are ignored
-					if (is_complete(f.file_path(i))) continue;
+					if (f.file_absolute_path(i)) continue;
 
-					std::string new_path = f.file_path(i, save_path);
-					stat_file(new_path, &s, e);
-					if (e != boost::system::errc::no_such_file_or_directory)
+					stat_file(f.file_path(i, save_path), &s, err);
+					if (err != boost::system::errc::no_such_file_or_directory)
 					{
-						ec.ec = e;
+						ec.ec = err;
 						ec.file = i;
 						ec.operation = storage_error::stat;
 						return piece_manager::file_exist;
@@ -1162,37 +1158,30 @@ namespace libtorrent
 			}
 		}
 
-		// collect all directories in to_move. This is because we
-		// try to move entire directories by default (instead of
-		// files independently).
-		std::map<std::string, int> to_move;
-		for (int i = 0; i < f.num_files(); ++i)
 		{
-			// files moved out to absolute paths are not moved
-			if (is_complete(f.file_path(i))) continue;
-
-			std::string split = split_path(f.file_path(i));
-			to_move.insert(to_move.begin(), std::make_pair(split, i));
-		}
-
-		e.clear();
-		stat_file(save_path, &s, e);
-		if (e == boost::system::errc::no_such_file_or_directory)
-		{
-			create_directories(save_path, ec.ec);
-			if (ec)
+			file_status s;
+			error_code err;
+			stat_file(save_path, &s, err);
+			if (err == boost::system::errc::no_such_file_or_directory)
 			{
+				err.clear();
+				create_directories(save_path, err);
+				if (err)
+				{
+					ec.ec = err;
+					ec.file = -1;
+					ec.operation = storage_error::mkdir;
+					return piece_manager::fatal_disk_error;
+				}
+			}
+			else if (err)
+			{
+				ec.ec = err;
 				ec.file = -1;
-				ec.operation = storage_error::mkdir;
+				ec.operation = storage_error::stat;
 				return piece_manager::fatal_disk_error;
 			}
 		}
-		else if (ec)
-		{
-			ec.file = -1;
-			ec.operation = storage_error::mkdir;
-			return piece_manager::fatal_disk_error;
-		}
 
 		m_pool.release(this);
 
@@ -1200,85 +1189,106 @@ namespace libtorrent
 		print_open_files("release files", m_files.name().c_str());
 #endif
 
-		for (std::map<std::string, int>::const_iterator i = to_move.begin()
-			, end(to_move.end()); i != end; ++i)
+		int i;
+		error_code e;
+		for (i = 0; i < f.num_files(); ++i)
 		{
-			std::string old_path = combine_path(m_save_path, i->first);
-			std::string new_path = combine_path(save_path, i->first);
+			// files moved out to absolute paths are not moved
+			if (f.file_absolute_path(i)) continue;
 
-			e.clear();
-			rename(old_path, new_path, e);
+			std::string const old_path = combine_path(m_save_path, f.file_path(i));
+			std::string const new_path = combine_path(save_path, f.file_path(i));
+
+			if (flags == dont_replace && exists(new_path))
+			{
+				if (ret == piece_manager::no_error) ret = piece_manager::need_full_check;
+				continue;
+			}
+
+			// TODO: ideally, if we end up copying files because of a move across
+			// volumes, the source should not be deleted until they've all been
+			// copied. That would let us rollback with higher confidence.
+			move_file(old_path, new_path, e);
 			// if the source file doesn't exist. That's not a problem
 			// we just ignore that file
 			if (e == boost::system::errc::no_such_file_or_directory)
 				e.clear();
 
-			// on OSX, the error when trying to rename a file across different
-			// volumes is EXDEV, which will make it fall back to copying.
-
 			if (e)
 			{
-				if (flags == dont_replace && e == boost::system::errc::file_exists)
-				{
-					if (ret == piece_manager::no_error) ret = piece_manager::need_full_check;
-					continue;
-				}
-
-				if (e == boost::system::errc::invalid_argument
-					|| e == boost::system::errc::permission_denied)
-				{
-					ec.ec = e;
-					ec.file = i->second;
-					ec.operation = storage_error::rename;
-					break;
-				}
-
-				if (e != boost::system::errc::no_such_file_or_directory)
-				{
-					e.clear();
-					recursive_copy(old_path, new_path, ec.ec);
-					if (ec.ec == boost::system::errc::no_such_file_or_directory)
-					{
-						// it's a bit weird that rename() would not return
-						// ENOENT, but the file still wouldn't exist. But,
-						// in case it does, we're done.
-						ec.ec.clear();
-						break;
-					}
-					if (ec)
-					{
-						ec.file = i->second;
-						ec.operation = storage_error::copy;
-					}
-					else
-					{
-						// ignore errors when removing
-						error_code ignore;
-						remove_all(old_path, ignore);
-					}
-					break;
-				}
+				ec.ec = e;
+				ec.file = i;
+				ec.operation = storage_error::rename;
+				break;
 			}
 		}
 
-		if (!ec)
+		if (!e && m_part_file)
 		{
-			if (m_part_file)
+			m_part_file->move_partfile(save_path, e);
+			if (e)
 			{
-				// TODO: if everything moves OK, except for the partfile
-				// we currently won't update the save path, which breaks things.
-				// it would probably make more sense to give up on the partfile
-				m_part_file->move_partfile(save_path, ec.ec);
-				if (ec)
+				ec.ec = e;
+				ec.file = -1;
+				ec.operation = storage_error::partfile_move;
+			}
+		}
+
+		if (e)
+		{
+			// rollback
+			while (--i >= 0)
+			{
+				// files moved out to absolute paths are not moved
+				if (f.file_absolute_path(i)) continue;
+
+				std::string const old_path = combine_path(m_save_path, f.file_path(i));
+				std::string const new_path = combine_path(save_path, f.file_path(i));
+
+				if (!exists(old_path))
 				{
-					ec.file = -1;
-					ec.operation = storage_error::partfile_move;
-					return piece_manager::fatal_disk_error;
+					// ignore errors when rolling back
+					error_code ignore;
+					move_file(new_path, old_path, ignore);
 				}
 			}
 
-			m_save_path = save_path;
+			return piece_manager::fatal_disk_error;
 		}
+
+		std::string const old_save_path = m_save_path;
+		m_save_path = save_path;
+
+		std::set<std::string> subdirs;
+		for (i = 0; i < f.num_files(); ++i)
+		{
+			// files moved out to absolute paths are not moved
+			if (f.file_absolute_path(i)) continue;
+
+			if (has_parent_path(f.file_path(i)))
+				subdirs.insert(parent_path(f.file_path(i)));
+
+			std::string const old_path = combine_path(old_save_path, f.file_path(i));
+
+			// we may still have some files in old old_save_path
+			// eg. if (flags == dont_replace && exists(new_path))
+			// ignore errors when removing
+			error_code ignore;
+			remove(old_path, ignore);
+		}
+
+		for (std::set<std::string>::iterator it(subdirs.begin())
+			 , end(subdirs.end()); it != end; ++it)
+		{
+			error_code err;
+			std::string subdir = combine_path(old_save_path, *it);
+			while (subdir != old_save_path && !err)
+			{
+				remove(subdir, err);
+				subdir = parent_path(subdir);
+			}
+		}
+
 		return ret;
 	}
 
