@@ -482,7 +482,7 @@ void udp_socket::on_read_impl(udp::endpoint const& ep
 			&& e != boost::asio::error::operation_aborted
 			&& e != boost::asio::error::network_reset
 			&& e != boost::asio::error::network_unreachable
-#ifdef WIN32
+#ifdef _WIN32
 			// ERROR_MORE_DATA means the same thing as EMSGSIZE
 			&& e != error_code(ERROR_MORE_DATA, system_category())
 			&& e != error_code(ERROR_HOST_UNREACHABLE, system_category())
