@@ -34,32 +34,36 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <deque>
 #include <map>
 
-#include "setup_transfer.hpp"
-
 #include "libtorrent/session.hpp"
 #include "libtorrent/hasher.hpp"
 #include "libtorrent/http_parser.hpp"
 #include "libtorrent/thread.hpp"
-
-#include "libtorrent/thread.hpp"
-#include <boost/tuple/tuple.hpp>
-#include <boost/bind.hpp>
-
-#include "test.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/socket_io.hpp" // print_endpoint
 #include "libtorrent/socket_type.hpp"
 #include "libtorrent/instantiate_connection.hpp"
+#include "libtorrent/ip_filter.hpp"
+#include "libtorrent/session_stats.hpp"
+#include "libtorrent/thread.hpp"
+#include "libtorrent/random.hpp"
+#include "libtorrent/torrent_info.hpp"
+#include "libtorrent/broadcast_socket.hpp" // for supports_ipv6()
+
+#include <boost/tuple/tuple.hpp>
+#include <boost/bind.hpp>
+#include <boost/make_shared.hpp>
+
+#include "test.hpp"
+#include "test_utils.hpp"
+#include "setup_transfer.hpp"
 
 #ifdef TORRENT_USE_OPENSSL
 #include <boost/asio/ssl/stream.hpp>
 #include <boost/asio/ssl/context.hpp>
 #endif
 
-#include <boost/detail/atomic_count.hpp>
-
 #ifndef _WIN32
 #include <spawn.h>
 #include <signal.h>
@@ -70,54 +74,109 @@ POSSIBILITY OF SUCH DAMAGE.
 #define DLOG if (DEBUG_WEB_SERVER) fprintf
 
 using namespace libtorrent;
+namespace lt = libtorrent;
 
-static int tests_failure = 0;
-static std::vector<std::string> failure_strings;
+#if defined TORRENT_WINDOWS
+#include <conio.h>
+#endif
 
-void report_failure(char const* err, char const* file, int line)
+boost::uint32_t g_addr = 0x92343023;
+
+void init_rand_address()
 {
-	char buf[500];
-	snprintf(buf, sizeof(buf), "\x1b[41m***** %s:%d \"%s\" *****\x1b[0m\n", file, line, err);
-	fprintf(stderr, "\n%s\n", buf);
-	failure_strings.push_back(buf);
-	++tests_failure;
+	g_addr = 0x92343023;
 }
 
-int print_failures()
+address rand_v4()
 {
-	if (tests_failure == 0)
-		fprintf(stderr, "\n\n\x1b[42;30m   == %d ALL TESTS PASSED ==\x1b[0m\n\n\n", tests_failure);
-	else
-		fprintf(stderr, "\n\n\x1b[41m   == %d TEST(S) FAILED ==\x1b[0m\n\n\n", tests_failure);
-	return tests_failure;
-}
-
-std::auto_ptr<alert> wait_for_alert(session& ses, int type, char const* name)
-{
-	std::auto_ptr<alert> ret;
-	ptime end = time_now() + seconds(10);
-	while (!ret.get())
+	address_v4 ret;
+	do
 	{
-		ptime now = time_now();
-		if (now > end) return std::auto_ptr<alert>();
+		g_addr += 0x3080ca;
+		ret = address_v4(g_addr);
+	} while (is_any(ret) || is_local(ret) || is_loopback(ret));
+	return ret;
+}
+
+sha1_hash rand_hash()
+{
+	sha1_hash ret;
+	for (int i = 0; i < 20; ++i)
+		ret[i] = lt::random();
+	return ret;
+}
+
+#if TORRENT_USE_IPV6
+address rand_v6()
+{
+	address_v6::bytes_type bytes;
+	for (int i = 0; i < int(bytes.size()); ++i) bytes[i] = rand();
+	return address_v6(bytes);
+}
+#endif
+
+static boost::uint16_t g_port = 0;
+
+tcp::endpoint rand_tcp_ep()
+{
+	// make sure we don't procude the same "random" port twice
+	g_port = (g_port + 1) % 14038;
+	return tcp::endpoint(rand_v4(), g_port + 1024);
+}
+
+udp::endpoint rand_udp_ep()
+{
+	g_port = (g_port + 1) % 14037;
+	return udp::endpoint(rand_v4(), g_port + 1024);
+}
+
+std::map<std::string, boost::int64_t> get_counters(libtorrent::session& s)
+{
+	using namespace libtorrent;
+	s.post_session_stats();
+
+	std::map<std::string, boost::int64_t> ret;
+	alert const* a = wait_for_alert(s, session_stats_alert::alert_type
+		, "get_counters()");
+
+	TEST_CHECK(a);
+	if (!a) return ret;
+
+	session_stats_alert const* sa = alert_cast<session_stats_alert>(a);
+	if (!sa) return ret;
+
+	static std::vector<stats_metric> metrics = session_stats_metrics();
+	for (int i = 0; i < int(metrics.size()); ++i)
+		ret[metrics[i].name] = sa->values[metrics[i].value_index];
+	return ret;
+}
+
+alert const* wait_for_alert(lt::session& ses, int type, char const* name)
+{
+	time_point end = libtorrent::clock_type::now() + seconds(10);
+	while (true)
+	{
+		time_point now = clock_type::now();
+		if (now > end) return NULL;
+
+		alert const* ret = NULL;
 
 		ses.wait_for_alert(end - now);
-		std::deque<alert*> alerts;
+		std::vector<alert*> alerts;
 		ses.pop_alerts(&alerts);
-		for (std::deque<alert*>::iterator i = alerts.begin()
+		for (std::vector<alert*>::iterator i = alerts.begin()
 			, end(alerts.end()); i != end; ++i)
 		{
 			fprintf(stderr, "%s: %s: [%s] %s\n", time_now_string(), name
 				, (*i)->what(), (*i)->message().c_str());
-			if (!ret.get() && (*i)->type() == type)
+			if ((*i)->type() == type && !ret)
 			{
-				ret = std::auto_ptr<alert>(*i);
+				ret = *i;
 			}
-			else
-				delete *i;
 		}
+		if (ret) return ret;
 	}
-	return ret;
+	return NULL;
 }
 
 int load_file(std::string const& filename, std::vector<char>& v, libtorrent::error_code& ec, int limit)
@@ -126,21 +185,21 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	FILE* f = fopen(filename.c_str(), "rb");
 	if (f == NULL)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		return -1;
 	}
 
 	int r = fseek(f, 0, SEEK_END);
 	if (r != 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
 	long s = ftell(f);
 	if (s < 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -154,7 +213,7 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	r = fseek(f, 0, SEEK_SET);
 	if (r != 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -169,7 +228,7 @@ int load_file(std::string const& filename, std::vector<char>& v, libtorrent::err
 	r = fread(&v[0], 1, v.size(), f);
 	if (r < 0)
 	{
-		ec.assign(errno, boost::system::get_generic_category());
+		ec.assign(errno, boost::system::system_category());
 		fclose(f);
 		return -1;
 	}
@@ -202,21 +261,21 @@ void save_file(char const* filename, char const* data, int size)
 
 }
 
-bool print_alerts(libtorrent::session& ses, char const* name
+bool print_alerts(lt::session& ses, char const* name
 	, bool allow_disconnects, bool allow_no_torrents, bool allow_failed_fastresume
-	, bool (*predicate)(libtorrent::alert*), bool no_output)
+	, bool (*predicate)(libtorrent::alert const*), bool no_output)
 {
 	bool ret = false;
 	std::vector<torrent_handle> handles = ses.get_torrents();
 	TEST_CHECK(!handles.empty() || allow_no_torrents);
 	torrent_handle h;
 	if (!handles.empty()) h = handles[0];
-	std::deque<alert*> alerts;
+	std::vector<alert*> alerts;
 	ses.pop_alerts(&alerts);
-	for (std::deque<alert*>::iterator i = alerts.begin(); i != alerts.end(); ++i)
+	for (std::vector<alert*>::iterator i = alerts.begin(); i != alerts.end(); ++i)
 	{
 		if (predicate && predicate(*i)) ret = true;
-		if (peer_disconnected_alert* p = alert_cast<peer_disconnected_alert>(*i))
+		if (peer_disconnected_alert const* p = alert_cast<peer_disconnected_alert>(*i))
 		{
 			fprintf(stderr, "%s: %s: [%s] (%s): %s\n", time_now_string(), name, (*i)->what(), print_endpoint(p->ip).c_str(), p->message().c_str());
 		}
@@ -230,7 +289,7 @@ bool print_alerts(libtorrent::session& ses, char const* name
 
 		TEST_CHECK(alert_cast<fastresume_rejected_alert>(*i) == 0 || allow_failed_fastresume);
 /*
-		peer_error_alert* pea = alert_cast<peer_error_alert>(*i);
+		peer_error_alert const* pea = alert_cast<peer_error_alert>(*i);
 		if (pea)
 		{
 			fprintf(stderr, "%s: peer error: %s\n", time_now_string(), pea->error.message().c_str());
@@ -247,13 +306,19 @@ bool print_alerts(libtorrent::session& ses, char const* name
 				|| (allow_disconnects && pea->error.message() == "End of file."));
 		}
 */
-		delete *i;
+
+		invalid_request_alert const* ira = alert_cast<invalid_request_alert>(*i);
+		if (ira)
+		{
+			fprintf(stderr, "peer error: %s\n", ira->message().c_str());
+			TEST_CHECK(false);
+		}
 	}
 	return ret;
 }
 
 bool listen_done = false;
-bool listen_alert(libtorrent::alert* a)
+bool listen_alert(libtorrent::alert const* a)
 {
 	if (alert_cast<listen_failed_alert>(a)
 		|| alert_cast<listen_succeeded_alert>(a))
@@ -261,7 +326,7 @@ bool listen_alert(libtorrent::alert* a)
 	return true;
 }
 
-void wait_for_listen(libtorrent::session& ses, char const* name)
+void wait_for_listen(lt::session& ses, char const* name)
 {
 	listen_done = false;
 	alert const* a = 0;
@@ -275,6 +340,35 @@ void wait_for_listen(libtorrent::session& ses, char const* name)
 	TEST_CHECK(listen_done);
 }
 
+bool downloading_done = false;
+bool downloading_alert(libtorrent::alert const* a)
+{
+	state_changed_alert const* sc = alert_cast<state_changed_alert>(a);
+	if (sc && sc->state == torrent_status::downloading)
+		downloading_done = true;
+	return true;
+}
+
+void wait_for_downloading(lt::session& ses, char const* name)
+{
+	time_point start = clock_type::now();
+	downloading_done = false;
+	alert const* a = 0;
+	do
+	{
+		print_alerts(ses, name, true, true, true, &downloading_alert, false);
+		if (downloading_done) break;
+		if (total_seconds(clock_type::now() - start) > 10) break;
+		a = ses.wait_for_alert(seconds(2));
+	} while (a);
+	if (!downloading_done)
+	{
+		fprintf(stderr, "%s: did not receive a state_changed_alert indicating "
+			"the torrent is downloading. waited: %d ms\n"
+			, name, int(total_milliseconds(clock_type::now() - start)));
+	}
+}
+
 void print_ses_rate(float time
 	, libtorrent::torrent_status const* st1
 	, libtorrent::torrent_status const* st2
@@ -288,7 +382,7 @@ void print_ses_rate(float time
 			, int(st1->progress * 100)
 			, st1->num_peers
 			, st1->connect_candidates
-			, st1->error.empty() ? "" : (" [" + st1->error + "]").c_str());
+			, st1->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 	}
 	if (st2)
 		fprintf(stderr, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
@@ -297,7 +391,7 @@ void print_ses_rate(float time
 			, int(st2->progress * 100)
 			, st2->num_peers
 			, st2->connect_candidates
-			, st2->error.empty() ? "" : (" [" + st2->error + "]").c_str());
+			, st2->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 	if (st3)
 		fprintf(stderr, " : %3.1fs | %dkB/s %dkB/s %d%% %d cc:%d%s", time
 			, int(st3->download_payload_rate / 1000)
@@ -305,14 +399,20 @@ void print_ses_rate(float time
 			, int(st3->progress * 100)
 			, st3->num_peers
 			, st3->connect_candidates
-			, st3->error.empty() ? "" : (" [" + st3->error + "]").c_str());
+			, st3->errc ? (" [" + st1->errc.message() + "]").c_str() : "");
 
 	fprintf(stderr, "\n");
 }
 
-void test_sleep(int millisec)
+void test_sleep(int milliseconds)
 {
-	libtorrent::sleep(millisec);
+#if defined TORRENT_WINDOWS || defined TORRENT_CYGWIN
+	Sleep(milliseconds);
+#elif defined TORRENT_BEOS
+	snooze_until(system_time() + boost::int64_t(milliseconds) * 1000, B_SYSTEM_TIMEBASE);
+#else
+	usleep(milliseconds * 1000);
+#endif
 }
 
 #ifdef _WIN32
@@ -358,7 +458,7 @@ pid_type async_run(char const* cmdline)
 	if (ret == 0)
 	{
 		int error = GetLastError();
-		fprintf(stderr, "failed (%d) %s\n", error, error_code(error, get_system_category()).message().c_str());
+		fprintf(stderr, "failed (%d) %s\n", error, error_code(error, system_category()).message().c_str());
 		return 0;
 	}
 	return pi.dwProcessId;
@@ -414,6 +514,7 @@ void stop_all_proxies()
 	}
 }
 
+// returns a port on success and -1 on failure
 int start_proxy(int proxy_type)
 {
 	using namespace libtorrent;
@@ -424,36 +525,45 @@ int start_proxy(int proxy_type)
 		if (i->second.type == proxy_type) { return i->first; }
 	}
 
-	unsigned int seed = total_microseconds(time_now_hires() - min_time()) & 0xffffffff;
-	printf("random seed: %u\n", seed);
-	std::srand(seed);
+	int port = 2000 + (lt::random() % 6000);
+	error_code ec;
+	io_service ios;
+
+	// make sure the port we pick is free
+	do {
+		++port;
+		tcp::socket s(ios);
+		s.open(tcp::v4(), ec);
+		if (ec) break;
+		s.bind(tcp::endpoint(address::from_string("127.0.0.1"), port), ec);
+	} while (ec);
+
 
-	int port = 5000 + (rand() % 55000);
 	char const* type = "";
 	char const* auth = "";
 	char const* cmd = "";
 
 	switch (proxy_type)
 	{
-		case proxy_settings::socks4:
+		case settings_pack::socks4:
 			type = "socks4";
 			auth = " --allow-v4";
 			cmd = "python ../socks.py";
 			break;
-		case proxy_settings::socks5:
+		case settings_pack::socks5:
 			type = "socks5";
 			cmd = "python ../socks.py";
 			break;
-		case proxy_settings::socks5_pw:
+		case settings_pack::socks5_pw:
 			type = "socks5";
 			auth = " --username testuser --password testpass";
 			cmd = "python ../socks.py";
 			break;
-		case proxy_settings::http:
+		case settings_pack::http:
 			type = "http";
 			cmd = "python ../http.py";
 			break;
-		case proxy_settings::http_pw:
+		case settings_pack::http_pw:
 			type = "http";
 			auth = " --username testuser --password testpass";
 			cmd = "python ../http.py";
@@ -465,7 +575,7 @@ int start_proxy(int proxy_type)
 	fprintf(stderr, "%s starting proxy on port %d (%s %s)...\n", time_now_string(), port, type, auth);
 	fprintf(stderr, "%s\n", buf);
 	pid_type r = async_run(buf);
-	if (r == 0) exit(1);
+	if (r == 0) abort();
 	proxy_t t = { r, proxy_type };
 	running_proxies.insert(std::make_pair(port, t));
 	fprintf(stderr, "%s launched\n", time_now_string());
@@ -476,9 +586,9 @@ int start_proxy(int proxy_type)
 using namespace libtorrent;
 
 template <class T>
-boost::intrusive_ptr<T> clone_ptr(boost::intrusive_ptr<T> const& ptr)
+boost::shared_ptr<T> clone_ptr(boost::shared_ptr<T> const& ptr)
 {
-	return boost::intrusive_ptr<T>(new T(*ptr));
+	return boost::make_shared<T>(*ptr);
 }
 
 unsigned char random_byte()
@@ -505,7 +615,7 @@ void create_random_files(std::string const& path, const int file_sizes[], int nu
 		file f(full_path, file::write_only, ec);
 		if (ec) fprintf(stderr, "failed to create file \"%s\": (%d) %s\n"
 			, full_path.c_str(), ec.value(), ec.message().c_str());
-		size_type offset = 0;
+		boost::int64_t offset = 0;
 		while (to_write > 0)
 		{
 			int s = (std::min)(to_write, 300000);
@@ -520,21 +630,20 @@ void create_random_files(std::string const& path, const int file_sizes[], int nu
 	free(random_data);
 }
 
-boost::intrusive_ptr<torrent_info> create_torrent(std::ostream* file, int piece_size
+boost::shared_ptr<torrent_info> create_torrent(std::ostream* file
+	, char const* name, int piece_size
 	, int num_pieces, bool add_tracker, std::string ssl_certificate)
 {
-	char const* tracker_url = "http://non-existent-name.com/announce";
 	// excercise the path when encountering invalid urls
 	char const* invalid_tracker_url = "http:";
 	char const* invalid_tracker_protocol = "foo://non/existent-name.com/announce";
-	
+
 	file_storage fs;
 	int total_size = piece_size * num_pieces;
-	fs.add_file("temporary", total_size);
+	fs.add_file(name, total_size);
 	libtorrent::create_torrent t(fs, piece_size);
 	if (add_tracker)
 	{
-		t.add_tracker(tracker_url);
 		t.add_tracker(invalid_tracker_url);
 		t.add_tracker(invalid_tracker_protocol);
 	}
@@ -559,7 +668,7 @@ boost::intrusive_ptr<torrent_info> create_torrent(std::ostream* file, int piece_
 	std::vector<char> piece(piece_size);
 	for (int i = 0; i < int(piece.size()); ++i)
 		piece[i] = (i % 26) + 'A';
-	
+
 	// calculate the hash for all pieces
 	int num = t.num_pieces();
 	sha1_hash ph = hasher(&piece[0], piece.size()).final();
@@ -574,7 +683,7 @@ boost::intrusive_ptr<torrent_info> create_torrent(std::ostream* file, int piece_
 			total_size -= piece.size();
 		}
 	}
-	
+
 	std::vector<char> tmp;
 	std::back_insert_iterator<std::vector<char> > out(tmp);
 
@@ -582,75 +691,76 @@ boost::intrusive_ptr<torrent_info> create_torrent(std::ostream* file, int piece_
 
 	bencode(out, tor);
 	error_code ec;
-	return boost::intrusive_ptr<torrent_info>(new torrent_info(
-		&tmp[0], tmp.size(), ec));
-}
-
-void update_settings(session_settings& sess_set, bool allow_multiple_ips)
-{
-	if (allow_multiple_ips) sess_set.allow_multiple_connections_per_ip = true;
-	sess_set.ignore_limits_on_local_network = false;
-	sess_set.mixed_mode_algorithm = session_settings::prefer_tcp;
-	sess_set.max_failcount = 1;
+	return boost::make_shared<torrent_info>(
+		&tmp[0], tmp.size(), boost::ref(ec), 0);
 }
 
 boost::tuple<torrent_handle, torrent_handle, torrent_handle>
-setup_transfer(session* ses1, session* ses2, session* ses3
+setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	, bool clear_files, bool use_metadata_transfer, bool connect_peers
 	, std::string suffix, int piece_size
-	, boost::intrusive_ptr<torrent_info>* torrent, bool super_seeding
+	, boost::shared_ptr<torrent_info>* torrent, bool super_seeding
 	, add_torrent_params const* p, bool stop_lsd, bool use_ssl_ports
-	, boost::intrusive_ptr<torrent_info>* torrent2)
+	, boost::shared_ptr<torrent_info>* torrent2)
 {
-	assert(ses1);
-	assert(ses2);
+	TORRENT_ASSERT(ses1);
+	TORRENT_ASSERT(ses2);
 
 	if (stop_lsd)
 	{
-		ses1->stop_lsd();
-		ses2->stop_lsd();
-		if (ses3) ses3->stop_lsd();
+		settings_pack pack;
+		pack.set_bool(settings_pack::enable_lsd, false);
+		ses1->apply_settings(pack);
+		ses2->apply_settings(pack);
+		if (ses3) ses3->apply_settings(pack);
 	}
 
-	session_settings sess_set = ses1->settings();
-	update_settings(sess_set, ses3 != NULL);
-	ses1->set_settings(sess_set);
-
-	sess_set = ses2->settings();
-	update_settings(sess_set, ses3 != NULL);
-	ses2->set_settings(sess_set);
-
-	if (ses3)
-	{
-		sess_set = ses3->settings();
-		update_settings(sess_set, ses3 != NULL);
-		ses3->set_settings(sess_set);
-	}
-
-	ses1->set_alert_mask(~(alert::progress_notification | alert::stats_notification));
-	ses2->set_alert_mask(~(alert::progress_notification | alert::stats_notification));
-	if (ses3) ses3->set_alert_mask(~(alert::progress_notification | alert::stats_notification));
+	// This has the effect of applying the global
+	// rule to all peers, regardless of if they're local or not
+	ip_filter f;
+	f.add_rule(address_v4::from_string("0.0.0.0")
+		, address_v4::from_string("255.255.255.255")
+		, 1 << lt::session::global_peer_class_id);
+	ses1->set_peer_class_filter(f);
+	ses2->set_peer_class_filter(f);
+	if (ses3) ses3->set_peer_class_filter(f);
 
+	settings_pack pack;
+	pack.set_int(settings_pack::alert_mask, ~(alert::progress_notification | alert::stats_notification));
+	if (ses3) pack.set_bool(settings_pack::allow_multiple_connections_per_ip, true);
+	pack.set_int(settings_pack::mixed_mode_algorithm, settings_pack::prefer_tcp);
+	pack.set_int(settings_pack::max_failcount, 1);
 	peer_id pid;
 	std::generate(&pid[0], &pid[0] + 20, random_byte);
-	ses1->set_peer_id(pid);
+	pack.set_str(settings_pack::peer_fingerprint, pid.to_string());
+	ses1->apply_settings(pack);
+	TORRENT_ASSERT(ses1->id() == pid);
+
 	std::generate(&pid[0], &pid[0] + 20, random_byte);
-	ses2->set_peer_id(pid);
-	assert(ses1->id() != ses2->id());
+	TORRENT_ASSERT(ses1->id() != pid);
+	pack.set_str(settings_pack::peer_fingerprint, pid.to_string());
+	ses2->apply_settings(pack);
+	TORRENT_ASSERT(ses2->id() == pid);
 	if (ses3)
 	{
 		std::generate(&pid[0], &pid[0] + 20, random_byte);
-		ses3->set_peer_id(pid);
-		assert(ses3->id() != ses2->id());
+		TORRENT_ASSERT(ses1->id() != pid);
+		TORRENT_ASSERT(ses2->id() != pid);
+		pack.set_str(settings_pack::peer_fingerprint, pid.to_string());
+		ses3->apply_settings(pack);
+		TORRENT_ASSERT(ses3->id() == pid);
 	}
 
-	boost::intrusive_ptr<torrent_info> t;
+	TORRENT_ASSERT(ses1->id() != ses2->id());
+	if (ses3) TORRENT_ASSERT(ses3->id() != ses2->id());
+
+	boost::shared_ptr<torrent_info> t;
 	if (torrent == 0)
 	{
 		error_code ec;
 		create_directory("tmp1" + suffix, ec);
 		std::ofstream file(combine_path("tmp1" + suffix, "temporary").c_str());
-		t = ::create_torrent(&file, piece_size, 9, false);
+		t = ::create_torrent(&file, "temporary", piece_size, 9, false);
 		file.close();
 		if (clear_files)
 		{
@@ -678,12 +788,18 @@ setup_transfer(session* ses1, session* ses2, session* ses3
 	param.flags |= add_torrent_params::flag_seed_mode;
 	error_code ec;
 	torrent_handle tor1 = ses1->add_torrent(param, ec);
+	if (ec)
+	{
+		fprintf(stderr, "ses1.add_torrent: %s\n", ec.message().c_str());
+		return boost::make_tuple(torrent_handle(), torrent_handle(), torrent_handle());
+	}
 	tor1.super_seeding(super_seeding);
 
 	// the downloader cannot use seed_mode
 	param.flags &= ~add_torrent_params::flag_seed_mode;
 
 	TEST_CHECK(!ses1->get_torrents().empty());
+
 	torrent_handle tor2;
 	torrent_handle tor3;
 
@@ -695,9 +811,9 @@ setup_transfer(session* ses1, session* ses2, session* ses3
 		TEST_CHECK(!ses3->get_torrents().empty());
 	}
 
-  	if (use_metadata_transfer)
+	if (use_metadata_transfer)
 	{
-		param.ti = 0;
+		param.ti.reset();
 		param.info_hash = t->info_hash();
 	}
 	else if (torrent2)
@@ -713,62 +829,59 @@ setup_transfer(session* ses1, session* ses2, session* ses3
 	tor2 = ses2->add_torrent(param, ec);
 	TEST_CHECK(!ses2->get_torrents().empty());
 
-	assert(ses1->get_torrents().size() == 1);
-	assert(ses2->get_torrents().size() == 1);
+	TORRENT_ASSERT(ses1->get_torrents().size() == 1);
+	TORRENT_ASSERT(ses2->get_torrents().size() == 1);
 
 //	test_sleep(100);
 
 	if (connect_peers)
 	{
-		std::auto_ptr<alert> a;
-/*		do
-		{
-			a = wait_for_alert(*ses2, state_changed_alert::alert_type, "ses2");
-		} while (static_cast<state_changed_alert*>(a.get())->state != torrent_status::downloading);
-*/
-//		wait_for_alert(*ses1, torrent_finished_alert::alert_type, "ses1");
+		wait_for_downloading(*ses2, "ses2");
 
 		error_code ec;
+		int port = 0;
 		if (use_ssl_ports)
 		{
-			fprintf(stderr, "%s: ses1: connecting peer port: %d\n", time_now_string(), int(ses2->ssl_listen_port()));
-			tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
-				, ses2->ssl_listen_port()));
+			port = ses2->ssl_listen_port();
+			fprintf(stderr, "%s: ses2->ssl_listen_port(): %d\n", time_now_string(), port);
 		}
-		else
+
+		if (port == 0)
 		{
-			fprintf(stderr, "%s: ses1: connecting peer port: %d\n", time_now_string(), int(ses2->listen_port()));
-			tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
-				, ses2->listen_port()));
+			port = ses2->listen_port();
+			fprintf(stderr, "%s: ses2->listen_port(): %d\n", time_now_string(), port);
 		}
 
+		fprintf(stderr, "%s: ses1: connecting peer port: %d\n"
+			, time_now_string(), port);
+		tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
+			, port));
+
 		if (ses3)
 		{
 			// give the other peers some time to get an initial
 			// set of pieces before they start sharing with each-other
 
+			wait_for_downloading(*ses3, "ses3");
+
+			port = 0;
+			int port2 = 0;
 			if (use_ssl_ports)
 			{
-				fprintf(stderr, "ses3: connecting peer port: %d\n", int(ses2->ssl_listen_port()));
-				tor3.connect_peer(tcp::endpoint(
-					address::from_string("127.0.0.1", ec)
-					, ses2->ssl_listen_port()));
-				fprintf(stderr, "ses3: connecting peer port: %d\n", int(ses1->ssl_listen_port()));
-				tor3.connect_peer(tcp::endpoint(
-					address::from_string("127.0.0.1", ec)
-					, ses1->ssl_listen_port()));
+				port = ses2->ssl_listen_port();
+				port2 = ses1->ssl_listen_port();
 			}
-			else
-			{
-				fprintf(stderr, "ses3: connecting peer port: %d\n", int(ses2->listen_port()));
+
+			if (port == 0) port = ses2->listen_port();
+			if (port2 == 0) port2 = ses1->listen_port();
+
+			fprintf(stderr, "ses3: connecting peer port: %d\n", port);
+			tor3.connect_peer(tcp::endpoint(
+					address::from_string("127.0.0.1", ec), port));
+			fprintf(stderr, "ses3: connecting peer port: %d\n", port2);
 				tor3.connect_peer(tcp::endpoint(
 					address::from_string("127.0.0.1", ec)
-					, ses2->listen_port()));
-				fprintf(stderr, "ses3: connecting peer port: %d\n", int(ses1->listen_port()));
-				tor3.connect_peer(tcp::endpoint(
-					address::from_string("127.0.0.1", ec)
-					, ses1->listen_port()));
-			}
+					, port2));
 		}
 	}
 
@@ -779,10 +892,18 @@ pid_type web_server_pid = 0;
 
 int start_web_server(bool ssl, bool chunked_encoding, bool keepalive)
 {
-	unsigned int seed = total_microseconds(time_now_hires() - min_time()) & 0xffffffff;
-	fprintf(stderr, "random seed: %u\n", seed);
-	std::srand(seed);
-	int port = 5000 + (rand() % 55000);
+	int port = 2000 + (lt::random() % 6000);
+	error_code ec;
+	io_service ios;
+
+	// make sure the port we pick is free
+	do {
+		++port;
+		tcp::socket s(ios);
+		s.open(tcp::v4(), ec);
+		if (ec) break;
+		s.bind(tcp::endpoint(address::from_string("127.0.0.1"), port), ec);
+	} while (ec);
 
 	char buf[200];
 	snprintf(buf, sizeof(buf), "python ../web_server.py %d %d %d %d"
@@ -792,7 +913,7 @@ int start_web_server(bool ssl, bool chunked_encoding, bool keepalive)
 
 	fprintf(stderr, "%s\n", buf);
 	pid_type r = async_run(buf);
-	if (r == 0) exit(1);
+	if (r == 0) abort();
 	web_server_pid = r;
 	fprintf(stderr, "%s launched\n", time_now_string());
 	test_sleep(500);
@@ -807,3 +928,8 @@ void stop_web_server()
 	web_server_pid = 0;
 }
 
+tcp::endpoint ep(char const* ip, int port)
+{
+	error_code ec;
+	return tcp::endpoint(address::from_string(ip, ec), port);
+}
