@@ -35,6 +35,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "setup_transfer.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include <sys/stat.h> // for chmod
+#include "libtorrent/torrent_info.hpp"
+#include "libtorrent/torrent_status.hpp"
+
 static const int file_sizes[] =
 { 5, 16 - 5, 16000, 17, 10, 8000, 8000, 1,1,1,1,1,100,1,1,1,1,100,1,1,1,1,1,1
 	,1,1,1,1,1,1,13,65000,34,75,2,30,400,500,23000,900,43000,400,4300,6, 4};
@@ -56,6 +59,7 @@ enum
 void test_checking(int flags = read_only_files)
 {
 	using namespace libtorrent;
+	namespace lt = libtorrent;
 
 	fprintf(stderr, "\n==== TEST CHECKING %s%s%s=====\n\n"
 		, (flags & read_only_files) ? "read-only-files ":""
@@ -90,7 +94,7 @@ void test_checking(int flags = read_only_files)
 	create_directory(combine_path("tmp1_checking", "test_torrent_dir"), ec);
 	if (ec) fprintf(stderr, "ERROR: creating directory test_torrent_dir: (%d) %s\n"
 		, ec.value(), ec.message().c_str());
-	
+
 	file_storage fs;
 	std::srand(10);
 	int piece_size = 0x4000;
@@ -99,7 +103,8 @@ void test_checking(int flags = read_only_files)
 		, file_sizes, num_files);
 
 	add_files(fs, combine_path("tmp1_checking", "test_torrent_dir"));
-	libtorrent::create_torrent t(fs, piece_size, 0x4000, libtorrent::create_torrent::optimize);
+	libtorrent::create_torrent t(fs, piece_size, 0x4000
+		, libtorrent::create_torrent::optimize_alignment);
 
 	// calculate the hash for all pieces
 	set_piece_hashes(t, "tmp1_checking", ec);
@@ -108,7 +113,7 @@ void test_checking(int flags = read_only_files)
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
-	boost::intrusive_ptr<torrent_info> ti(new torrent_info(&buf[0], buf.size(), ec));
+	boost::shared_ptr<torrent_info> ti(new torrent_info(&buf[0], buf.size(), ec));
 
 	fprintf(stderr, "generated torrent: %s tmp1_checking/test_torrent_dir\n"
 		, to_hex(ti->info_hash().to_string()).c_str());
@@ -138,6 +143,7 @@ void test_checking(int flags = read_only_files)
 	// overwrite the files with new random data
 	if (flags & corrupt_files)
 	{
+		fprintf(stderr, "corrupt file test. overwriting files\n");
 		// increase the size of some files. When they're read only that forces
 		// the checker to open them in write-mode to truncate them
 		static const int file_sizes2[] =
@@ -149,6 +155,7 @@ void test_checking(int flags = read_only_files)
 	// make the files read only
 	if (flags & read_only_files)
 	{
+		fprintf(stderr, "making files read-only\n");
 		for (int i = 0; i < num_files; ++i)
 		{
 			char name[1024];
@@ -158,6 +165,7 @@ void test_checking(int flags = read_only_files)
 
 			std::string path = combine_path(combine_path("tmp1_checking", "test_torrent_dir"), dirname);
 			path = combine_path(path, name);
+			fprintf(stderr, "   %s\n", path.c_str());
 
 #ifdef TORRENT_WINDOWS
 			SetFileAttributesA(path.c_str(), FILE_ATTRIBUTE_READONLY);
@@ -167,8 +175,20 @@ void test_checking(int flags = read_only_files)
 		}
 	}
 
-	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48000, 49000), "0.0.0.0", 0);
-	ses1.set_alert_mask(alert::all_categories);
+	int mask = alert::all_categories
+		& ~(alert::progress_notification
+			| alert::performance_warning
+			| alert::stats_notification);
+
+	settings_pack pack;
+	pack.set_bool(settings_pack::enable_lsd, false);
+	pack.set_bool(settings_pack::enable_natpmp, false);
+	pack.set_bool(settings_pack::enable_upnp, false);
+	pack.set_bool(settings_pack::enable_dht, false);
+	pack.set_int(settings_pack::alert_mask, mask);
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48000");
+	pack.set_int(settings_pack::max_retry_port_bind, 1000);
+	lt::session ses1(pack);
 
 	add_torrent_params p;
 	p.save_path = "tmp1_checking";
@@ -177,21 +197,24 @@ void test_checking(int flags = read_only_files)
 	TEST_CHECK(!ec);
 
 	torrent_status st;
-	for (int i = 0; i < 5; ++i)
+	for (int i = 0; i < 20; ++i)
 	{
 		print_alerts(ses1, "ses1");
 
 		st = tor1.status();
 
-		printf("%d %f %s\n", st.state, st.progress_ppm / 10000.f, st.error.c_str());
+		printf("%d %f %s\n", st.state, st.progress_ppm / 10000.f, st.errc.message().c_str());
 
-		if (st.state != torrent_status::queued_for_checking
-			&& st.state != torrent_status::checking_files
+		if (
+#ifndef TORRENT_NO_DEPRECATE
+			st.state != torrent_status::queued_for_checking &&
+#endif
+			st.state != torrent_status::checking_files
 			&& st.state != torrent_status::checking_resume_data)
 			break;
 
-		if (!st.error.empty()) break;
-		test_sleep(1000);
+		if (st.errc) break;
+		test_sleep(500);
 	}
 	if (flags & incomplete_files)
 	{
@@ -205,22 +228,38 @@ void test_checking(int flags = read_only_files)
 	if (flags & corrupt_files)
 	{
 		TEST_CHECK(!st.is_seeding);
-		TEST_CHECK(!st.error.empty());
 
-		// wait a while to make sure libtorrent survived the error
-		test_sleep(5000);
+		if (flags & read_only_files)
+		{
+			// we expect our checking of the files to trigger
+			// attempts to truncate them, since the files are
+			// read-only here, we expect the checking to fail.
+			TEST_CHECK(st.errc);
+			if (st.errc)
+				fprintf(stderr, "error: %s\n", st.errc.message().c_str());
 
-		st = tor1.status();
-		TEST_CHECK(!st.is_seeding);
-		TEST_CHECK(!st.error.empty());
+			// wait a while to make sure libtorrent survived the error
+			test_sleep(1000);
+
+			st = tor1.status();
+			TEST_CHECK(!st.is_seeding);
+			TEST_CHECK(st.errc);
+			if (st.errc)
+				fprintf(stderr, "error: %s\n", st.errc.message().c_str());
+		}
+		else
+		{
+			TEST_CHECK(!st.errc);
+			if (st.errc)
+				fprintf(stderr, "error: %s\n", st.errc.message().c_str());
+		}
 	}
 
 	if ((flags & (incomplete_files | corrupt_files)) == 0)
 	{
 		TEST_CHECK(st.is_seeding);
-		if (!st.error.empty())
-			fprintf(stderr, "ERROR: %s\n", st.error.c_str());
-		TEST_CHECK(st.error.empty());
+		if (st.errc)
+			fprintf(stderr, "error: %s\n", st.errc.message().c_str());
 	}
 
 	// make the files writable again
@@ -248,13 +287,28 @@ void test_checking(int flags = read_only_files)
 		, ec.value(), ec.message().c_str());
 }
 
-int test_main()
+TORRENT_TEST(checking)
 {
 	test_checking();
+}
+
+TORRENT_TEST(read_only_corrupt)
+{
 	test_checking(read_only_files | corrupt_files);
+}
+
+TORRENT_TEST(read_only)
+{
 	test_checking(read_only_files);
+}
+
+TORRENT_TEST(incomplete)
+{
 	test_checking(incomplete_files);
+}
 
-	return 0;
+TORRENT_TEST(corrupt)
+{
+	test_checking(corrupt_files);
 }
 
