@@ -31,19 +31,21 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "test.hpp"
+#include "setup_transfer.hpp"
 #include "libtorrent/magnet_uri.hpp"
 #include "libtorrent/session.hpp"
 #include "libtorrent/torrent_handle.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/torrent_info.hpp" // for announce_entry
 #include "libtorrent/announce_entry.hpp"
+#include "settings.hpp"
 
 using namespace libtorrent;
 namespace lt = libtorrent;
 
 void test_remove_url(std::string url)
 {
-	lt::session s;
+	lt::session s(settings());
 	add_torrent_params p;
 	p.flags &= ~add_torrent_params::flag_paused;
 	p.flags &= ~add_torrent_params::flag_auto_managed;
@@ -75,7 +77,7 @@ TORRENT_TEST(magnet)
 	session_proxy p2;
 
 	// test session state load/restore
-	settings_pack pack;
+	settings_pack pack = settings();
 	pack.set_str(settings_pack::user_agent, "test");
 	pack.set_int(settings_pack::tracker_receive_timeout, 1234);
 	pack.set_int(settings_pack::file_pool_size, 543);
@@ -176,7 +178,7 @@ TORRENT_TEST(magnet)
 	TEST_EQUAL(to_hex(t.info_hash().to_string()), "cdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd");
 
 	p1 = s->abort();
-	s.reset(new lt::session());
+	s.reset(new lt::session(settings()));
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), session_state);
@@ -276,6 +278,20 @@ TORRENT_TEST(parse_space_hash)
 	ec.clear();
 }
 
+TORRENT_TEST(parse_peer)
+{
+	std::vector<tcp::endpoint> peers;
+	parse_magnet_uri_peers("magnet:?xt=urn:btih:cdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd&dn=foo&x.pe=127.0.0.1:43&x.pe=<invalid1>&x.pe=<invalid2>:100&x.pe=[::1]:45", peers);
+#if TORRENT_USE_IPV6
+	TEST_EQUAL(peers.size(), 2);
+	TEST_EQUAL(peers[0], ep("127.0.0.1", 43));
+	TEST_EQUAL(peers[1], ep("::1", 45));
+#else
+	TEST_EQUAL(peers.size(), 1);
+	TEST_EQUAL(peers[0], ep("127.0.0.1", 43));
+#endif
+}
+
 #ifndef TORRENT_DISABLE_DHT
 TORRENT_TEST(parse_dht_node)
 {
