@@ -43,6 +43,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "test.hpp"
 #include "setup_transfer.hpp"
+#include "settings.hpp"
 
 using namespace libtorrent;
 namespace lt = libtorrent;
@@ -193,7 +194,7 @@ void default_tests(torrent_status const& s)
 
 TORRENT_TEST(piece_priorities)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
@@ -239,7 +240,7 @@ TORRENT_TEST(piece_priorities)
 
 TORRENT_TEST(file_priorities_default)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses, 0, "", "").file_priorities();
 
 	TEST_EQUAL(file_priorities.size(), 3);
@@ -251,7 +252,7 @@ TORRENT_TEST(file_priorities_default)
 TORRENT_TEST(file_priorities_resume_seed_mode)
 {
 	// in share mode file priorities should always be 0
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses,
 		add_torrent_params::flag_share_mode, "", "123").file_priorities();
 
@@ -264,7 +265,7 @@ TORRENT_TEST(file_priorities_resume_seed_mode)
 TORRENT_TEST(file_priorities_seed_mode)
 {
 	// in share mode file priorities should always be 0
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses,
 		add_torrent_params::flag_share_mode, "123", "").file_priorities();
 
@@ -278,7 +279,7 @@ TORRENT_TEST(zero_file_prio)
 {
 	fprintf(stderr, "test_file_prio\n");
 
-	lt::session ses;
+	lt::session ses(settings());
 	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
@@ -317,7 +318,7 @@ void test_seed_mode(bool file_prio, bool pieces_have, bool piece_prio
 	fprintf(stderr, "test_seed_mode file_prio: %d pieces_have: %d piece_prio: %d\n"
 		, file_prio, pieces_have, piece_prio);
 
-	lt::session ses;
+	lt::session ses(settings());
 	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
@@ -397,7 +398,7 @@ TORRENT_TEST(seed_mode_preserve)
 
 TORRENT_TEST(resume_save_load)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_handle h = test_resume_flags(ses, 0, "123", "");
 
 	h.save_resume_data();
@@ -423,7 +424,7 @@ TORRENT_TEST(resume_save_load)
 
 TORRENT_TEST(resume_save_load_resume)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_handle h = test_resume_flags(ses, 0, "", "123");
 
 	h.save_resume_data();
@@ -452,7 +453,7 @@ TORRENT_TEST(file_priorities_resume_override)
 	// make sure that an empty file_priorities vector in add_torrent_params won't
 	// override the resume data file priorities, even when override resume data
 	// flag is set.
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses,
 		add_torrent_params::flag_override_resume_data, "", "123").file_priorities();
 
@@ -464,7 +465,7 @@ TORRENT_TEST(file_priorities_resume_override)
 
 TORRENT_TEST(file_priorities_resume)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses, 0, "", "123").file_priorities();
 
 	TEST_EQUAL(file_priorities.size(), 3);
@@ -475,7 +476,7 @@ TORRENT_TEST(file_priorities_resume)
 
 TORRENT_TEST(file_priorities1)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses, 0, "010").file_priorities();
 
 	TEST_EQUAL(file_priorities.size(), 3);
@@ -488,7 +489,7 @@ TORRENT_TEST(file_priorities1)
 
 TORRENT_TEST(file_priorities2)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses, 0, "123").file_priorities();
 
 	TEST_EQUAL(file_priorities.size(), 3);
@@ -499,7 +500,7 @@ TORRENT_TEST(file_priorities2)
 
 TORRENT_TEST(file_priorities3)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	std::vector<int> file_priorities = test_resume_flags(ses, 0, "4321").file_priorities();
 
 	TEST_EQUAL(file_priorities.size(), 3);
@@ -510,7 +511,7 @@ TORRENT_TEST(file_priorities3)
 
 TORRENT_TEST(plain)
 {
-	lt::session ses;
+	lt::session ses(settings());
 
 	torrent_status s = test_resume_flags(ses, 0).status();
 	default_tests(s);
@@ -533,7 +534,7 @@ TORRENT_TEST(plain)
 
 TORRENT_TEST(use_resume_save_path)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_use_resume_save_path).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
@@ -555,7 +556,7 @@ TORRENT_TEST(use_resume_save_path)
 
 TORRENT_TEST(override_resume_data)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_status s = test_resume_flags(ses
 		, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_paused).status();
@@ -580,7 +581,7 @@ TORRENT_TEST(override_resume_data)
 
 TORRENT_TEST(seed_mode)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_seed_mode).status();
 	default_tests(s);
@@ -603,7 +604,7 @@ TORRENT_TEST(seed_mode)
 
 TORRENT_TEST(upload_mode)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_upload_mode).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
@@ -625,7 +626,7 @@ TORRENT_TEST(upload_mode)
 
 TORRENT_TEST(share_mode)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_status s = test_resume_flags(ses
 		, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_share_mode).status();
@@ -649,7 +650,7 @@ TORRENT_TEST(share_mode)
 
 TORRENT_TEST(auto_managed)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	// resume data overrides the auto-managed flag
 	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_auto_managed).status();
 	default_tests(s);
@@ -672,7 +673,7 @@ TORRENT_TEST(auto_managed)
 
 TORRENT_TEST(paused)
 {
-	lt::session ses;
+	lt::session ses(settings());
 	// resume data overrides the paused flag
 	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_paused).status();
 	default_tests(s);
@@ -701,7 +702,7 @@ TORRENT_TEST(url_seed_resume_data)
 {
 	// merge url seeds with resume data
 	fprintf(stderr, "flags: merge_resume_http_seeds\n");
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_handle h = test_resume_flags(ses,
 		add_torrent_params::flag_merge_resume_http_seeds);
 	std::set<std::string> us = h.url_seeds();
@@ -724,7 +725,7 @@ TORRENT_TEST(resume_override_torrent)
 {
 	// resume data overrides the .torrent_file
 	fprintf(stderr, "flags: no merge_resume_http_seed\n");
-	lt::session ses;
+	lt::session ses(settings());
 	torrent_handle h = test_resume_flags(ses,
 		add_torrent_params::flag_merge_resume_trackers);
 	std::set<std::string> us = h.url_seeds();
