@@ -31,7 +31,13 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/session.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/bind.hpp>
+#include <boost/ref.hpp>
+
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #include "test.hpp"
 #include "setup_transfer.hpp"
@@ -40,17 +46,21 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/performance_counters.hpp"
 #include "libtorrent/bdecode.hpp"
 #include "libtorrent/bencode.hpp"
+#include "libtorrent/torrent_info.hpp"
+#include "settings.hpp"
+
+#include <fstream>
 
 using namespace libtorrent;
 namespace lt = libtorrent;
 
 TORRENT_TEST(session)
 {
-	settings_pack p;
+	settings_pack p = settings();
 	p.set_int(settings_pack::alert_mask, ~0);
 	lt::session ses(p);
 
-	settings_pack sett;
+	settings_pack sett = settings();
 	sett.set_int(settings_pack::cache_size, 100);
 	sett.set_int(settings_pack::max_queued_disk_bytes, 1000 * 16 * 1024);
 
@@ -59,7 +69,6 @@ TORRENT_TEST(session)
 	// verify that we get the appropriate performance warning because
 	// we're allowing a larger queue than we have cache.
 
-
 	alert const* a;
 	for (;;)
 	{
@@ -93,6 +102,23 @@ TORRENT_TEST(session)
 	// the session object
 }
 
+TORRENT_TEST(load_empty_file)
+{
+	settings_pack p = settings();
+	p.set_int(settings_pack::alert_mask, ~0);
+	lt::session ses(p);
+
+	add_torrent_params atp;
+	error_code ignore_errors;
+	atp.ti = boost::make_shared<torrent_info>("", 0, boost::ref(ignore_errors));
+	atp.save_path = ".";
+	error_code ec;
+	torrent_handle h = ses.add_torrent(atp, ec);
+
+	TEST_CHECK(!h.is_valid());
+	TEST_CHECK(ec == error_code(errors::no_metadata))
+}
+
 TORRENT_TEST(session_stats)
 {
 	std::vector<stats_metric> stats = session_stats_metrics();
@@ -107,6 +133,27 @@ TORRENT_TEST(session_stats)
 		TEST_EQUAL(stats[i].value_index, i);
 	}
 }
+
+TORRENT_TEST(paused_session)
+{
+	lt::session s(settings());
+	s.pause();
+
+	lt::add_torrent_params ps;
+	std::ofstream file("temporary");
+	ps.ti = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
+	ps.flags = lt::add_torrent_params::flag_paused;
+	ps.save_path = ".";
+
+	torrent_handle h = s.add_torrent(ps);
+
+	test_sleep(2000);
+	h.resume();
+	test_sleep(1000);
+
+	TEST_CHECK(!h.status().paused);
+}
+
 #if __cplusplus >= 201103L
 
 template <typename Set, typename Save, typename Default, typename Load>
@@ -114,14 +161,14 @@ void test_save_restore(Set setup, Save s, Default d, Load l)
 {
 	entry st;
 	{
-		settings_pack p;
+		settings_pack p = settings();
 		setup(p);
 		lt::session ses(p);
 		s(ses, st);
 	}
 
 	{
-		settings_pack p;
+		settings_pack p = settings();
 		d(p);
 		lt::session ses(p);
 		// the loading function takes a bdecode_node, so we have to transform the
