@@ -34,11 +34,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/thread.hpp"
 #include "libtorrent/file.hpp"
+#include "libtorrent/session_status.hpp"
+#include "libtorrent/torrent_info.hpp"
+
+#include "test.hpp"
+#include "test_utils.hpp"
+#include "setup_transfer.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/bind.hpp>
 #include <boost/tuple/tuple.hpp>
 
-#include "test.hpp"
-#include "setup_transfer.hpp"
 #include <fstream>
 #include <iostream>
 #include <boost/asio/connect.hpp>
@@ -47,6 +54,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/asio/ssl/error.hpp> // for asio::error::get_ssl_category()
 #include <boost/asio/ssl.hpp>
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 using namespace libtorrent;
 using boost::tuples::ignore;
 
@@ -60,6 +69,7 @@ struct test_config_t
 	bool use_ssl_ports;
 	bool seed_has_cert;
 	bool downloader_has_cert;
+	bool downloader_has_ssl_listen_port;
 	bool expected_to_complete;
 	int peer_errors;
 	int ssl_disconnects;
@@ -67,25 +77,34 @@ struct test_config_t
 
 test_config_t test_config[] =
 {
-	{"nobody has a cert (connect to regular port)", false, false, false, false, 0, 0},
-	{"nobody has a cert (connect to ssl port)", true, false, false, false, 1, 1},
-	{"seed has a cert, but not downloader (connect to regular port)", false, true, false, false, 0, 0},
-	{"seed has a cert, but not downloader (connect to ssl port)", true, true, false, false, 1, 1},
-	{"downloader has a cert, but not seed (connect to regular port)", false, false, true, false, 0, 0},
-	{"downloader has a cert, but not seed (connect to ssl port)", true, false, true, false, 1, 1},
-	{"both downloader and seed has a cert (connect to regular port)", false, true, true, false, 0, 0},
-	{"both downloader and seed has a cert (connect to ssl port)", true, true, true, true, 0, 0},
+	// name                                                              sslport sd-cert dl-cert dl-port expect peer-error ssl-disconn
+	{"nobody has a cert (connect to regular port)",                      false,  false,  false,  true,   false, 0, 1},
+	{"nobody has a cert (connect to ssl port)",                          true,   false,  false,  true,   false, 1, 1},
+	{"seed has a cert, but not downloader (connect to regular port)",    false,  true,   false,  true,   false, 0, 1},
+	{"seed has a cert, but not downloader (connect to ssl port)",        true,   true,   false,  true,   false, 1, 1},
+	{"downloader has a cert, but not seed (connect to regular port)",    false,  false,  true,   true,   false, 0, 1},
+	{"downloader has a cert, but not seed (connect to ssl port)",        true,   false,  true,   true,   false, 1, 1},
+	{"both downloader and seed has a cert (connect to regular port)",    false,  true,   true,   true,   false, 0, 1},
+	{"both downloader and seed has a cert (connect to ssl port)",        true,   true,   true,   true,   true,  0, 0},
+	// there is a disconnect (or failed connection attempt), that's not a peer
+	// error though, so both counters stay 0
+	{"both downloader and seed has a cert (downloader has no SSL port)", true,   true,   true,   false,  false, 0, 0},
 };
 
 int peer_disconnects = 0;
 int peer_errors = 0;
 int ssl_peer_disconnects = 0;
 
-bool on_alert(alert* a)
+bool on_alert(alert const* a)
 {
-	if (alert_cast<peer_disconnected_alert>(a))
+	if (peer_disconnected_alert const* e = alert_cast<peer_disconnected_alert>(a))
+	{
 		++peer_disconnects;
-	if (peer_error_alert* e = alert_cast<peer_error_alert>(a))
+		if (e->error.category() == boost::asio::error::get_ssl_category())
+			++ssl_peer_disconnects;
+	}
+
+	if (peer_error_alert const* e = alert_cast<peer_error_alert>(a))
 	{
 		++peer_disconnects;
 		++peer_errors;
@@ -113,31 +132,41 @@ void test_ssl(int test_idx, bool use_utp)
 	remove_all("tmp1_ssl", ec);
 	remove_all("tmp2_ssl", ec);
 
-	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48075, 49000), "0.0.0.0", 0, alert_mask);
-	session ses2(fingerprint("LT", 0, 1, 0, 0), std::make_pair(49075, 50000), "0.0.0.0", 0, alert_mask);
+	int ssl_port = 1024 + rand() % 50000;
+	settings_pack sett;
+	sett.set_int(settings_pack::alert_mask, alert_mask);
+	sett.set_int(settings_pack::max_retry_port_bind, 100);
+	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48075");
+	sett.set_bool(settings_pack::enable_incoming_utp, use_utp);
+	sett.set_bool(settings_pack::enable_outgoing_utp, use_utp);
+	sett.set_bool(settings_pack::enable_incoming_tcp, !use_utp);
+	sett.set_bool(settings_pack::enable_outgoing_tcp, !use_utp);
+	sett.set_bool(settings_pack::enable_dht, false);
+	sett.set_bool(settings_pack::enable_lsd, false);
+	sett.set_bool(settings_pack::enable_upnp, false);
+	sett.set_bool(settings_pack::enable_natpmp, false);
+	// if a peer fails once, don't try it again
+	sett.set_int(settings_pack::max_failcount, 1);
+	sett.set_int(settings_pack::ssl_listen, ssl_port);
+
+	libtorrent::session ses1(sett, 0);
+
+	if (test.downloader_has_ssl_listen_port)
+		sett.set_int(settings_pack::ssl_listen, ssl_port + 20);
+	else
+		sett.set_int(settings_pack::ssl_listen, 0);
+
+	libtorrent::session ses2(sett, 0);
 
 	wait_for_listen(ses1, "ses1");
 	wait_for_listen(ses2, "ses2");
 
-	session_settings sett;
-
-	sett.enable_incoming_utp = use_utp;
-	sett.enable_outgoing_utp = use_utp;
-	sett.enable_incoming_tcp = !use_utp;
-	sett.enable_outgoing_tcp = !use_utp;
-
-	sett.ssl_listen = 1024 + rand() % 50000;
-
-	ses1.set_settings(sett);
-	sett.ssl_listen += 10;
-	ses2.set_settings(sett);
-
 	torrent_handle tor1;
 	torrent_handle tor2;
 
 	create_directory("tmp1_ssl", ec);
 	std::ofstream file("tmp1_ssl/temporary");
-	boost::intrusive_ptr<torrent_info> t = ::create_torrent(&file
+	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
 		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
@@ -146,15 +175,12 @@ void test_ssl(int test_idx, bool use_utp)
 	addp.flags &= ~add_torrent_params::flag_paused;
 	addp.flags &= ~add_torrent_params::flag_auto_managed;
 
-	wait_for_listen(ses1, "ses1");
-	wait_for_listen(ses2, "ses2");
-
 	peer_disconnects = 0;
 	ssl_peer_disconnects = 0;
 	peer_errors = 0;
 
 	boost::tie(tor1, tor2, ignore) = setup_transfer(&ses1, &ses2, 0
-		, true, false, true, "_ssl", 16 * 1024, &t, false, &addp, true, test.use_ssl_ports);
+		, true, false, false, "_ssl", 16 * 1024, &t, false, &addp, true);
 
 	if (test.seed_has_cert)
 	{
@@ -174,6 +200,33 @@ void test_ssl(int test_idx, bool use_utp)
 			, "test");
 	}
 
+	// make sure they've taken effect
+	if (test.downloader_has_cert || test.seed_has_cert)
+	{
+		// this will cause a round-trip to the main thread, and make sure the
+		// previous async. calls have completed
+		ses1.listen_port();
+		ses2.listen_port();
+	}
+
+	wait_for_alert(ses1, torrent_finished_alert::alert_type, "ses1");
+	wait_for_downloading(ses2, "ses2");
+
+	// connect the peers after setting the certificates
+	int port = 0;
+	if (test.use_ssl_ports)
+		if (test.downloader_has_ssl_listen_port)
+			port = ses2.ssl_listen_port();
+		else
+			port = 13512;
+	else
+		port = ses2.listen_port();
+
+	fprintf(stderr, "\n\n%s: ses1: connecting peer port: %d\n\n\n"
+		, time_now_string(), port);
+	tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
+		, port));
+
 #ifdef TORRENT_USE_VALGRIND
 	const int timeout = 100;
 #else
@@ -213,7 +266,7 @@ void test_ssl(int test_idx, bool use_utp)
 
 		if (st2.state != torrent_status::downloading)
 		{
-			static char const* state_str[] =	
+			static char const* state_str[] =
 				{"checking (q)", "checking", "dl metadata"
 				, "downloading", "finished", "seeding", "allocating", "checking (r)"};
 			std::cerr << "st2 state: " << state_str[st2.state] << std::endl;
@@ -227,13 +280,16 @@ void test_ssl(int test_idx, bool use_utp)
 		test_sleep(100);
 	}
 
-	fprintf(stderr, "peer_errors: %d  expected: %d\n", peer_errors, test.peer_errors);
-	TEST_EQUAL(peer_errors, test.peer_errors);
+	fprintf(stderr, "peer_errors: %d expected_errors: %d\n"
+		, peer_errors, test.peer_errors);
+	TEST_EQUAL(peer_errors > 0, test.peer_errors > 0);
+
 	fprintf(stderr, "ssl_disconnects: %d  expected: %d\n", ssl_peer_disconnects, test.ssl_disconnects);
-	TEST_EQUAL(ssl_peer_disconnects, test.ssl_disconnects);
+	TEST_EQUAL(ssl_peer_disconnects > 0, test.ssl_disconnects > 0);
+
 	fprintf(stderr, "%s: EXPECT: %s\n", time_now_string(), test.expected_to_complete ? "SUCCEESS" : "FAILURE");
 	fprintf(stderr, "%s: RESULT: %s\n", time_now_string(), tor2.status().is_seeding ? "SUCCEESS" : "FAILURE");
-	TEST_CHECK(tor2.status().is_seeding == test.expected_to_complete);
+	TEST_EQUAL(tor2.status().is_seeding, test.expected_to_complete);
 
 	// this allows shutting down the sessions in parallel
 	p1 = ses1.abort();
@@ -283,8 +339,8 @@ attack_t attacks[] =
 
 const int num_attacks = sizeof(attacks)/sizeof(attacks[0]);
 
-bool try_connect(session& ses1, int port
-	, boost::intrusive_ptr<torrent_info> const& t, boost::uint32_t flags)
+bool try_connect(libtorrent::session& ses1, int port
+	, boost::shared_ptr<torrent_info> const& t, boost::uint32_t flags)
 {
 	using boost::asio::ssl::context;
 
@@ -300,7 +356,7 @@ bool try_connect(session& ses1, int port
 	if (flags & valid_bittorrent_hash) fprintf(stderr, "valid-bittorrent-hash ");
 	else fprintf(stderr, "invalid-bittorrent-hash ");
 
-	fprintf(stderr, "\n");
+	fprintf(stderr, " port: %d\n", port);
 
 	error_code ec;
 	boost::asio::io_service ios;
@@ -377,7 +433,7 @@ bool try_connect(session& ses1, int port
 		}
 	}
 
-	boost::asio::ssl::stream<boost::asio::ip::tcp::socket> ssl_sock(ios, ctx);
+	boost::asio::ssl::stream<tcp::socket> ssl_sock(ios, ctx);
 
 	fprintf(stderr, "connecting 127.0.0.1:%d\n", port);
 	ssl_sock.lowest_layer().connect(tcp::endpoint(
@@ -405,13 +461,13 @@ bool try_connect(session& ses1, int port
 		name.reserve(40);
 		for (int i = 0; i < 40; ++i)
 			name += hex_alphabet[rand() % 16];
-		
+
 		fprintf(stderr, "SNI: %s\n", name.c_str());
 		SSL_set_tlsext_host_name(ssl_sock.native_handle(), name.c_str());
 	}
 
 	fprintf(stderr, "SSL handshake\n");
-	ssl_sock.handshake(asio::ssl::stream_base::client, ec);
+	ssl_sock.handshake(boost::asio::ssl::stream_base::client, ec);
 
 	print_alerts(ses1, "ses1", true, true, true, &on_alert);
 	if (ec)
@@ -425,7 +481,7 @@ bool try_connect(session& ses1, int port
 		"                    " // space for info-hash
 		"aaaaaaaaaaaaaaaaaaaa" // peer-id
 		"\0\0\0\x01\x02"; // interested
-	
+
 	// fill in the info-hash
 	if (flags & valid_bittorrent_hash)
 	{
@@ -442,17 +498,19 @@ bool try_connect(session& ses1, int port
 	std::generate(handshake + 48, handshake + 68, &rand);
 
 	fprintf(stderr, "bittorrent handshake\n");
-	boost::asio::write(ssl_sock, libtorrent::asio::buffer(handshake, (sizeof(handshake) - 1)), ec);
+	boost::asio::write(ssl_sock, boost::asio::buffer(handshake, (sizeof(handshake) - 1)), ec);
+	print_alerts(ses1, "ses1", true, true, true, &on_alert);
 	if (ec)
 	{
 		fprintf(stderr, "failed to write bittorrent handshake: %s\n"
 			, ec.message().c_str());
 		return false;
 	}
-	
+
 	char buf[68];
 	fprintf(stderr, "read bittorrent handshake\n");
-	boost::asio::read(ssl_sock, libtorrent::asio::buffer(buf, sizeof(buf)), ec);
+	boost::asio::read(ssl_sock, boost::asio::buffer(buf, sizeof(buf)), ec);
+	print_alerts(ses1, "ses1", true, true, true, &on_alert);
 	if (ec)
 	{
 		fprintf(stderr, "failed to read bittorrent handshake: %s\n"
@@ -483,17 +541,24 @@ void test_malicious_peer()
 	remove_all("tmp3_ssl", ec);
 
 	// set up session
-	session ses1(fingerprint("LT", 0, 1, 0, 0)
-		, std::make_pair(48075, 49000), "0.0.0.0", 0, alert_mask);
+	int ssl_port = 1024 + rand() % 50000;
+	settings_pack sett;
+	sett.set_int(settings_pack::alert_mask, alert_mask);
+	sett.set_int(settings_pack::max_retry_port_bind, 100);
+	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48075");
+	sett.set_int(settings_pack::ssl_listen, ssl_port);
+	sett.set_bool(settings_pack::enable_dht, false);
+	sett.set_bool(settings_pack::enable_lsd, false);
+	sett.set_bool(settings_pack::enable_upnp, false);
+	sett.set_bool(settings_pack::enable_natpmp, false);
+
+	libtorrent::session ses1(sett, 0);
 	wait_for_listen(ses1, "ses1");
-	session_settings sett;
-	sett.ssl_listen = 1024 + rand() % 50000;
-	ses1.set_settings(sett);
 
 	// create torrent
 	create_directory("tmp3_ssl", ec);
 	std::ofstream file("tmp3_ssl/temporary");
-	boost::intrusive_ptr<torrent_info> t = ::create_torrent(&file
+	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary"
 		, 16 * 1024, 13, false, combine_path("..", combine_path("ssl", "root_ca_cert.pem")));
 	file.close();
 
@@ -513,35 +578,48 @@ void test_malicious_peer()
 		, combine_path("..", combine_path("ssl", "dhparams.pem"))
 		, "test");
 
-	wait_for_listen(ses1, "ses1");
-	wait_for_alert(ses1, torrent_finished_alert::alert_type);
+	alert const* a = wait_for_alert(ses1
+		, torrent_finished_alert::alert_type, "ses1");
+	TEST_CHECK(a);
+	if (a)
+	{
+		TEST_EQUAL(a->type(), torrent_finished_alert::alert_type);
+	}
 
 	for (int i = 0; i < num_attacks; ++i)
 	{
-		bool success = try_connect(ses1, sett.ssl_listen, t, attacks[i].flags);
-		TEST_EQUAL(attacks[i].expect, success);
+		bool success = try_connect(ses1, ssl_port, t, attacks[i].flags);
+		TEST_EQUAL(success, attacks[i].expect);
 	}
 }
 #endif // TORRENT_USE_OPENSSL
 
-int test_main()
+TORRENT_TEST(malicious_peer)
 {
-	using namespace libtorrent;
-
 #ifdef TORRENT_USE_OPENSSL
 	test_malicious_peer();
-
-	// No support for SSL/uTP yet, so always pass in false
-	for (int i = 0; i < sizeof(test_config)/sizeof(test_config[0]); ++i)
-		test_ssl(i, false);
-	
-	error_code ec;
-	remove_all("tmp1_ssl", ec);
-	remove_all("tmp2_ssl", ec);
-
-#endif // TORRENT_USE_OPENSSL
-	return 0;
+#endif
 }
 
+#ifdef TORRENT_USE_OPENSSL
+TORRENT_TEST(utp_config0) { test_ssl(0, true); }
+TORRENT_TEST(utp_config1) { test_ssl(1, true); }
+TORRENT_TEST(utp_config2) { test_ssl(2, true); }
+TORRENT_TEST(utp_config3) { test_ssl(3, true); }
+TORRENT_TEST(utp_config4) { test_ssl(4, true); }
+TORRENT_TEST(utp_config5) { test_ssl(5, true); }
+TORRENT_TEST(utp_config6) { test_ssl(6, true); }
+TORRENT_TEST(utp_config7) { test_ssl(7, true); }
+TORRENT_TEST(utp_config8) { test_ssl(8, true); }
 
+TORRENT_TEST(tcp_config0) { test_ssl(0, false); }
+TORRENT_TEST(tcp_config1) { test_ssl(1, false); }
+TORRENT_TEST(tcp_config2) { test_ssl(2, false); }
+TORRENT_TEST(tcp_config3) { test_ssl(3, false); }
+TORRENT_TEST(tcp_config4) { test_ssl(4, false); }
+TORRENT_TEST(tcp_config5) { test_ssl(5, false); }
+TORRENT_TEST(tcp_config6) { test_ssl(6, false); }
+TORRENT_TEST(tcp_config7) { test_ssl(7, false); }
+TORRENT_TEST(tcp_config8) { test_ssl(8, false); }
+#endif
 
