@@ -38,6 +38,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/thread.hpp"
 #include "libtorrent/torrent.hpp"
+#include "libtorrent/peer_info.hpp"
+#include "libtorrent/extensions.hpp"
+#include "settings.hpp"
 #include <boost/tuple/tuple.hpp>
 #include <boost/make_shared.hpp>
 #include <iostream>
@@ -50,7 +53,7 @@ namespace lt = libtorrent;
 
 void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t file_size)
 {
-	settings_pack pack;
+	settings_pack pack = settings();
 	pack.set_int(settings_pack::alert_mask, alert::storage_notification);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
@@ -82,7 +85,7 @@ void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t f
 //	test_sleep(500);
 	torrent_status st = h.status();
 
-	TEST_EQUAL(st.total_wanted, file_size * 3);
+	TEST_EQUAL(st.total_wanted, file_size); // we want the single file
 	TEST_EQUAL(st.total_wanted_done, 0);
 
 	std::vector<int> prio(info->num_files(), 1);
@@ -90,7 +93,7 @@ void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t f
 	h.prioritize_files(prio);
 	st = h.status();
 
-	TEST_EQUAL(st.total_wanted, file_size * 2);
+	TEST_EQUAL(st.total_wanted, 0); // we don't want anything
 	TEST_EQUAL(st.total_wanted_done, 0);
 	TEST_EQUAL(int(h.file_priorities().size()), info->num_files());
 	if (!st.is_seeding)
@@ -129,13 +132,12 @@ void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t f
 		std::vector<char> piece(info->piece_length());
 		for (int i = 0; i < int(piece.size()); ++i)
 			piece[i] = (i % 26) + 'A';
-		h.add_piece(0, &piece[0]);
+		h.add_piece(0, &piece[0], torrent_handle::overwrite_existing);
 
 		// wait until the piece is done writing and hashing
-		// TODO: wait for an alert rather than just waiting 10 seconds. This is kind of silly
-		test_sleep(2000);
+		wait_for_alert(ses, piece_finished_alert::alert_type, "piece_finished_alert");
 		st = h.status();
-		TEST_CHECK(st.pieces.size() > 0 && st.pieces[0] == true);
+		TEST_CHECK(st.pieces.size() > 0);
 
 		std::cout << "reading piece 0" << std::endl;
 		h.read_piece(0);
@@ -146,7 +148,7 @@ void test_running_torrent(boost::shared_ptr<torrent_info> info, boost::int64_t f
 		if (rpa)
 		{
 			std::cout << "SUCCEEDED!" << std::endl;
-			TEST_CHECK(memcmp(&piece[0], rpa->buffer.get(), piece.size()) == 0);
+			TEST_CHECK(memcmp(&piece[0], rpa->buffer.get(), info->piece_size(0)) == 0);
 			TEST_CHECK(rpa->size == info->piece_size(0));
 			TEST_CHECK(rpa->piece == 0);
 			TEST_CHECK(hasher(&piece[0], piece.size()).final() == info->hash_for_piece(0));
@@ -188,7 +190,7 @@ TORRENT_TEST(total_wanted)
 	boost::shared_ptr<torrent_info> info(boost::make_shared<torrent_info>(
 		&tmp[0], tmp.size(), boost::ref(ec)));
 
-	settings_pack pack;
+	settings_pack pack = settings();
 	pack.set_int(settings_pack::alert_mask, alert::storage_notification);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
@@ -213,6 +215,36 @@ TORRENT_TEST(total_wanted)
 	TEST_EQUAL(st.total_wanted_done, 0);
 }
 
+TORRENT_TEST(added_peers)
+{
+	file_storage fs;
+
+	fs.add_file("test_torrent_dir4/tmp1", 1024);
+
+	libtorrent::create_torrent t(fs, 1024);
+	std::vector<char> tmp;
+	bencode(std::back_inserter(tmp), t.generate());
+	error_code ec;
+	boost::shared_ptr<torrent_info> info(boost::make_shared<torrent_info>(
+		&tmp[0], tmp.size(), boost::ref(ec)));
+
+	settings_pack pack = settings();
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
+	pack.set_int(settings_pack::max_retry_port_bind, 10);
+	lt::session ses(pack);
+
+	add_torrent_params p;
+	p.ti = info;
+	p.save_path = ".";
+	p.url = "?x.pe=127.0.0.1:48081&x.pe=127.0.0.2:48082";
+
+	torrent_handle h = ses.add_torrent(p);
+
+	std::vector<peer_list_entry> v;
+	h.get_full_peer_list(v);
+	TEST_EQUAL(v.size(), 2);
+}
+
 TORRENT_TEST(torrent)
 {
 /*	{
@@ -251,17 +283,113 @@ TORRENT_TEST(torrent)
 	{
 		file_storage fs;
 
-		fs.add_file("test_torrent_dir2/tmp1", 0);
+		fs.add_file("test_torrent_dir2/tmp1", 1024);
 		libtorrent::create_torrent t(fs, 128 * 1024, 6);
 
+		std::vector<char> piece(128 * 1024);
+		for (int i = 0; i < int(piece.size()); ++i)
+			piece[i] = (i % 26) + 'A';
+
+		// calculate the hash for all pieces
+		sha1_hash ph = hasher(&piece[0], piece.size()).final();
+		int num = t.num_pieces();
+		TEST_CHECK(t.num_pieces() > 0);
+		for (int i = 0; i < num; ++i)
+			t.set_hash(i, ph);
+
 		std::vector<char> tmp;
 		std::back_insert_iterator<std::vector<char> > out(tmp);
 		bencode(out, t.generate());
 		error_code ec;
 		boost::shared_ptr<torrent_info> info(boost::make_shared<torrent_info>(&tmp[0], tmp.size(), boost::ref(ec), 0));
-		test_running_torrent(info, 0);
+		test_running_torrent(info, 1024);
+	}
+}
+
+#ifndef TORRENT_DISABLE_EXTENSIONS
+struct test_plugin : libtorrent::torrent_plugin {};
+
+struct plugin_creator
+{
+	plugin_creator(int& c) : m_called(c) {}
+
+	boost::shared_ptr<libtorrent::torrent_plugin>
+	operator()(torrent_handle const&, void*)
+	{
+		++m_called;
+		return boost::make_shared<test_plugin>();
 	}
 
+	int& m_called;
+};
+
+TORRENT_TEST(duplicate_is_not_error)
+{
+	file_storage fs;
+
+	fs.add_file("test_torrent_dir2/tmp1", 1024);
+	libtorrent::create_torrent t(fs, 128 * 1024, 6);
+
+	std::vector<char> piece(128 * 1024);
+	for (int i = 0; i < int(piece.size()); ++i)
+		piece[i] = (i % 26) + 'A';
+
+	// calculate the hash for all pieces
+	sha1_hash ph = hasher(&piece[0], piece.size()).final();
+	int num = t.num_pieces();
+	TEST_CHECK(t.num_pieces() > 0);
+	for (int i = 0; i < num; ++i)
+		t.set_hash(i, ph);
+
+	std::vector<char> tmp;
+	std::back_insert_iterator<std::vector<char> > out(tmp);
+	bencode(out, t.generate());
+	error_code ec;
+
+	int called = 0;
+	plugin_creator creator(called);
+
+	add_torrent_params p;
+	p.ti = boost::make_shared<torrent_info>(&tmp[0], tmp.size(), boost::ref(ec), 0);
+	p.flags &= ~add_torrent_params::flag_paused;
+	p.flags &= ~add_torrent_params::flag_auto_managed;
+	p.flags &= ~add_torrent_params::flag_duplicate_is_error;
+	p.save_path = ".";
+	p.extensions.push_back(creator);
+
+	lt::session ses(settings());
+	ses.async_add_torrent(p);
+	ses.async_add_torrent(p);
+
+	wait_for_downloading(ses, "ses");
+
+	// we should only have added the plugin once
+	TEST_EQUAL(called, 1);
+}
+#endif
+
+TORRENT_TEST(torrent_total_size_zero)
+{
+	file_storage fs;
+	error_code ec;
+
+	fs.add_file("test_torrent_dir2/tmp1", 0);
+	TEST_CHECK(fs.num_files() == 1);
+	TEST_CHECK(fs.total_size() == 0);
+
+	ec.clear();
+	libtorrent::create_torrent t1(fs);
+	set_piece_hashes(t1, ".", ec);
+	TEST_CHECK(ec);
+
+	fs.add_file("test_torrent_dir2/tmp2", 0);
+	TEST_CHECK(fs.num_files() == 2);
+	TEST_CHECK(fs.total_size() == 0);
+
+	ec.clear();
+	libtorrent::create_torrent t2(fs);
+	set_piece_hashes(t2, ".", ec);
+	TEST_CHECK(ec);
 }
 
 
