@@ -36,7 +36,13 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/announce_entry.hpp"
 #include "libtorrent/aux_/escape_string.hpp" // for convert_path_to_posix
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #include <boost/make_shared.hpp>
+#include <boost/ref.hpp>
+
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #if TORRENT_USE_IOSTREAM
 #include <sstream>
@@ -125,6 +131,9 @@ static test_torrent_t test_torrents[] =
 	{ "invalid_name2.torrent" },
 	{ "invalid_name3.torrent" },
 	{ "symlink1.torrent" },
+	{ "unordered.torrent" },
+	{ "symlink_zero_size.torrent" },
+	{ "pad_file_no_path.torrent" },
 };
 
 struct test_failing_torrent_t
@@ -144,13 +153,14 @@ test_failing_torrent_t test_error_torrents[] =
 	{ "string.torrent", errors::torrent_is_no_dict },
 	{ "negative_size.torrent", errors::torrent_invalid_length },
 	{ "negative_file_size.torrent", errors::torrent_invalid_length },
-	{ "invalid_path_list.torrent", errors::torrent_missing_name},
+	{ "invalid_path_list.torrent", errors::torrent_invalid_name},
 	{ "missing_path_list.torrent", errors::torrent_missing_name },
 	{ "invalid_pieces.torrent", errors::torrent_missing_pieces },
 	{ "unaligned_pieces.torrent", errors::torrent_invalid_hashes },
 	{ "invalid_root_hash.torrent", errors::torrent_invalid_hashes },
 	{ "invalid_root_hash2.torrent", errors::torrent_missing_pieces },
 	{ "invalid_file_size.torrent", errors::torrent_invalid_length },
+	{ "invalid_symlink.torrent", errors::torrent_invalid_name },
 };
 
 // TODO: test remap_files
@@ -161,7 +171,6 @@ test_failing_torrent_t test_error_torrents[] =
 // TODO: torrent with 'l' (symlink) attribute
 // TODO: creating a merkle torrent (torrent_info::build_merkle_list)
 // TODO: torrent with multiple trackers in multiple tiers, making sure we shuffle them (how do you test shuffling?, load it multiple times and make sure it's in different order at least once)
-// TODO: torrents with a missing name
 // TODO: torrents with a zero-length name
 // TODO: torrents with a merkle tree and add_merkle_nodes
 // TODO: torrent with a non-dictionary info-section
@@ -711,6 +720,16 @@ TORRENT_TEST(parse_torrents)
 			TEST_EQUAL(ti->num_files(), 1);
 			TEST_EQUAL(ti->files().file_path(0), "temp....abc");
 		}
+		else if (std::string(test_torrents[i].file) == "symlink_zero_size.torrent")
+		{
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().symlink(1), combine_path("foo", "bar"));
+		}
+		else if (std::string(test_torrents[i].file) == "pad_file_no_path.torrent")
+		{
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().file_path(1), combine_path(".pad", "0"));
+		}
 
 		file_storage const& fs = ti->files();
 		for (int i = 0; i < fs.num_files(); ++i)
@@ -854,6 +873,26 @@ TORRENT_TEST(resolve_duplicates)
 		test_resolve_duplicates(i);
 }
 
+TORRENT_TEST(empty_file)
+{
+	error_code ec;
+	boost::shared_ptr<torrent_info> ti = boost::make_shared<torrent_info>("", 0, boost::ref(ec));
+	TEST_CHECK(ec);
+}
+
+TORRENT_TEST(empty_file2)
+{
+	try
+	{
+		boost::shared_ptr<torrent_info> ti = boost::make_shared<torrent_info>("", 0);
+		TEST_ERROR("expected exception thrown");
+	}
+	catch (libtorrent_exception& e)
+	{
+		printf("Expected error: %s\n", e.error().message().c_str());
+	}
+}
+
 TORRENT_TEST(copy)
 {
 	using namespace libtorrent;
